// Implementation of main functions

#include "dic.h"

////////////////////////// Functions to do the lookup //////////////////////////


void initReadBuffer(HANDLE hfile)
{	
	g_readBuffer = new WCHAR[READ_BUFFERSIZE];
	g_readIndex = 0;

	// Strip UTF-16 header
	WCHAR header;
	unsigned long numRead;

	ReadFile(hfile, &header, sizeof(WCHAR), &numRead, NULL);
	
	if (header != 0xFEFF)
		assert(false);


	readBuffer(hfile, g_readBuffer, READ_BUFFERSIZE);
}

bool readBuffer(HANDLE hfile, LPWSTR buffer, int bufferSize)
{
	assert (hfile != INVALID_HANDLE_VALUE);

	unsigned long numRead;
	ReadFile(hfile, buffer, sizeof(WCHAR) * (bufferSize - 1), &numRead, NULL);

	if (numRead == 0) // EOF
		return false;
	return true;
}

void closeReadBuffer()
{
	delete[] g_readBuffer;
}

WCHAR readWChar(HANDLE hfile)
{
	assert (hfile != INVALID_HANDLE_VALUE);

	WCHAR temp;

	unsigned long numRead;
	ReadFile(hfile, &temp, sizeof(temp), &numRead, NULL);

	if (numRead == 0) // EOF
		return L'\0';
	return temp;
}

// Fill string with WCHARS from the file hfile until stop is reached 
// Returns true if the stop char was found.
bool getString(wstring* str, HANDLE hfile, WCHAR stop)
{
	assert (hfile != INVALID_HANDLE_VALUE);

	while (g_readBuffer[g_readIndex] != stop)
	{
		str->push_back(g_readBuffer[g_readIndex]);
		++g_readIndex;

		if (g_readIndex >= READ_BUFFERSIZE)  // If buffer is empty
		{
			if (!readBuffer(hfile, g_readBuffer, READ_BUFFERSIZE))
				return false;
			else
				g_readIndex = 0;
		}
	}
	return true;
}

bool clearSymbol(HANDLE hfile, WCHAR sym)
{
	assert (hfile != INVALID_HANDLE_VALUE);

	while (g_readBuffer[g_readIndex] == sym)
	{
		++g_readIndex;

		if (g_readIndex >= READ_BUFFERSIZE)
		{
			if (!readBuffer(hfile, g_readBuffer, READ_BUFFERSIZE))
				return false;
			else
				g_readIndex = 0;
		}
	}
	return true;
}

word readWord(HANDLE hfile)
{
	wstring kanji;
	wstring kana;
	wstring engDef;

	if (!getString(&kanji, hfile, UNI_KANJI_STOP))
		return word();


	if (!clearSymbol(hfile, UNI_SPACE))
		return word();

	if (g_readBuffer[g_readIndex] == UNI_KANA_START)
	{
		++g_readIndex;
		if (!getString(&kana, hfile, UNI_KANA_STOP))
			return word();
	
		if (!clearSymbol(hfile, UNI_KANA_STOP))
			return word();

		if (!clearSymbol(hfile, UNI_SPACE))
			return word();
	}
	else
		kana[0] = L'\0';

	if (!getString(&engDef, hfile, UNI_ENGDEF_STOP))
		return word();

	++g_readIndex; // Zap the stupid CR
	return word(kanji, kana, engDef);
}

bool loadDict(LPWSTR dictpath, Dictionary* dict)
{
	HANDLE hfile;

	hfile = CreateFile(dictpath,
						GENERIC_READ,
						FILE_SHARE_READ,
						NULL,
						OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL,
						NULL);

	if (hfile == INVALID_HANDLE_VALUE)
	{
		cout << "Invalid file name" <<endl;
		return false;
	}

	initReadBuffer(hfile);

	word nextWord = readWord(hfile);

	while (nextWord.exists())
	{
		dict->insert(nextWord);
		nextWord = readWord(hfile);
	}

	closeReadBuffer();
	CloseHandle(hfile);
	return true;
}


// These could end up reading arbitrary memory
wstring tokenizeWhileInside(LPCWSTR srcBuffer,
							int* currentPos,
							WCHAR lower,
							WCHAR upper)
{

	wstring token;

	while (srcBuffer[*currentPos] > lower 
		   &&
		   srcBuffer[*currentPos] < upper)
	{
		token.push_back(srcBuffer[*currentPos]);
		++*currentPos;

		if (srcBuffer[*currentPos] == L'\0')
			return token;
	}
	return token;
}

wstring tokenizeWhileOutside(LPCWSTR srcBuffer, 
							 int* currentPos,
							 WCHAR lower,
							 WCHAR upper)
{
	wstring token;

	while (srcBuffer[*currentPos] < lower
		   ||
		   srcBuffer[*currentPos] > upper)
	{
		token.push_back(srcBuffer[*currentPos]);
		++*currentPos;

		if (srcBuffer[*currentPos] == L'\0')
			return token;
	}
	return token;
}

HWND* createTransWindow(HINSTANCE hInst)
{
	HWND* hwnd = new HWND;

	*hwnd  = CreateWindow(WC_EDIT,   
		   			      L"Translation", 
						  WS_OVERLAPPEDWINDOW | WS_VISIBLE | WS_VSCROLL | 
                                    ES_LEFT | ES_MULTILINE | ES_AUTOVSCROLL 
									| ES_READONLY,
						  CW_USEDEFAULT,
						  CW_USEDEFAULT, 
						  TRANS_WIN_WIDTH, 
						  TRANS_WIN_HEIGHT, 
						  NULL,    
						  NULL,    
						  hInst, 
						  NULL);    

	if (!(*hwnd))
	{
		DWORD error = GetLastError();
		return NULL;
	}

	ShowWindow(*hwnd, SW_SHOWDEFAULT); 
	UpdateWindow(*hwnd); 
	return hwnd;
}

void getFilename(LPWSTR buffer, int bufferSize)
{
	OPENFILENAME ofn;
	
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = NULL;
	ofn.lpstrFile = buffer;

	ofn.lpstrFile[0] = L'\0';
	ofn.nMaxFile = bufferSize;
	ofn.lpstrFilter = L"Dictionary\0*.*\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	GetOpenFileName(&ofn);
}

/////////////////////////// DLL Functions ////////////////////////////////////////


OTHEXPORT int OTHCALL OthTransInit(OTHTR *pTR)
{
	InitCommonControls();
	GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);


	
	pTR->pSetting = createTransWindow(hInst);

	orgStrLen = INITIAL_ORG_STR_LEN;
	orgStr = new WCHAR[INITIAL_ORG_STR_LEN];


	WCHAR path[2001]; 
    GetModuleFileName(hInst, path, 2001); 
    int len = lstrlen(path) - 1; 
    if (len > 2000)  
		assert(false); 
    for (; len > 0; --len) 
		if (path[len] == L'\\')		// Strip .dll name
			break; 

	path[len] = L'\0'; 
    lstrcat(path, L"\\jedic\\edictUnicode");

	loadDict(path, &dict);


	g_font = CreateFont(0,
						0,
						0,
						0,
						0,
						false,
						false,
						false,
						SHIFTJIS_CHARSET,
						OUT_DEFAULT_PRECIS,
						CLIP_DEFAULT_PRECIS,
						DEFAULT_QUALITY,
						FF_DONTCARE|DEFAULT_PITCH,
						NULL);
	SendMessage(*(HWND*)(pTR->pSetting), WM_SETFONT, (WPARAM) g_font, true);


	if (pTR->pSetting == NULL)
		assert(false);

	return 0;
}


OTHEXPORT int OTHCALL OthTransSetting(OTHTR *pTR)
{
  return 0;
}


OTHEXPORT int OTHCALL OthTransCopySetting(OTHTR *pDestTR, OTHTR *pSrcTR)
{

  return 0;
}


OTHEXPORT int OTHCALL OthTranslate(OTHTR *pTR, LPCWSTR lpStr, int iLen)
{
	if (iLen > 1)
	{
		int x = 1 + 1;
	}

	if (iLen > orgStrLen)
	{
		delete[] orgStr;
		orgStr = new WCHAR[2*orgStrLen];
	}

	wcsncpy(orgStr, lpStr, iLen + 1);

	int index = 0;
	
	wstring value;


	while (pTR->lpSelStr[index] != L'\0')
	{
		wstring token = tokenizeWhileOutside(pTR->lpSelStr, 
											 &index,
											 FIRST_KANJI,
											 LAST_KANJI);

		wstring kanji = tokenizeWhileInside(pTR->lpSelStr, 
											&index,
											FIRST_KANJI, 
											LAST_KANJI);
		
		value.append(kanji);
		value.push_back(L' ');
		value.append(dict.lookUpKana(kanji));
		value.push_back(L' ');
        value.append(dict.lookUpEngDef(kanji));
		value.push_back(L'\n');
	}

	SendMessage(*(HWND*)(pTR->pSetting), WM_SETTEXT, 0, (LPARAM) value.c_str());

	pTR->lpStr = orgStr;
	pTR->iLen = iLen;
	return 0;
}


OTHEXPORT int OTHCALL OthTransDestroy(OTHTR *pTR)
{
	delete pTR->pSetting;
	delete[] orgStr;
	DeleteObject(g_font);
	GdiplusShutdown(gdiplusToken);

  return 0;
}


BOOL APIENTRY DllMain( HINSTANCE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
  if (ul_reason_for_call == DLL_PROCESS_ATTACH)
    hInst = hModule;

    return TRUE;
}

