/*

Taeho Oh ( ohhara@postech.edu, ohhara@plus.or.kr )    http://ohhara.sarang.net
Postech ( Pohang University of Science and Technology ) http://www.postech.edu
PLUS ( Postech Laboratory for Unix Security )            http://www.plus.or.kr

 */

#include <windows.h>
#include <malloc.h>

#define OTHEXPORT __declspec(dllexport)

#include "oh_trmod.h"
#include "resource.h"

#define HIRAGANA_BEGIN  ((WCHAR)0x3040)
#define HIRAGANA_END    ((WCHAR)0x309f)
#define KATAKANA_BEGIN  ((WCHAR)0x30a0)
#define KATAKANA_END    ((WCHAR)0x30ff)

#define GET_JPKRSET(x) ((JPKRSETTING*)(x)->pSetting)

static HINSTANCE hInst;

static const WCHAR jpkrtable[] =
  {
    (WCHAR)0x3040,
    (WCHAR)0x314f,
    (WCHAR)0xc544,
    (WCHAR)0x3163,
    (WCHAR)0xc774,
    (WCHAR)0x315c,
    (WCHAR)0xc6b0,
    (WCHAR)0x3154,
    (WCHAR)0xc5d0,
    (WCHAR)0x3157,
    (WCHAR)0xc624,
    (WCHAR)0xce74,
    (WCHAR)0xac00,
    (WCHAR)0xd0a4,
    (WCHAR)0xae30,
    (WCHAR)0xcfe0,
    (WCHAR)0xad6c,
    (WCHAR)0xcf00,
    (WCHAR)0xac8c,
    (WCHAR)0xcf54,
    (WCHAR)0xace0,
    (WCHAR)0xc0ac,
    (WCHAR)0xc790,
    (WCHAR)0xc2dc,
    (WCHAR)0xc9c0,
    (WCHAR)0xc218,
    (WCHAR)0xc8fc,
    (WCHAR)0xc138,
    (WCHAR)0xc81c,
    (WCHAR)0xc18c,
    (WCHAR)0xc870,
    (WCHAR)0xd0c0,
    (WCHAR)0xb2e4,
    (WCHAR)0xce58,
    (WCHAR)0xc9c0,
    (WCHAR)0x3145,
    (WCHAR)0xce20,
    (WCHAR)0xc988,
    (WCHAR)0xd14c,
    (WCHAR)0xb370,
    (WCHAR)0xd1a0,
    (WCHAR)0xb3c4,
    (WCHAR)0xb098,
    (WCHAR)0xb2c8,
    (WCHAR)0xb204,
    (WCHAR)0xb124,
    (WCHAR)0xb178,
    (WCHAR)0xd558,
    (WCHAR)0xbc14,
    (WCHAR)0xd30c,
    (WCHAR)0xd788,
    (WCHAR)0xbe44,
    (WCHAR)0xd53c,
    (WCHAR)0xd6c4,
    (WCHAR)0xbd80,
    (WCHAR)0xd478,
    (WCHAR)0xd5e4,
    (WCHAR)0xbca0,
    (WCHAR)0xd398,
    (WCHAR)0xd638,
    (WCHAR)0xbcf4,
    (WCHAR)0xd3ec,
    (WCHAR)0xb9c8,
    (WCHAR)0xbbf8,
    (WCHAR)0xbb34,
    (WCHAR)0xba54,
    (WCHAR)0xbaa8,
    (WCHAR)0x3151,
    (WCHAR)0xc57c,
    (WCHAR)0x3160,
    (WCHAR)0xc720,
    (WCHAR)0x315b,
    (WCHAR)0xc694,
    (WCHAR)0xb77c,
    (WCHAR)0xb9ac,
    (WCHAR)0xb8e8,
    (WCHAR)0xb808,
    (WCHAR)0xb85c,
    (WCHAR)0x3158,
    (WCHAR)0xc640,
    (WCHAR)0xc704,
    (WCHAR)0xc6e8,
    (WCHAR)0xc624,
    (WCHAR)0x3134,
    (WCHAR)0x3142,
    (WCHAR)0x3095,
    (WCHAR)0x3096,
    (WCHAR)0x3097,
    (WCHAR)0x3098,
    (WCHAR)0x3099,
    (WCHAR)0x309a,
    (WCHAR)0x309b,
    (WCHAR)0x309c,
    (WCHAR)0x309d,
    (WCHAR)0x309e,
    (WCHAR)0x309f,
    (WCHAR)0x30a0,
    (WCHAR)0x314f,
    (WCHAR)0xc544,
    (WCHAR)0x3163,
    (WCHAR)0xc774,
    (WCHAR)0x315c,
    (WCHAR)0xc6b0,
    (WCHAR)0x3154,
    (WCHAR)0xc5d0,
    (WCHAR)0x3157,
    (WCHAR)0xc624,
    (WCHAR)0xce74,
    (WCHAR)0xac00,
    (WCHAR)0xd0a4,
    (WCHAR)0xae30,
    (WCHAR)0xcfe0,
    (WCHAR)0xad6c,
    (WCHAR)0xcf00,
    (WCHAR)0xac8c,
    (WCHAR)0xcf54,
    (WCHAR)0xace0,
    (WCHAR)0xc0ac,
    (WCHAR)0xc790,
    (WCHAR)0xc2dc,
    (WCHAR)0xc9c0,
    (WCHAR)0xc218,
    (WCHAR)0xc8fc,
    (WCHAR)0xc138,
    (WCHAR)0xc81c,
    (WCHAR)0xc18c,
    (WCHAR)0xc870,
    (WCHAR)0xd0c0,
    (WCHAR)0xb2e4,
    (WCHAR)0xce58,
    (WCHAR)0xc9c0,
    (WCHAR)0x3145,
    (WCHAR)0xce20,
    (WCHAR)0xc988,
    (WCHAR)0xd14c,
    (WCHAR)0xb370,
    (WCHAR)0xd1a0,
    (WCHAR)0xb3c4,
    (WCHAR)0xb098,
    (WCHAR)0xb2c8,
    (WCHAR)0xb204,
    (WCHAR)0xb124,
    (WCHAR)0xb178,
    (WCHAR)0xd558,
    (WCHAR)0xbc14,
    (WCHAR)0xd30c,
    (WCHAR)0xd788,
    (WCHAR)0xbe44,
    (WCHAR)0xd53c,
    (WCHAR)0xd6c4,
    (WCHAR)0xbd80,
    (WCHAR)0xd478,
    (WCHAR)0xd5e4,
    (WCHAR)0xbca0,
    (WCHAR)0xd398,
    (WCHAR)0xd638,
    (WCHAR)0xbcf4,
    (WCHAR)0xd3ec,
    (WCHAR)0xb9c8,
    (WCHAR)0xbbf8,
    (WCHAR)0xbb34,
    (WCHAR)0xba54,
    (WCHAR)0xbaa8,
    (WCHAR)0x3151,
    (WCHAR)0xc57c,
    (WCHAR)0x3160,
    (WCHAR)0xc720,
    (WCHAR)0x315b,
    (WCHAR)0xc694,
    (WCHAR)0xb77c,
    (WCHAR)0xb9ac,
    (WCHAR)0xb8e8,
    (WCHAR)0xb808,
    (WCHAR)0xb85c,
    (WCHAR)0x3158,
    (WCHAR)0xc640,
    (WCHAR)0xc704,
    (WCHAR)0xc6e8,
    (WCHAR)0xc624,
    (WCHAR)0x3134,
    (WCHAR)0x3142,
    (WCHAR)0xce74,
    (WCHAR)0xcf00,
    (WCHAR)0xbc14,
    (WCHAR)0xbe44,
    (WCHAR)0xbca0,
    (WCHAR)0xbcf4,
    (WCHAR)0x30fb,
    (WCHAR)0x30fc,
    (WCHAR)0x30fd,
    (WCHAR)0x30fe,
    (WCHAR)0x30ff,
    (WCHAR)0x30ff
  };

static LPWSTR jpkrbuf;
static int jpkrlen;

typedef struct JPKRSETTING
{
  int bChangeHiragana;
  int bChangeKatakana;
} JPKRSETTING;

static OTHTR *pTRDlg;

static BOOL CALLBACK JPKRSettingDlgProc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam);

static BOOL CALLBACK JPKRSettingDlgProc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam)
{
  static HWND hHiragana;
  static HWND hKatakana;

  switch (message)
  {
    case WM_INITDIALOG:
      hHiragana = GetDlgItem(hDlg, IDC_CHECK_HIRAGANA);
      hKatakana = GetDlgItem(hDlg, IDC_CHECK_KATAKANA);

      if (GET_JPKRSET(pTRDlg)->bChangeHiragana)
        SendMessage(hHiragana , BM_SETCHECK, BST_CHECKED, 0);
      if (GET_JPKRSET(pTRDlg)->bChangeKatakana)
        SendMessage(hKatakana, BM_SETCHECK, BST_CHECKED, 0);

      return TRUE;

    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDOK:
          GET_JPKRSET(pTRDlg)->bChangeHiragana = 0;
          GET_JPKRSET(pTRDlg)->bChangeKatakana = 0;
          if (SendMessage(hHiragana, BM_GETCHECK, 0, 0) == BST_CHECKED)
            GET_JPKRSET(pTRDlg)->bChangeHiragana = 1;
          if (SendMessage(hKatakana, BM_GETCHECK, 0, 0) == BST_CHECKED)
            GET_JPKRSET(pTRDlg)->bChangeKatakana = 1;
          EndDialog(hDlg, 0);
          return TRUE;
        case IDCANCEL:
          EndDialog(hDlg, 0);
          return TRUE;
      }
  }
  return FALSE;
}

OTHEXPORT int OTHCALL OthTransInit(OTHTR *pTR)
{
  JPKRSETTING *setting;

  jpkrbuf = NULL;
  jpkrlen = 0;

  GET_JPKRSET(pTR) = malloc(sizeof(JPKRSETTING));
  if (GET_JPKRSET(pTR) == NULL)
    return -1;
  GET_JPKRSET(pTR)->bChangeHiragana = 1;
  GET_JPKRSET(pTR)->bChangeKatakana = 1;

  return 0;
}

OTHEXPORT int OTHCALL OthTransSetting(OTHTR *pTR)
{
  pTRDlg = pTR;

  DialogBox(hInst, MAKEINTRESOURCE(IDD_DIALOG_JPKR), NULL, JPKRSettingDlgProc);

  return 0;
}

OTHEXPORT int OTHCALL OthTransCopySetting(OTHTR *pDestTR, OTHTR *pSrcTR)
{
  *GET_JPKRSET(pDestTR) = *GET_JPKRSET(pSrcTR);

  return 0;
}

OTHEXPORT int OTHCALL OthTranslate(OTHTR *pTR, LPCWSTR lpStr, int iLen)
{
  int i;

  if (iLen > jpkrlen)
  {
    jpkrbuf = realloc(jpkrbuf, iLen << 1);
    if (jpkrbuf == NULL && iLen != 0)
      return -1;

    jpkrlen = iLen;
  }

  for (i = 0; i < iLen; i++)
  {
    jpkrbuf[i] = lpStr[i];

    if (GET_JPKRSET(pTR)->bChangeHiragana)
    {
      if (lpStr[i] >= HIRAGANA_BEGIN && lpStr[i] <= HIRAGANA_END)
      {
        jpkrbuf[i] = jpkrtable[lpStr[i] - HIRAGANA_BEGIN];
      }
    }

    if (GET_JPKRSET(pTR)->bChangeKatakana)
    {
      if (lpStr[i] >= KATAKANA_BEGIN && lpStr[i] <= KATAKANA_END)
      {
        jpkrbuf[i] = jpkrtable[lpStr[i] - HIRAGANA_BEGIN];
      }
    }
  }

  pTR->lpStr = jpkrbuf;
  pTR->iLen = iLen;

  return 0;
}

OTHEXPORT int OTHCALL OthTransDestroy(OTHTR *pTR)
{
  free(GET_JPKRSET(pTR));
  free(jpkrbuf);
  jpkrbuf = NULL;
  jpkrlen = 0;

  return 0;
}

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
    hInst = hinstDLL;

  return TRUE;
}



