
/**
 * @file oh_io.c
 * @version $Id$
 *
 * The source file that has the subset of POSIX filesystem functions.
 */

#include <windows.h>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>

#include "oh_port.h"
#include "oh_io.h"
#include "oh_ver.h"

extern oh_int oh_io_init(oh_void);

oh_int oh_io_init(oh_void)
{
  int hConHandle;
  long lStdHandle;
  CONSOLE_SCREEN_BUFFER_INFO coninfo;
  FILE *fp;

  if (!AllocConsole())
    return -1;

  if (!SetConsoleTitleA(oh_vr_getInfo(OH_VR_TITLE)))
  {
    FreeConsole();
    return -5;
  }

  GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), &coninfo);

  coninfo.dwSize.Y = 2000;

  SetConsoleScreenBufferSize(GetStdHandle(STD_OUTPUT_HANDLE), coninfo.dwSize);

  lStdHandle = (long)GetStdHandle(STD_OUTPUT_HANDLE);

  hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
  if (hConHandle < 0)
  {
    FreeConsole();
    return -2;
  }

  fp = _fdopen(hConHandle, "w");

  *stdout = *fp;

  setvbuf(stdout, NULL, _IONBF, 0);

  lStdHandle = (long)GetStdHandle(STD_INPUT_HANDLE);

  hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
  if (hConHandle < 0)
  {
    FreeConsole();
    return -3;
  }

  fp = _fdopen(hConHandle, "r");

  *stdin = *fp;

  setvbuf(stdin, NULL, _IONBF, 0);

  lStdHandle = (long)GetStdHandle(STD_ERROR_HANDLE);

  hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);

  fp = _fdopen(hConHandle, "w");
  if (hConHandle < 0)
  {
    FreeConsole();
    return -4;
  }

  *stderr = *fp;

  setvbuf(stderr, NULL, _IONBF, 0);

  return 0;
}

/**
 * ANSI C putchar() function.
 */
oh_int oh_io_putChar(oh_int ch)
{
  return putchar(ch);
}

/**
 * ANSI C getchar() function.
 */
oh_int oh_io_getChar(oh_void)
{
  return getchar();
}

/**
 * ANSI C fputs() function with "stdout".
 */
oh_int oh_io_putString(const oh_char *s)
{
  return fputs(s, stdout);
}

/**
 * ANSI C fgets() function with "stdin".
 */
oh_char *oh_io_getString(oh_char *s, oh_int num)
{
  return fgets(s, num, stdin);
}

/**
 * ANSI C fflush() function with "0".
 */
oh_int oh_io_flush(oh_void)
{
  return fflush(0);
}
