
/**
 * @file oh_clib.h
 * @version $Id$
 *
 * The header file that has the subset of ANSI C functions.
 */

#ifndef _OH_CLIB_H_
#define _OH_CLIB_H_

#include "oh_conf.h"
#include "oh_type.h"
#include "oh_cliba.h"

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

#ifndef oh_c_strlen
extern oh_ulong oh_c_strlen(const oh_char *str);
#endif /* oh_c_strlen */

#ifndef oh_c_strcpy
extern oh_char *oh_c_strcpy(oh_char *dst, const oh_char *src);
#endif /* oh_c_strcpy */

#ifndef oh_c_strncpy
extern oh_char *oh_c_strncpy(oh_char *dst, const oh_char *src, oh_ulong n);
#endif /* oh_c_strncpy */

#ifndef oh_c_strcat
extern oh_char *oh_c_strcat(oh_char *dst, const oh_char *src);
#endif /* oh_c_strcat */

#ifndef oh_c_strncat
extern oh_char *oh_c_strncat(oh_char *dst, const oh_char *src, oh_ulong n);
#endif /* oh_c_strncat */

#ifndef oh_c_strcmp
extern oh_int oh_c_strcmp(const oh_char *s1, const oh_char *s2);
#endif /* oh_c_strcmp */

#ifndef oh_c_strncmp
extern oh_int oh_c_strncmp(const oh_char *s1, const oh_char *s2, oh_ulong n);
#endif /* oh_c_strncmp */

#ifndef oh_c_strstr
extern oh_char *oh_c_strstr(const oh_char *haystack, const oh_char *needle);
#endif /* oh_c_strstr */

#ifndef oh_c_strchr
extern oh_char *oh_c_strchr(const oh_char *s, oh_int c);
#endif /* oh_c_strchr */

#ifndef oh_c_strrchr
extern oh_char *oh_c_strrchr(const oh_char *s, oh_int c);
#endif /* oh_c_strrchr */

#ifndef oh_c_memset
extern oh_void *oh_c_memset(oh_void *s, oh_int c, oh_ulong n);
#endif /* oh_c_memset */

#ifndef oh_c_memcpy
extern oh_void *oh_c_memcpy(oh_void *dst, const oh_void *src, oh_ulong n);
#endif /* oh_c_memcpy */

#ifndef oh_c_memmove
extern oh_void *oh_c_memmove(oh_void *dst, const oh_void *src, oh_ulong n);
#endif /* oh_c_memmove */

#ifndef oh_c_memcmp
extern oh_int oh_c_memcmp(const oh_void *s1, const oh_void *s2, oh_ulong n);
#endif /* oh_c_memcmp */

#ifndef oh_c_memchr
extern oh_void *oh_c_memchr(const oh_void *s, oh_int c, oh_ulong n);
#endif /* oh_c_memchr */

#ifndef oh_c_isupper
extern oh_int oh_c_isupper(oh_int c);
#endif /* oh_c_isupper */

#ifndef oh_c_islower
extern oh_int oh_c_islower(oh_int c);
#endif /* oh_c_islower */

#ifndef oh_c_isspace
extern oh_int oh_c_isspace(oh_int c);
#endif /* oh_c_isspace */

#ifndef oh_c_isalpha
extern oh_int oh_c_isalpha(oh_int c);
#endif /* oh_c_isalpha */

#ifndef oh_c_isdigit
extern oh_int oh_c_isdigit(oh_int c);
#endif /* oh_c_isdigit */

#ifndef oh_c_is_alnum
extern oh_int oh_c_isalnum(oh_int c);
#endif /* oh_c_isalnum */

#ifndef oh_c_isprint
extern oh_int oh_c_isprint(oh_int c);
#endif /* oh_c_isprint */

#ifndef oh_c_toupper
extern oh_int oh_c_toupper(oh_int c);
#endif /* oh_c_toupper */

#ifndef oh_c_tolower
extern oh_int oh_c_tolower(oh_int c);
#endif /* oh_c_tolower */

#ifndef oh_c_sprintf
extern oh_int oh_c_sprintf(oh_char *str, const oh_char *format, ...);
#endif /* oh_c_sprintf */

#ifndef oh_c_vsprintf
extern oh_int oh_c_vsprintf(oh_char *str, const oh_char *format, va_list list);
#endif /* oh_c_vsprintf */

#ifndef oh_c_snprintf
extern oh_int oh_c_snprintf(oh_char *str, oh_ulong size, const oh_char *format,
  ...);
#endif /* oh_c_snprintf */

#ifndef oh_c_vsnprintf
extern oh_int oh_c_vsnprintf(oh_char *str, oh_ulong size,
  const oh_char *format, va_list list);
#endif /* oh_c_vsnprintf */

#ifndef oh_c_strtol
extern oh_long oh_c_strtol(const oh_char *nptr, oh_char **endptr, oh_int base);
#endif /* oh_c_strtol */

#ifndef oh_c_strtoul
extern oh_ulong oh_c_strtoul(const oh_char *nptr, oh_char **endptr,
  oh_int base);
#endif /* oh_c_strtoul */

#ifndef oh_c_qsort
extern oh_void oh_c_qsort(oh_void *base, oh_ulong nmemb, oh_ulong size,
  oh_int (*compar)(const oh_void *, const oh_void *));
#endif /* oh_c_qsort */

#ifndef oh_c_bsearch
extern oh_void *oh_c_bsearch(const oh_void *key, const oh_void *base,
  oh_ulong nmemb, oh_ulong size,
  oh_int (*compar)(const oh_void *, const oh_void *));
#endif /* oh_c_bsearch */

#ifndef oh_c_calloc
extern oh_void *oh_c_calloc(oh_ulong nmemb, oh_ulong size);
#endif /* oh_c_calloc */

#ifndef oh_c_malloc
extern oh_void *oh_c_malloc(oh_ulong size);
#endif /* oh_c_malloc */

#ifndef oh_c_free
extern oh_void oh_c_free(oh_void *ptr);
#endif /* oh_c_free */

#ifndef oh_c_realloc
extern oh_void *oh_c_realloc(oh_void *ptr, oh_ulong size);
#endif /* oh_c_realloc */

#ifndef oh_c_strdup
extern oh_char *oh_c_strdup(const oh_char *s);
#endif /* oh_c_strdup */

#ifndef oh_c_exit
extern oh_void oh_c_exit(oh_int status);
#endif /* oh_c_exit */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _OH_CLIB_H_ */
