#ifdef OH_DEBUG

/**
 * @file oh_clib.c
 * @version $Id$
 *
 * The source file that has the subset of ANSI C functions.
 */

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>

#include "oh_port.h"
#include "oh_clib.h"
#include "oh_debug.h"

#include "snprintf.h"

/**
 * ANSI C strlen() function.
 */
oh_ulong oh_c_strlen(const oh_char *str)
{
  OH_ASSERT(str != OH_NULL);

  return strlen(str);
}

/**
 * ANSI C strcpy() function.
 */
oh_char *oh_c_strcpy(oh_char *dst, const oh_char *src)
{
  OH_ASSERT(dst != OH_NULL);
  OH_ASSERT(src != OH_NULL);

  return strcpy(dst, src);
}

/**
 * ANSI C strncpy() function.
 */
oh_char *oh_c_strncpy(oh_char *dst, const oh_char *src, oh_ulong n)
{
  OH_ASSERT(dst != OH_NULL);
  OH_ASSERT(src != OH_NULL);
  OH_ASSERT(((oh_long)n) >= 0);

  return strncpy(dst, src, n);
}

/**
 * ANSI C strcat() function.
 */
oh_char *oh_c_strcat(oh_char *dst, const oh_char *src)
{
  OH_ASSERT(dst != OH_NULL);
  OH_ASSERT(src != OH_NULL);

  return strcat(dst, src);
}

/**
 * ANSI C strncat() function.
 */
oh_char *oh_c_strncat(oh_char *dst, const oh_char *src, oh_ulong n)
{
  OH_ASSERT(dst != OH_NULL);
  OH_ASSERT(src != OH_NULL);
  OH_ASSERT(((oh_long)n) >= 0);

  return strncat(dst, src, n);
}

/**
 * ANSI C strcmp() function.
 */
oh_int oh_c_strcmp(const oh_char *s1, const oh_char *s2)
{
  OH_ASSERT(s1 != OH_NULL);
  OH_ASSERT(s2 != OH_NULL);

  return strcmp(s1, s2);
}

/**
 * ANSI C strncmp() function.
 */
oh_int oh_c_strncmp(const oh_char *s1, const oh_char *s2, oh_ulong n)
{
  OH_ASSERT(s1 != OH_NULL);
  OH_ASSERT(s2 != OH_NULL);
  OH_ASSERT(((oh_long)n) >= 0);

  return strncmp(s1, s2, n);
}

/**
 * ANSI C strstr() function.
 */
oh_char *oh_c_strstr(const oh_char *haystack, const oh_char *needle)
{
  OH_ASSERT(haystack != OH_NULL);
  OH_ASSERT(needle != OH_NULL);

  return strstr(haystack, needle);
}

/**
 * ANSI C strchr() function.
 */
oh_char *oh_c_strchr(const oh_char *s, oh_int c)
{
  OH_ASSERT(s != OH_NULL);

  return strchr(s, c);
}

/**
 * ANSI C strrchr() function.
 */
oh_char *oh_c_strrchr(const oh_char *s, oh_int c)
{
  OH_ASSERT(s != OH_NULL);

  return strrchr(s, c);
}

/**
 * ANSI C memset() function.
 */
oh_void *oh_c_memset(oh_void *s, oh_int c, oh_ulong n)
{
  OH_ASSERT(s != OH_NULL);
  OH_ASSERT(((oh_long)n) >= 0);

  return memset(s, c, n);
}

/**
 * ANSI C memcpy() function.
 */
oh_void *oh_c_memcpy(oh_void *dst, const oh_void *src, oh_ulong n)
{
  OH_ASSERT(dst != OH_NULL);
  OH_ASSERT(src != OH_NULL);
  OH_ASSERT(((oh_long)n) >= 0);

  return memcpy(dst, src, n);
}

/**
 * ANSI C memmove() function.
 */
oh_void *oh_c_memmove(oh_void *dst, const oh_void *src, oh_ulong n)
{
  OH_ASSERT(dst != OH_NULL);
  OH_ASSERT(src != OH_NULL);
  OH_ASSERT(((oh_long)n) >= 0);

  return memmove(dst, src, n);
}

/**
 * ANSI C memcmp() function.
 */
oh_int oh_c_memcmp(const oh_void *s1, const oh_void *s2, oh_ulong n)
{
  OH_ASSERT(s1 != OH_NULL);
  OH_ASSERT(s2 != OH_NULL);
  OH_ASSERT(((oh_long)n) >= 0);

  return memcmp(s1, s2, n);
}

/**
 * ANSI C memchr() function.
 */
oh_void *oh_c_memchr(const oh_void *s, oh_int c, oh_ulong n)
{
  OH_ASSERT(s != OH_NULL);
  OH_ASSERT(((oh_long)n) >= 0);

  return memchr(s, c, n);
}

/**
 * ANSI C isupper() function.
 */
oh_int oh_c_isupper(oh_int c)
{
  OH_ASSERT(c >= 0);
  OH_ASSERT(c <= 255);

  return isupper(c);
}

/**
 * ANSI C islower() function.
 */
oh_int oh_c_islower(oh_int c)
{
  OH_ASSERT(c >= 0);
  OH_ASSERT(c <= 255);

  return islower(c);
}

/**
 * ANSI C isspace() function.
 */
oh_int oh_c_isspace(oh_int c)
{
  OH_ASSERT(c >= 0);
  OH_ASSERT(c <= 255);

  return isspace(c);
}

/**
 * ANSI C isalpha() function.
 */
oh_int oh_c_isalpha(oh_int c)
{
  OH_ASSERT(c >= 0);
  OH_ASSERT(c <= 255);

  return isalpha(c);
}

/**
 * ANSI C isdigit() function.
 */
oh_int oh_c_isdigit(oh_int c)
{
  OH_ASSERT(c >= 0);
  OH_ASSERT(c <= 255);

  return isdigit(c);
}

/**
 * ANSI C isalnum() function.
 */
oh_int oh_c_isalnum(oh_int c)
{
  OH_ASSERT(c >= 0);
  OH_ASSERT(c <= 255);

  return isalnum(c);
}

/**
 * ANSI C isprint() function.
 */
oh_int oh_c_isprint(oh_int c)
{
  OH_ASSERT(c >= 0);
  OH_ASSERT(c <= 255);

  return isprint(c);
}

/**
 * ANSI C toupper() function.
 */
oh_int oh_c_toupper(oh_int c)
{
  OH_ASSERT(c >= 0);
  OH_ASSERT(c <= 255);

  return toupper(c);
}

/**
 * ANSI C tolower() function.
 */
oh_int oh_c_tolower(oh_int c)
{
  OH_ASSERT(c >= 0);
  OH_ASSERT(c <= 255);

  return tolower(c);
}

/**
 * ANSI C sprintf() function.
 */
oh_int oh_c_sprintf(oh_char *str, const oh_char *format, ...)
{
  oh_int ret;
  va_list args;

  OH_ASSERT(str != OH_NULL);
  OH_ASSERT(format != OH_NULL);

  va_start(args, format);
  ret = vsprintf(str, format, args);
  va_end(args);

  return ret;
}

/**
 * ANSI C vsprintf() function.
 */
oh_int oh_c_vsprintf(oh_char *str, const oh_char *format, va_list list)
{
  OH_ASSERT(str != OH_NULL);
  OH_ASSERT(format != OH_NULL);

  return vsprintf(str, format, list);
}

/**
 * ANSI C ( C99 ) snprintf() function.
 */
oh_int oh_c_snprintf(oh_char *str, oh_ulong size, const oh_char *format,
  ...)
{
  oh_int ret;
  va_list args;

  OH_ASSERT(str != OH_NULL);
  OH_ASSERT(format != OH_NULL);
  OH_ASSERT(((oh_long)size) >= 0);

  va_start(args, format);
  /* _vsnprintf of Visual C++ 6.0 is buggy.                           */
  /* therefore use Marks Martinec's portable snprintf implementation. */
  ret = portable_vsnprintf(str, size, format, args);
  va_end(args);

  return ret;
}

/**
 * ANSI C ( C99 ) vsnprintf() function.
 */
oh_int oh_c_vsnprintf(oh_char *str, oh_ulong size, const oh_char *format,
  va_list list)
{
  OH_ASSERT(str != OH_NULL);
  OH_ASSERT(format != OH_NULL);
  OH_ASSERT(((oh_long)size) >= 0);

  /* _vsnprintf of Visual C++ 6.0 is buggy.                           */
  /* therefore use Marks Martinec's portable snprintf implementation. */
  return portable_vsnprintf(str, size, format, list);
}

/**
 * ANSI C strtol() function.
 */
oh_long oh_c_strtol(const oh_char *nptr, oh_char **endptr, oh_int base)
{
  OH_ASSERT(base == 0 || (base >= 2 && base <= 36));
  OH_ASSERT(nptr != OH_NULL);

  return strtol(nptr, endptr, base);
}

/**
 * ANSI C strtoul() function.
 */
oh_ulong oh_c_strtoul(const oh_char *nptr, oh_char **endptr, oh_int base)
{
  OH_ASSERT(base == 0 || (base >= 2 && base <= 36));
  OH_ASSERT(nptr != OH_NULL);

  return strtoul(nptr, endptr, base);
}

/**
 * ANSI C qsort() function.
 */
oh_void oh_c_qsort(oh_void *base, oh_ulong nmemb, oh_ulong size,
  oh_int (*compar)(const oh_void *, const oh_void *))
{
  OH_ASSERT(base != OH_NULL);
  OH_ASSERT(compar != OH_NULL);
  OH_ASSERT(((oh_long)nmemb) > 0);
  OH_ASSERT(((oh_long)size) > 0);

  qsort(base, nmemb, size, compar);
}

/**
 * ANSI C bsearch() function.
 */
oh_void *oh_c_bsearch(const oh_void *key, const oh_void *base, oh_ulong nmemb,
  oh_ulong size, oh_int (*compar)(const oh_void *, const oh_void *))
{
  OH_ASSERT(key != OH_NULL);
  OH_ASSERT(base != OH_NULL);
  OH_ASSERT(compar != OH_NULL);
  OH_ASSERT(((oh_long)nmemb) > 0);
  OH_ASSERT(((oh_long)size) > 0);

  return bsearch(key, base, nmemb, size, compar);
}

/**
 * ANSI C calloc() function.
 */
oh_void *oh_c_calloc(oh_ulong nmemb, oh_ulong size)
{
  OH_ASSERT(((oh_long)nmemb) >= 0);
  OH_ASSERT(((oh_long)size) >= 0);

  return calloc(nmemb, size);
}

/**
 * ANSI C malloc() function.
 */
oh_void *oh_c_malloc(oh_ulong size)
{
  OH_ASSERT(((oh_long)size) >= 0);

  return malloc(size);
}

/**
 * ANSI C free() function.
 */
oh_void oh_c_free(oh_void *ptr)
{
  OH_ASSERT(ptr != OH_NULL);

  free(ptr);
}

/**
 * ANSI C realloc() function.
 */
oh_void *oh_c_realloc(oh_void *ptr, oh_ulong size)
{
  OH_ASSERT(((oh_long)size) >= 0);

  return realloc(ptr, size);
}

/**
 * ANSI C strdup() function.
 */
oh_char *oh_c_strdup(const oh_char *s)
{
  OH_ASSERT(s != OH_NULL);

  return strdup(s);
}

/**
 * ANSI C exit() function.
 */
oh_void oh_c_exit(oh_int status)
{

#ifdef OH_DEBUG
  {
    LPVOID msg_buf;
    FormatMessageA(
      FORMAT_MESSAGE_ALLOCATE_BUFFER |
      FORMAT_MESSAGE_FROM_SYSTEM |
      FORMAT_MESSAGE_IGNORE_INSERTS,
      NULL,
      GetLastError(),
      MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US),
      (LPSTR)&msg_buf,
      0,
      NULL);
    oh_db_log((OL_ERR, OD_SYSTEM, "%s", msg_buf));
    LocalFree(msg_buf);
    oh_db_log((OL_MSG, OD_PORT, "exit status: %d", status));
    *(oh_int *)(0) = 0; /* to cause crash */
  }
#else /* OH_DEBUG */
  exit(status);
#endif /* OH_DEBUG */

  return;
}

#endif
