#include <windows.h>
#include <Shlwapi.h>
#include <malloc.h>
#include <string.h>
#include "oh_dlres.h"

#include "oh_othmn.h"
#include "oh_txthk.h"
#include "oh_trmod.h"

#include "oh_debug.h"

#define IDM_OH_EDITBOX 50000

#define OH_VR_TITLE "Oh! Text Hooker"
#define OH_VR_LICENSE "GPL"

#define OH_MAXFILELEN 1024
#define OH_RT_SIZE 40

#if 0
#define OTHLOCKINIT() (0)
#define OTHLOCK() (0)
#define OTHUNLOCK() (0)
#define OTHTRYLOCK() (0)
#define OTHLOCKDEL() (0)
#endif

#if 0
static HANDLE hSem;
#define OTHLOCKINIT() (hSem = CreateSemaphore(NULL, 1, 1, NULL))
#define OTHLOCK() WaitForSingleObject(hSem, INFINITE)
#define OTHUNLOCK() ReleaseSemaphore(hSem, 1, NULL)
#define OTHTRYLOCK() WaitForSingleObject(hSem, 0)
#define OTHLOCKDEL() CloseHandle(hSem)
#endif

#if _WIN32_WINNT >= 0x0400
static CRITICAL_SECTION CriticalSection;
#define OTHLOCKINIT() InitializeCriticalSection(&CriticalSection)
#define OTHLOCK() EnterCriticalSection(&CriticalSection)
#define OTHUNLOCK() LeaveCriticalSection(&CriticalSection)
#define OTHTRYLOCK() (TryEnterCriticalSection(&CriticalSection) == 0)
#define OTHLOCKDEL() DeleteCriticalSection(&CriticalSection)
#else /* _WIN32_WINNT >= 0x0400 */
/* this code is buggy, but no way for win9x/me compatibility. */
static CRITICAL_SECTION CriticalSection;
static int bInLock;
#define OTHLOCKINIT() InitializeCriticalSection(&CriticalSection)
#define OTHLOCK() \
  do \
  { \
    EnterCriticalSection(&CriticalSection); \
    bInLock = 1; \
  } while (0)
#define OTHUNLOCK() \
  do \
  { \
    bInLock = 0; \
    LeaveCriticalSection(&CriticalSection); \
  } while (0)
#define OTHTRYLOCK() ((bInLock) ? (-1) : \
 (EnterCriticalSection(&CriticalSection), 0))
#define OTHLOCKDEL() DeleteCriticalSection(&CriticalSection)
#endif /* _WIN32_WINNT >= 0x0400 */

#define OTHPRINTEDLEN 2048
#define OTHBUFFEREDLEN (OTHPRINTEDLEN >> 2)

typedef struct _OTHREPTEXTBUF
{
  WCHAR pszPrintedText[OTHPRINTEDLEN];
  int iPrintedTextEnd;
  WCHAR pszBufferedText[OTHBUFFEREDLEN];
  int iBufferedTextEnd;
} OTHREPTEXTBUF;

typedef struct OTHTRCTXT
{
  OTHTR Setting;
  HMODULE Mod;
  int (OTHCALL *TransInit)(OTHTR *pTR);
  int (OTHCALL *TransSetting)(OTHTR *pOldTR);
  int (OTHCALL *TransCopySetting)(OTHTR *pDestTR, OTHTR *pSrcTR);
  int (OTHCALL *Translate)(OTHTR *pTR, LPCWSTR lpStr, int iLen);
  int (OTHCALL *TransDestroy)(OTHTR *pTR);
} OTHTRCTXT;

typedef struct OTHTXTWIN
{
  HWND hwndEdit;
  int bStop;
  int bPrintInvisibleText;
  int bUpdateOrg;
  int bChangeFontSize;
  int bAddNewLine;
  int bDontRemoveRepeat;
  int bDontRemoveRepeatSmart;
  int bRemoveRepeatSentence;
  UINT iCharSet;
  HFONT hFont;
  OTHWSTR LastStr;
  int iMaxRepeatIndex;
  int iRepeatCount;
  int iRepeatTable[OH_RT_SIZE];
  OTHTRCTXT TransCtxt;
  OTHREPTEXTBUF RepTextBuf;
} OTHTXTWIN;

typedef struct OTHAWSTR
{
  OTHSTR *inStr;
  OTHWSTR *outStr;
  int bCharConv;
} OTHAWSTR;

typedef struct OTHWWSTR
{
  OTHWSTR *inStr;
  OTHWSTR *outStr;
  int bCharConv;
} OTHWWSTR;

typedef struct OTHCHARSET
{
  LPCTSTR lpszCharSet;
  UINT iCharSet;
} OTHCHARSET;

static OTHCHARSET CharSetList[] =
  {
    {TEXT("ANSI"), ANSI_CHARSET},
    {TEXT("BALTIC"), BALTIC_CHARSET},
    {TEXT("CHINESEBIG5"), CHINESEBIG5_CHARSET},
    {TEXT("DEFAULT"), DEFAULT_CHARSET},
    {TEXT("EASTEUROPE"), EASTEUROPE_CHARSET},
    {TEXT("GB2312"), GB2312_CHARSET},
    {TEXT("GREEK"), GREEK_CHARSET},
    {TEXT("HANGUL"), HANGUL_CHARSET},
    {TEXT("MAC"), MAC_CHARSET},
    {TEXT("OEM"), OEM_CHARSET},
    {TEXT("RUSSIAN"), RUSSIAN_CHARSET},
    {TEXT("SHIFTJIS"), SHIFTJIS_CHARSET},
    {TEXT("SYMBOL"), SYMBOL_CHARSET},
    {TEXT("TURKISH"), TURKISH_CHARSET},
    {TEXT("VIETNAMESE"), VIETNAMESE_CHARSET},
    {TEXT("JOHAB"), JOHAB_CHARSET},
    {TEXT("ARABIC"), ARABIC_CHARSET},
    {TEXT("HEBREW"), HEBREW_CHARSET},
    {TEXT("THAI"), THAI_CHARSET}
  };
static int nCharSetList = sizeof(CharSetList) / sizeof(OTHCHARSET);

static HINSTANCE hInst;
static HMENU hChildMenu;
static HMENU hMenu;
static HWND hwndClient, hwndFrame;
static TCHAR szAppName[] = TEXT(OH_VR_TITLE) TEXT(" - ") TEXT(OH_VR_LICENSE);
static TCHAR szFrameClass[] = TEXT("OhTextHookerFrame");
static TCHAR szChildClass[] = TEXT("OhTextHookerChild");
static TCHAR szChildTitle[] = TEXT("Text Window");
static OTHTXTWIN *lpGlobalTxtWin;
static OTHTXTWIN *lpSettingTxtWin;
static DWORD HookingThreadId;
static int nChildWindows;
static TCHAR szPluginDir[OH_MAXFILELEN];
static TCHAR szPluginDll[OH_MAXFILELEN];

static void OthMoveToPrintedText(OTHREPTEXTBUF *pRepTextBuf);
static LPWSTR OthGetPrintText(OTHREPTEXTBUF *pRepTextBuf, LPWSTR pszText);
static int LoadTRMod(OTHTRCTXT *ctxt, LPCTSTR szModFileName);
static int UnloadTRMod(OTHTRCTXT *ctxt);
static HFONT LockedHookDC(HDC hdc);
static BOOL CALLBACK LogEnumProc(HWND hwnd, LPARAM lParam);
static BOOL CALLBACK ApplyAllEnumProc(HWND hwnd, LPARAM lParam);
static BOOL CALLBACK CloseEnumProc(HWND hwnd, LPARAM lParam);
static LRESULT CALLBACK FrameWndProc(HWND hwnd, UINT message, WPARAM wParam,
  LPARAM lParam);
static LRESULT CALLBACK ChildWndProc(HWND hwnd, UINT message, WPARAM wParam,
  LPARAM lParam);
static BOOL CALLBACK AboutDlgProc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam);
static BOOL CALLBACK SettingDlgProc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam);
static BOOL CALLBACK SendTextEnumProcA(HWND hwnd, LPARAM lParam);
static BOOL CALLBACK SendTextEnumProcW(HWND hwnd, LPARAM lParam);

static void OthMoveToPrintedText(OTHREPTEXTBUF *pRepTextBuf)
{
  if (pRepTextBuf->iPrintedTextEnd + pRepTextBuf->iBufferedTextEnd + 1 >
    OTHPRINTEDLEN)
  {
    MoveMemory(pRepTextBuf->pszPrintedText,
      &pRepTextBuf->pszPrintedText[OTHBUFFEREDLEN],
      (OTHPRINTEDLEN - OTHBUFFEREDLEN) << 1);
    pRepTextBuf->iPrintedTextEnd -= OTHBUFFEREDLEN;
  }
  lstrcpynW(&pRepTextBuf->pszPrintedText[pRepTextBuf->iPrintedTextEnd],
    pRepTextBuf->pszBufferedText, OTHBUFFEREDLEN);
  pRepTextBuf->iPrintedTextEnd = lstrlenW(pRepTextBuf->pszPrintedText);
  pRepTextBuf->iBufferedTextEnd = 0;
}

static LPWSTR OthGetPrintText(OTHREPTEXTBUF *pRepTextBuf, LPWSTR pszText)
{
  LPWSTR str;

  if (pszText == NULL || pszText[0] == 0)
  {
    return NULL;
  }

  if (pRepTextBuf->iBufferedTextEnd == OTHBUFFEREDLEN - 1)
  {
    pRepTextBuf->iBufferedTextEnd = 0;
  }

  lstrcpynW(&pRepTextBuf->pszBufferedText[pRepTextBuf->iBufferedTextEnd],
    pszText, OTHBUFFEREDLEN - pRepTextBuf->iBufferedTextEnd);
  pRepTextBuf->iBufferedTextEnd = lstrlenW(pRepTextBuf->pszBufferedText);

  str = StrStrW(pRepTextBuf->pszPrintedText, pRepTextBuf->pszBufferedText);
  if (str == NULL)
  {
    OthMoveToPrintedText(pRepTextBuf);

    return pRepTextBuf->pszBufferedText;
  }
  else
  {
    do
    {
      if ((&str[pRepTextBuf->iBufferedTextEnd]) ==
        (&pRepTextBuf->pszPrintedText[pRepTextBuf->iPrintedTextEnd]) &&
        pRepTextBuf->iBufferedTextEnd > 2)
      {
        pRepTextBuf->iBufferedTextEnd = 0;
        return NULL;
      }
      str = StrStrW(str + 1, pRepTextBuf->pszBufferedText);
    }
    while (str);
  }

  return NULL;
}

static BOOL CALLBACK LogEnumProc(HWND hwnd, LPARAM lParam)
{
  HWND hwndEdit;

  hwndEdit = GetDlgItem(hwnd, IDM_OH_EDITBOX);
  if (hwndEdit == NULL) /* hack */
    return TRUE;

  switch (lParam)
  {
    case IDM_OH_LOG_START_ALL:
      SendMessage(hwnd, WM_COMMAND, MAKEWPARAM(IDM_OH_LOG_START, 0), 0);
      break;
    case IDM_OH_LOG_STOP_ALL:
      SendMessage(hwnd, WM_COMMAND, MAKEWPARAM(IDM_OH_LOG_STOP, 0), 0);
      break;
    case IDM_OH_LOG_CLEAR_ALL:
      SendMessage(hwnd, WM_COMMAND, MAKEWPARAM(IDM_OH_LOG_CLEAR, 0), 0);
      break;
  }

  return TRUE;
}

static BOOL CALLBACK ApplyAllEnumProc(HWND hwnd, LPARAM lParam)
{
  OTHTXTWIN *txtwin;
  HWND hwndEdit;
  LOGFONT lf;
  HFONT hFont;

  hwndEdit = GetDlgItem(hwnd, IDM_OH_EDITBOX);
  if (hwndEdit == NULL) /* hack */
    return TRUE;

  if (lpSettingTxtWin->hwndEdit == hwndEdit)
    return TRUE;

  txtwin = (OTHTXTWIN *)GetWindowLong(hwnd, 0);
  txtwin->bPrintInvisibleText = lpSettingTxtWin->bPrintInvisibleText;
  txtwin->bAddNewLine = lpSettingTxtWin->bAddNewLine;
  txtwin->bDontRemoveRepeat = lpSettingTxtWin->bDontRemoveRepeat;
  txtwin->bDontRemoveRepeatSmart = lpSettingTxtWin->bDontRemoveRepeatSmart;
  txtwin->bRemoveRepeatSentence = lpSettingTxtWin->bRemoveRepeatSentence;
  txtwin->bChangeFontSize = lpSettingTxtWin->bChangeFontSize;
  txtwin->iCharSet = lpSettingTxtWin->iCharSet;

  if (lpSettingTxtWin->hFont)
  {
    GetObject(lpSettingTxtWin->hFont, sizeof(LOGFONT), &lf);
    hFont = CreateFontIndirect(&lf);
    OH_ASSERT(hFont);
    SendMessage(hwndEdit, WM_SETFONT, (WPARAM)hFont, TRUE);
    if (txtwin->hFont)
      DeleteObject(txtwin->hFont);
    txtwin->hFont = hFont;
  }
  else
  {
    if (txtwin->hFont)
      DeleteObject(txtwin->hFont);
    txtwin->hFont = NULL;
  }

  return TRUE;
}

static BOOL CALLBACK CloseEnumProc(HWND hwnd, LPARAM lParam)
{
  if (GetWindow(hwnd, GW_OWNER))
    return TRUE;
  SendMessage(GetParent(hwnd), WM_MDIRESTORE, (WPARAM)hwnd, 0);
  if (!SendMessage(hwnd, WM_QUERYENDSESSION, 0, 0))
    return TRUE;
  SendMessage(GetParent(hwnd), WM_MDIDESTROY, (WPARAM)hwnd, 0);
  return TRUE;
}

static LRESULT CALLBACK FrameWndProc(HWND hwnd, UINT message, WPARAM wParam,
  LPARAM lParam)
{
  HWND hwndChild;
  MDICREATESTRUCT mdicreate;
  CLIENTCREATESTRUCT clientcreate;

  switch (message)
  {
    case WM_CREATE:
      hwndFrame = hwnd;
      clientcreate.hWindowMenu = hChildMenu;
      clientcreate.idFirstChild = IDM_OH_EDITBOX;
      hwndClient =
        CreateWindow(TEXT("MDICLIENT"), NULL,
          WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE, 0, 0, 0, 0, hwnd, NULL,
          hInst, (PSTR)&clientcreate);
      OH_ASSERT(hwndClient != 0);
      PostMessage(hwnd, WM_COMMAND, IDM_OH_FILE_NEW, 0);
      PostMessage(hwnd, WM_COMMAND, IDM_OH_OPTIONS_SETTINGS, 0);
      SetTimer(hwnd, 1, 500, NULL);
      return 0;
    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDM_OH_FILE_NEW:
          mdicreate.szClass = szChildClass;
          mdicreate.szTitle = szChildTitle;
          mdicreate.hOwner = hInst;
          mdicreate.x = CW_USEDEFAULT;
          mdicreate.y = CW_USEDEFAULT;
          mdicreate.cx = CW_USEDEFAULT;
          mdicreate.cy = CW_USEDEFAULT;
          mdicreate.style = 0;
          mdicreate.lParam = 0;
          hwndChild =
            (HWND)SendMessage(hwndClient, WM_MDICREATE, 0,
              (LPARAM)(LPMDICREATESTRUCT)&mdicreate);
          return 0;
        case IDM_OH_FILE_CLOSE:
          hwndChild = (HWND)SendMessage(hwndClient, WM_MDIGETACTIVE, 0, 0);
          if (SendMessage(hwndChild, WM_QUERYENDSESSION, 0, 0))
            SendMessage(hwndClient, WM_MDIDESTROY, (WPARAM)hwndChild, 0);
          return 0;
        case IDM_OH_FILE_EXIT:
          SendMessage(hwnd, WM_CLOSE, 0, 0);
          return 0;
        case IDM_OH_OPTIONS_ALWAYS_ON_TOP:
          if (GetMenuState(hMenu, IDM_OH_OPTIONS_ALWAYS_ON_TOP, MF_BYCOMMAND)
            & MF_CHECKED)
          {
            SetWindowPos(hwnd, HWND_NOTOPMOST, 0, 0, 0, 0,
              SWP_NOSIZE | SWP_NOMOVE);
            CheckMenuItem(hMenu, IDM_OH_OPTIONS_ALWAYS_ON_TOP, MF_UNCHECKED);
          }
          else
          {
            SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0,
              SWP_NOSIZE | SWP_NOMOVE);
            CheckMenuItem(hMenu, IDM_OH_OPTIONS_ALWAYS_ON_TOP, MF_CHECKED);
          }
          return 0;
        case IDM_OH_WINDOW_TILE_HORIZONTAL:
          SendMessage(hwndClient, WM_MDITILE, MDITILE_HORIZONTAL, 0);
          return 0;
        case IDM_OH_WINDOW_TILE_VERTICAL:
          SendMessage(hwndClient, WM_MDITILE, MDITILE_VERTICAL, 0);
          return 0;
        case IDM_OH_WINDOW_CASCADE:
          SendMessage(hwndClient, WM_MDICASCADE, 0, 0);
          return 0;
        case IDM_OH_WINDOW_ARRANGE:
          SendMessage(hwndClient, WM_MDIICONARRANGE, 0, 0);
          return 0;
        case IDM_OH_WINDOW_CLOSE_ALL:
          EnumChildWindows(hwndClient, CloseEnumProc, 0);
          return 0;
        case IDM_OH_OPTIONS_SETTINGS:
          hwndChild = (HWND)SendMessage(hwndClient, WM_MDIGETACTIVE, 0, 0);
          if (!IsWindow(hwndChild))
            return 0;
          lpSettingTxtWin = (OTHTXTWIN *)GetWindowLong(hwndChild, 0);
          if (GetMenuState(hMenu, IDM_OH_OPTIONS_ALWAYS_ON_TOP, MF_BYCOMMAND)
            & MF_CHECKED)
          {
            SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0,
              SWP_NOSIZE | SWP_NOMOVE);
          }
          if (OTHTRYLOCK() != 0)
          {
            PostMessage(hwnd, message, wParam, lParam);
          }
          else
          {
            DialogBox(hInst, MAKEINTRESOURCE(IDD_OH_SETTINGS), hwndChild,
              SettingDlgProc);
            OTHUNLOCK();
          }
          return 0;
        case IDM_OH_OPTIONS_LOAD_SETTINGS:
        case IDM_OH_OPTIONS_SAVE_SETTINGS:
        case IDM_OH_HELP_HELP:
        case IDM_OH_FILE_SAVE:
          MessageBox(hwnd, TEXT("Not Implemented yet"),
            TEXT("Not Implemented yet"), MB_ICONINFORMATION | MB_OK);
          return 0;
        case IDM_OH_HELP_ABOUT:
          DialogBox(hInst, MAKEINTRESOURCE(IDD_OH_ABOUT), hwnd, AboutDlgProc);
          return 0;
        default:
          hwndChild = (HWND)SendMessage(hwndClient, WM_MDIGETACTIVE, 0, 0);
          if (IsWindow(hwndChild))
            SendMessage(hwndChild, WM_COMMAND, wParam, lParam);
          break;
      }
      break;
    case WM_CLOSE:
      if (IDOK != MessageBox(hwnd, TEXT("Exit from ") TEXT(OH_VR_TITLE)
        TEXT(" will terminate the hooked process.\nAre you sure?"),
        TEXT("Confirm to exit"), MB_ICONWARNING | MB_OKCANCEL |
        MB_DEFBUTTON2))
        return 0;
      break;
    case WM_TIMER:
      if (OTHTRYLOCK() == 0)
      {
        OTHWWSTR wwstr;
        OTHWSTR inStr;
        OTHWSTR outStr;

        inStr.lpStr = L"";
        inStr.iLen = lstrlenW(inStr.lpStr) + 1;
        outStr.lpStr = NULL;
        outStr.iLen = 0;
        wwstr.inStr = &inStr;
        wwstr.outStr = &outStr;
        wwstr.bCharConv = 0;

        oh_db_log((OL_MSG, OD_DEF, "WM_TIMER"));
        EnumChildWindows(hwndClient, SendTextEnumProcW, (LPARAM)&wwstr);
        OTHUNLOCK();

        if (inStr.lpStr != outStr.lpStr)
          free(outStr.lpStr);
      }
      return 0;
    case WM_DESTROY:
      PostQuitMessage(0);
      return 0;
  }
  return DefFrameProc(hwnd, hwndClient, message, wParam, lParam);
}

static LRESULT CALLBACK ChildWndProc(HWND hwnd, UINT message, WPARAM wParam,
  LPARAM lParam)
{
  HWND hwndEdit;
  OTHTXTWIN *txtwin;
  RECT rcClient;

  switch (message)
  {
    case WM_CREATE:
      hwndClient = GetParent(hwnd);
      hwndFrame = GetParent(hwndClient);
      hwndEdit =
        CreateWindowExW(WS_EX_CLIENTEDGE, L"EDIT", NULL,
          WS_CHILD | WS_VISIBLE | WS_VSCROLL | ES_LEFT | ES_MULTILINE |
          ES_AUTOVSCROLL, 0, 0, 0, 0, hwnd, (HMENU)IDM_OH_EDITBOX, hInst,
          NULL);
      OH_ASSERT(hwndEdit != 0);
      SendMessage(hwndEdit, EM_LIMITTEXT, -1, 0);
      txtwin = malloc(sizeof(OTHTXTWIN));
      OH_ASSERT(txtwin != NULL);
      ZeroMemory(txtwin, sizeof(OTHTXTWIN));
      txtwin->iCharSet = SHIFTJIS_CHARSET;
      txtwin->hFont =
        CreateFont(0, 0, 0, 0, 0, 0, 0, 0, SHIFTJIS_CHARSET, 0, 0, 0, 0,
          TEXT("MS Gothic"));
      OH_ASSERT(txtwin->hFont);
      txtwin->hwndEdit = hwndEdit;
      SendMessage(txtwin->hwndEdit, WM_SETFONT, (WPARAM)txtwin->hFont, TRUE);
      SetWindowLong(hwnd, 0, (long)txtwin);
      nChildWindows++;
      PostMessage(hwndClient, WM_MDIMAXIMIZE, (WPARAM)hwnd, 0);
      return 0;
    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDM_OH_LOG_START:
          txtwin = (OTHTXTWIN *)GetWindowLong(hwnd, 0);
          txtwin->bStop = 0;
          CheckMenuItem(hMenu, IDM_OH_LOG_START, MF_CHECKED);
          CheckMenuItem(hMenu, IDM_OH_LOG_STOP, MF_UNCHECKED);
          return 0;
        case IDM_OH_LOG_STOP:
          txtwin = (OTHTXTWIN *)GetWindowLong(hwnd, 0);
          txtwin->bStop = TRUE;
          CheckMenuItem(hMenu, IDM_OH_LOG_START, MF_UNCHECKED);
          CheckMenuItem(hMenu, IDM_OH_LOG_STOP, MF_CHECKED);
          return 0;
        case IDM_OH_LOG_CLEAR:
          txtwin = (OTHTXTWIN *)GetWindowLong(hwnd, 0);

          SendMessage(txtwin->hwndEdit, EM_SETSEL, 0, -1);
          SendMessage(txtwin->hwndEdit, EM_REPLACESEL, 0, (LPARAM)TEXT(""));

          return 0;
        case IDM_OH_LOG_START_ALL:
          EnumChildWindows(hwndClient, LogEnumProc, IDM_OH_LOG_START_ALL);
          return 0;
        case IDM_OH_LOG_STOP_ALL:
          EnumChildWindows(hwndClient, LogEnumProc, IDM_OH_LOG_STOP_ALL);
          return 0;
        case IDM_OH_LOG_CLEAR_ALL:
          EnumChildWindows(hwndClient, LogEnumProc, IDM_OH_LOG_CLEAR_ALL);
          return 0;
      }
      return 0;
    case WM_SIZE:
    case WM_SIZING:
      GetClientRect(hwnd, &rcClient);
      hwndEdit = GetDlgItem(hwnd, IDM_OH_EDITBOX);

      SetWindowPos(hwndEdit, NULL, 0, 0, rcClient.right, rcClient.bottom,
        SWP_NOZORDER);
      break;
    case WM_MDIACTIVATE:
      if (hwnd == (HWND)lParam)
      {
        txtwin = (OTHTXTWIN *)GetWindowLong(hwnd, 0);
        if (txtwin->bStop)
        {
          CheckMenuItem(hMenu, IDM_OH_LOG_START, MF_UNCHECKED);
          CheckMenuItem(hMenu, IDM_OH_LOG_STOP, MF_CHECKED);
        }
        else
        {
          CheckMenuItem(hMenu, IDM_OH_LOG_START, MF_CHECKED);
          CheckMenuItem(hMenu, IDM_OH_LOG_STOP, MF_UNCHECKED);
        }
      }
      return 0;
    case WM_QUERYENDSESSION:
    case WM_CLOSE:
      if (nChildWindows <= 1)
        return 0;
      nChildWindows--;
      break;
    case WM_DESTROY:
      txtwin = (OTHTXTWIN *)GetWindowLong(hwnd, 0);
      if (txtwin->TransCtxt.Mod)
      {
        OTHTRCTXT *ctxt;

        ctxt = &txtwin->TransCtxt;
        ctxt->TransDestroy(&ctxt->Setting);
        FreeLibrary(ctxt->Mod);
        ctxt->Mod = NULL;
      }
      if (txtwin == lpGlobalTxtWin)
      {
        lpGlobalTxtWin = NULL;
      }
      DeleteObject(txtwin->hFont);
      free(txtwin->LastStr.lpStr);
      free(txtwin);
      return 0;
  }
  return DefMDIChildProc(hwnd, message, wParam, lParam);
}

static BOOL CALLBACK AboutDlgProc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam)
{
  switch (message)
  {
    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDOK:
          EndDialog(hDlg, 0);
          return TRUE;
      }
      break;
  }
  return FALSE;
}

static int LoadTRMod(OTHTRCTXT *ctxt, LPCTSTR szModFileName)
{
  TCHAR szModFullFileName[OH_MAXFILELEN];

  wsprintf(szModFullFileName, TEXT("%s%s"), szPluginDir, szModFileName);

  ZeroMemory(ctxt, sizeof(OTHTRCTXT));
  ctxt->Mod = LoadLibrary(szModFullFileName);
  if (ctxt->Mod == NULL)
  {
    oh_db_log((OL_WRN, OD_DEF, "load dll fail"));
    goto invalidmod;
  }
  ctxt->TransInit =
    (int (OTHCALL *)(OTHTR *))
      GetProcAddress(ctxt->Mod, "_OthTransInit@4");
  if (ctxt->TransInit == NULL)
  {
    ctxt->TransInit =
      (int (OTHCALL *)(OTHTR *))
        GetProcAddress(ctxt->Mod, "OthTransInit");
    if (ctxt->TransInit == NULL)
    {
      oh_db_log((OL_WRN, OD_DEF, "OthTransInit not found"));
      goto invalidmod;
    }
  }
  ctxt->TransSetting =
    (int (OTHCALL *)(OTHTR *))
      GetProcAddress(ctxt->Mod, "_OthTransSetting@4");
  if (ctxt->TransSetting == NULL)
  {
    ctxt->TransSetting =
      (int (OTHCALL *)(OTHTR *))
        GetProcAddress(ctxt->Mod, "OthTransSetting");
    if (ctxt->TransSetting == NULL)
    {
      oh_db_log((OL_WRN, OD_DEF, "OthTransSetting not found"));
      goto invalidmod;
    }
  }
  ctxt->TransCopySetting =
    (int (OTHCALL *)(OTHTR *, OTHTR *))
      GetProcAddress(ctxt->Mod, "_OthTransCopySetting@8");
  if (ctxt->TransCopySetting == NULL)
  {
    ctxt->TransCopySetting =
      (int (OTHCALL *)(OTHTR *, OTHTR *))
        GetProcAddress(ctxt->Mod, "OthTransCopySetting");
    if (ctxt->TransCopySetting == NULL)
    {
      oh_db_log((OL_WRN, OD_DEF, "OthTransCopySetting not found"));
      goto invalidmod;
    }
  }
  ctxt->Translate =
    (int (OTHCALL *)(OTHTR *, LPCWSTR, int))
      GetProcAddress(ctxt->Mod, "_OthTranslate@12");
  if (ctxt->Translate == NULL)
  {
    ctxt->Translate =
      (int (OTHCALL *)(OTHTR *, LPCWSTR, int))
        GetProcAddress(ctxt->Mod, "OthTranslate");
    if (ctxt->Translate == NULL)
    {
      oh_db_log((OL_WRN, OD_DEF, "OthTranslate not found"));
      goto invalidmod;
    }
  }
  ctxt->TransDestroy =
    (int (OTHCALL *)(OTHTR *))
      GetProcAddress(ctxt->Mod, "_OthTransDestroy@4");
  if (ctxt->TransDestroy == NULL)
  {
    ctxt->TransDestroy =
      (int (OTHCALL *)(OTHTR *))
        GetProcAddress(ctxt->Mod, "OthTransDestroy");
    if (ctxt->TransDestroy == NULL)
    {
      oh_db_log((OL_WRN, OD_DEF, "OthTransDestroy not found"));
      goto invalidmod;
    }
  }

  if (ctxt->TransInit(&ctxt->Setting) != 0)
  {
    oh_db_log((OL_WRN, OD_DEF, "TransInit fail"));
    goto invalidmod;
  }

  return 0;

invalidmod:
  if (ctxt->Mod)
    FreeLibrary(ctxt->Mod);
  ctxt->Mod = NULL;

  return -1;
}

static int UnloadTRMod(OTHTRCTXT *ctxt)
{
  if (ctxt->Mod)
  {
    ctxt->TransDestroy(&ctxt->Setting);
    FreeLibrary(ctxt->Mod);
    ctxt->Mod = NULL;
  }
  return 0;
}

static BOOL CALLBACK SettingDlgProc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam)
{
  int i;
  LRESULT idx;
  LOGFONT lf;
  CHOOSEFONT cf;
  static HFONT hFont;
  static HWND hFontButton;
  static HWND hCharSet;
  static HWND hPrintInvisibleText;
  static HWND hUpdateOrg;
  static HWND hDontChangeFontSize;
  static HWND hAddNewLine;
  static HWND hRemoveRepeat;
  static HWND hRemoveRepeatSmart;
  static HWND hRemoveRepeatSentence;
  static HWND hApplyToAll;
  static HWND hTransMod;
  static OTHTRCTXT TransCtxt;
  static LRESULT nopluginidx;
  OTHTRCTXT NewTransCtxt;
  TCHAR szModFileName[OH_MAXFILELEN];
  TCHAR szModFullFileName[OH_MAXFILELEN];

  switch (message)
  {
    case WM_INITDIALOG:

      hFontButton = GetDlgItem(hDlg, IDC_BUTTON_OH_FONT);
      hCharSet = GetDlgItem(hDlg, IDC_COMBO_OH_CHAR_SET);
      hTransMod = GetDlgItem(hDlg, IDC_COMBO_OH_TRANS_MOD);

      hPrintInvisibleText = GetDlgItem(hDlg, IDC_CHECK_OH_PRINT_INV_TEXT);
      hUpdateOrg = GetDlgItem(hDlg, IDC_CHECK_OH_UPDATE_ORIGINAL_TEXT);
      hDontChangeFontSize = GetDlgItem(hDlg, IDC_CHECK_OH_DONT_CHANGE_SIZE);
      hAddNewLine = GetDlgItem(hDlg, IDC_CHECK_OH_ADD_NEWLINE);
      hRemoveRepeat = GetDlgItem(hDlg, IDC_CHECK_OH_REMOVE_REPEAT);
      hRemoveRepeatSmart = GetDlgItem(hDlg, IDC_CHECK_OH_REMOVE_REPEAT_SMART);
      hRemoveRepeatSentence = GetDlgItem(hDlg,
        IDC_CHECK_OH_REMOVE_REPEAT_SENTENCE);
      hApplyToAll = GetDlgItem(hDlg, IDC_CHECK_OH_APPLY_TO_ALL);

      if (lpSettingTxtWin->bPrintInvisibleText)
        SendMessage(hPrintInvisibleText, BM_SETCHECK, BST_CHECKED, 0);
      if (lpSettingTxtWin->bAddNewLine)
        SendMessage(hAddNewLine, BM_SETCHECK, BST_CHECKED, 0);
      if (!lpSettingTxtWin->bDontRemoveRepeat)
        SendMessage(hRemoveRepeat, BM_SETCHECK, BST_CHECKED, 0);
      if (!lpSettingTxtWin->bDontRemoveRepeatSmart)
        SendMessage(hRemoveRepeatSmart, BM_SETCHECK, BST_CHECKED, 0);
      if (lpSettingTxtWin->bRemoveRepeatSentence)
        SendMessage(hRemoveRepeatSentence, BM_SETCHECK, BST_CHECKED, 0);
      if (!lpSettingTxtWin->bChangeFontSize)
        SendMessage(hDontChangeFontSize, BM_SETCHECK, BST_CHECKED, 0);
      if (lpSettingTxtWin->bUpdateOrg)
      {
        SendMessage(hUpdateOrg, BM_SETCHECK, BST_CHECKED, 0);
        EnableWindow(hAddNewLine, FALSE);
        EnableWindow(hRemoveRepeat, FALSE);
        EnableWindow(hRemoveRepeatSmart, FALSE);
        EnableWindow(hRemoveRepeatSentence, FALSE);
        EnableWindow(hPrintInvisibleText, FALSE);
      }
      if (lpSettingTxtWin->bDontRemoveRepeat)
      {
        EnableWindow(hRemoveRepeatSmart, FALSE);
      }

      SendMessage(hCharSet, CB_RESETCONTENT, 0, 0);
      for (i = 0; i < nCharSetList; i++)
      {
        idx = SendMessage(hCharSet, CB_ADDSTRING, 0,
          (LPARAM)CharSetList[i].lpszCharSet);
        SendMessage(hCharSet, CB_SETITEMDATA, idx,
          (LPARAM)CharSetList[i].iCharSet);
        if (CharSetList[i].iCharSet == lpSettingTxtWin->iCharSet)
        {
          SendMessage(hCharSet, CB_SETCURSEL, i, 0);
        }
      }
      if (lpSettingTxtWin->hFont)
      {
        GetObject(lpSettingTxtWin->hFont, sizeof(LOGFONT), &lf);
        SetWindowText(hFontButton, lf.lfFaceName);
        hFont = CreateFontIndirect(&lf);
        OH_ASSERT(hFont);
      }
      else
      {
        hFont = NULL;
      }

      NewTransCtxt = lpSettingTxtWin->TransCtxt;

      SendMessage(hTransMod, CB_DIR, DDL_ARCHIVE | DDL_HIDDEN | DDL_READONLY
        | DDL_READWRITE | DDL_SYSTEM, (LPARAM)szPluginDll);
      nopluginidx =
        SendMessage(hTransMod, CB_ADDSTRING, 0, (LPARAM)TEXT("<no plugin>"));
      if (NewTransCtxt.Mod)
      {
        DWORD ret;
        TCHAR *szModName;

        ret =
          GetModuleFileName(NewTransCtxt.Mod, szModFileName, OH_MAXFILELEN);
        OH_ASSERT(ret);
        ret =
          GetFullPathName(szModFileName, OH_MAXFILELEN, szModFullFileName,
            &szModName);
        OH_ASSERT(ret);
        ZeroMemory(&TransCtxt, sizeof(OTHTRCTXT));
        if (LoadTRMod(&TransCtxt, szModName) == 0)
        {
          TransCtxt.TransCopySetting(&TransCtxt.Setting,
            &NewTransCtxt.Setting);
          idx =
            SendMessage(hTransMod, CB_FINDSTRINGEXACT, -1, (LPARAM)szModName);
          OH_ASSERT(idx != CB_ERR);
          SendMessage(hTransMod, CB_SETCURSEL, idx, 0);
        }
        else
        {
          SendMessage(hTransMod, CB_SETCURSEL, nopluginidx, 0);
          ZeroMemory(&TransCtxt, sizeof(OTHTRCTXT));
        }
      }
      else
      {
        SendMessage(hTransMod, CB_SETCURSEL, nopluginidx, 0);
        ZeroMemory(&TransCtxt, sizeof(OTHTRCTXT));
      }

      if (GetMenuState(hMenu, IDM_OH_OPTIONS_ALWAYS_ON_TOP, MF_BYCOMMAND)
        & MF_CHECKED)
      {
        SetWindowPos(hDlg, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
      }

      return TRUE;
    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDC_COMBO_OH_CHAR_SET:
          if (HIWORD(wParam) != CBN_SELCHANGE)
            return FALSE;
          ZeroMemory(&lf, sizeof(lf));
          if (hFont)
          {
            GetObject(hFont, sizeof(LOGFONT), &lf);
            DeleteObject(hFont);
          }
          idx = SendMessage(hCharSet, CB_GETCURSEL, 0, 0);
          lf.lfCharSet = (BYTE)SendMessage(hCharSet, CB_GETITEMDATA, idx,
            0);
          hFont = CreateFontIndirect(&lf);

          return TRUE;

        case IDC_BUTTON_OH_FONT:

          ZeroMemory(&cf, sizeof(cf));
          ZeroMemory(&lf, sizeof(lf));

          if (hFont)
            GetObject(hFont, sizeof(LOGFONT), &lf);

          cf.lStructSize = sizeof(cf);
          cf.hwndOwner = hDlg;
          cf.Flags =
            CF_SCREENFONTS | CF_INITTOLOGFONTSTRUCT;
          cf.lpLogFont = &lf;

          if (ChooseFont(&cf))
          {
            if (hFont)
              DeleteObject(hFont);
            if (lstrlen(lf.lfFaceName) == 0)
            {
              hFont = NULL;
              SetWindowText(hFontButton, TEXT("<default font>"));
            }
            else
            {
              idx = SendMessage(hCharSet, CB_GETCURSEL, 0, 0);
              lf.lfCharSet = (BYTE)SendMessage(hCharSet, CB_GETITEMDATA, idx,
                0);
              hFont = CreateFontIndirect(&lf);
              SetWindowText(hFontButton, lf.lfFaceName);
            }
          }

          return TRUE;

        case IDC_BUTTON_OH_TRANS_MOD_SETTINGS:
          if (TransCtxt.Mod)
            TransCtxt.TransSetting(&TransCtxt.Setting);
          return TRUE;

        case IDC_COMBO_OH_TRANS_MOD:
          if (HIWORD(wParam) != CBN_SELCHANGE)
            return FALSE;
          if ((HWND)lParam != hTransMod)
            return FALSE;
          idx = SendMessage(hTransMod, CB_GETCURSEL, 0, 0);
          SendMessage(hTransMod, CB_GETLBTEXT, idx, (LPARAM)szModFileName);
          ZeroMemory(&NewTransCtxt, sizeof(OTHTRCTXT));
          if (lstrcmp(szModFileName, TEXT("<no plugin>")) == 0)
          {
            UnloadTRMod(&TransCtxt);
            return FALSE;
          }
          if (LoadTRMod(&NewTransCtxt, szModFileName) != 0)
          {
            UnloadTRMod(&TransCtxt);
            SendMessage(hTransMod, CB_SETCURSEL, nopluginidx, 0);
            MessageBox(hDlg, TEXT("Invalid Plugin DLL"), TEXT("Plugin Error"),
              MB_ICONSTOP | MB_OK);
            return FALSE;
          }
          UnloadTRMod(&TransCtxt);
          TransCtxt = NewTransCtxt;

          return FALSE;

        case IDC_CHECK_OH_UPDATE_ORIGINAL_TEXT:
          if (HIWORD(wParam) != BN_CLICKED)
            return FALSE;
          if ((HWND)lParam != hUpdateOrg)
            return FALSE;
          if (SendMessage(hUpdateOrg, BM_GETCHECK, 0, 0) == BST_CHECKED)
          {
            SendMessage(hAddNewLine, BM_SETCHECK, BST_UNCHECKED, 0);
            EnableWindow(hAddNewLine, FALSE);
            SendMessage(hRemoveRepeat, BM_SETCHECK, BST_UNCHECKED, 0);
            EnableWindow(hRemoveRepeat, FALSE);
            SendMessage(hRemoveRepeatSmart, BM_SETCHECK, BST_UNCHECKED, 0);
            EnableWindow(hRemoveRepeatSmart, FALSE);
            SendMessage(hRemoveRepeatSentence, BM_SETCHECK, BST_UNCHECKED, 0);
            EnableWindow(hRemoveRepeatSentence, FALSE);
            SendMessage(hPrintInvisibleText, BM_SETCHECK, BST_UNCHECKED, 0);
            EnableWindow(hPrintInvisibleText, FALSE);
          }
          else
          {
            EnableWindow(hAddNewLine, TRUE);
            EnableWindow(hRemoveRepeat, TRUE);
            EnableWindow(hRemoveRepeatSmart, TRUE);
            EnableWindow(hRemoveRepeatSentence, TRUE);
            EnableWindow(hPrintInvisibleText, TRUE);
          }

          return FALSE;

        case IDC_CHECK_OH_REMOVE_REPEAT:
          if (HIWORD(wParam) != BN_CLICKED)
            return FALSE;
          if ((HWND)lParam != hRemoveRepeat)
            return FALSE;
          if (SendMessage(hRemoveRepeat, BM_GETCHECK, 0, 0) != BST_CHECKED)
          {
            SendMessage(hRemoveRepeatSmart, BM_SETCHECK, BST_UNCHECKED, 0);
            EnableWindow(hRemoveRepeatSmart, FALSE);
          }
          else
          {
            EnableWindow(hRemoveRepeatSmart, TRUE);
          }

          return FALSE;

        case IDOK:

          idx = SendMessage(hCharSet, CB_GETCURSEL, 0, 0);
          idx = SendMessage(hCharSet, CB_GETITEMDATA, idx, 0);
          lpSettingTxtWin->iCharSet = idx;

          SendMessage(lpSettingTxtWin->hwndEdit, WM_SETFONT, (WPARAM)hFont,
            TRUE);
          DeleteObject(lpSettingTxtWin->hFont);
          lpSettingTxtWin->hFont = hFont;

          UnloadTRMod(&lpSettingTxtWin->TransCtxt);
          lpSettingTxtWin->TransCtxt = TransCtxt;

          lpSettingTxtWin->bPrintInvisibleText = FALSE;
          lpSettingTxtWin->bUpdateOrg = FALSE;
          lpSettingTxtWin->bAddNewLine = FALSE;
          lpSettingTxtWin->bDontRemoveRepeat = FALSE;
          lpSettingTxtWin->bDontRemoveRepeatSmart = FALSE;
          lpSettingTxtWin->bRemoveRepeatSentence = FALSE;
          lpSettingTxtWin->bChangeFontSize = FALSE;

          if (SendMessage(hPrintInvisibleText , BM_GETCHECK, 0, 0) ==
            BST_CHECKED)
            lpSettingTxtWin->bPrintInvisibleText = TRUE;
          if (SendMessage(hAddNewLine, BM_GETCHECK, 0, 0) == BST_CHECKED)
            lpSettingTxtWin->bAddNewLine = TRUE;
          if (SendMessage(hRemoveRepeat, BM_GETCHECK, 0, 0) != BST_CHECKED)
            lpSettingTxtWin->bDontRemoveRepeat = TRUE;
          if (SendMessage(hRemoveRepeatSmart, BM_GETCHECK, 0, 0) !=
            BST_CHECKED)
            lpSettingTxtWin->bDontRemoveRepeatSmart = TRUE;
          if (SendMessage(hRemoveRepeatSentence, BM_GETCHECK, 0, 0) ==
            BST_CHECKED)
            lpSettingTxtWin->bRemoveRepeatSentence = TRUE;
          if (SendMessage(hDontChangeFontSize , BM_GETCHECK, 0, 0) !=
            BST_CHECKED)
            lpSettingTxtWin->bChangeFontSize = TRUE;
          if (SendMessage(hUpdateOrg, BM_GETCHECK, 0, 0) == BST_CHECKED)
          {
            lpSettingTxtWin->bStop = 1;

            CheckMenuItem(hMenu, IDM_OH_LOG_START,MF_UNCHECKED);
            CheckMenuItem(hMenu, IDM_OH_LOG_STOP, MF_CHECKED);

            if (lpGlobalTxtWin)
              lpGlobalTxtWin->bUpdateOrg = 0;
            lpGlobalTxtWin = lpSettingTxtWin;
            lpGlobalTxtWin->bUpdateOrg = TRUE;
          }
          if (SendMessage(hApplyToAll, BM_GETCHECK, 0, 0) == BST_CHECKED)
            EnumChildWindows(hwndClient, ApplyAllEnumProc, 0);

          EndDialog(hDlg, 0);
          return TRUE;
        case IDCANCEL:
          if (hFont)
            DeleteObject(hFont);
          EndDialog(hDlg, 0);
          return TRUE;
      }
      break;
  }
  return FALSE;
}

int OthMain(HINSTANCE hInstance)
{
  HACCEL hAccel;
  WNDCLASS wndclass;
  MSG msg;
  int ret;

  hInst = hInstance;

  wndclass.style = CS_HREDRAW | CS_VREDRAW;
  wndclass.lpfnWndProc = FrameWndProc;
  wndclass.cbClsExtra = 0;
  wndclass.cbWndExtra = 0;
  wndclass.hInstance = hInstance;
  wndclass.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_OH));
  wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
  wndclass.hbrBackground = (HBRUSH)(COLOR_APPWORKSPACE + 1);
  wndclass.lpszMenuName = NULL;
  wndclass.lpszClassName = szFrameClass;

  RegisterClass(&wndclass);

  wndclass.style = CS_HREDRAW | CS_VREDRAW;
  wndclass.lpfnWndProc = ChildWndProc;
  wndclass.cbClsExtra = 0;
  wndclass.cbWndExtra = sizeof(LPVOID);
  wndclass.hInstance = hInstance;
  wndclass.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_OH));
  wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
  wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
  wndclass.lpszMenuName = NULL;
  wndclass.lpszClassName = szChildClass;

  RegisterClass(&wndclass);

  hMenu = LoadMenu(hInstance, MAKEINTRESOURCE(IDR_MENU_OH));
  OH_ASSERT(hMenu != 0);
  hChildMenu = GetSubMenu(hMenu, 3);
  OH_ASSERT(hChildMenu != 0);

  hAccel = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDR_ACCELERATOR_OH));
  OH_ASSERT(hAccel != 0);

  hwndFrame =
    CreateWindow(szFrameClass, szAppName,
      WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN, CW_USEDEFAULT, CW_USEDEFAULT,
      CW_USEDEFAULT, CW_USEDEFAULT, NULL, hMenu, hInstance, NULL);
  OH_ASSERT(hwndFrame != 0);

  hwndClient = GetWindow(hwndFrame, GW_CHILD);
  OH_ASSERT(hwndClient != 0);

  ShowWindow(hwndFrame, SW_SHOWNORMAL);
  UpdateWindow(hwndFrame);

  HookingThreadId = GetCurrentThreadId();
  OTHLOCKINIT();

  ret = OthTxtHookInit(hInstance);
  OH_ASSERT(ret == 0);

  {
    TCHAR modfile[OH_MAXFILELEN];
    TCHAR *dirend;
    GetModuleFileName(hInst, modfile, OH_MAXFILELEN);
    GetFullPathName(modfile, OH_MAXFILELEN, szPluginDir, &dirend);
    *(dirend - 1) = 0;
    if (lstrlen(szPluginDir) > 1000)
      goto finishreturn;
    lstrcat(szPluginDir, TEXT("\\..\\plugins\\"));
    lstrcpy(szPluginDll, szPluginDir);
    lstrcat(szPluginDll, TEXT("*.dll"));
  }

  while (GetMessage(&msg, NULL, 0, 0))
  {
    if (!TranslateMDISysAccel(hwndClient, &msg) &&
      !TranslateAccelerator(hwndFrame, hAccel, &msg))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
  }

finishreturn:

  OTHLOCKDEL();

  return msg.wParam;
}

int OthBeginHookTextA(HDC hdc, OTHSTR *inStr, OTHWSTR *outStr)
{
  OTHAWSTR awstr;

  awstr.inStr = inStr;
  awstr.outStr = outStr;
  awstr.bCharConv = 0;
  outStr->lpStr = NULL;
  outStr->iLen = 0;

  if (inStr->iLen <= 0)
    return 0;

  if (GetCurrentThreadId() != HookingThreadId)
  {
    OTHLOCK();

    EnumChildWindows(hwndClient, SendTextEnumProcA, (LPARAM)&awstr);

    OTHUNLOCK();
  }

  if (outStr->lpStr == NULL)
  {
    int ret;
    int codepage;
    CHARSETINFO csinfo;
    BOOL bret = FALSE;
    if (hdc)
    {
      bret =
        TranslateCharsetInfo((DWORD *)GetTextCharset(hdc), &csinfo,
          TCI_SRCCHARSET);
    }
    if (bret)
      codepage = csinfo.ciACP;
    else
      codepage = CP_OEMCP;

    outStr->iLen =
      MultiByteToWideChar(codepage, 0, inStr->lpStr, inStr->iLen, 0, 0);
    OH_ASSERT(outStr->iLen != 0);
    outStr->lpStr = malloc(outStr->iLen << 1);
    OH_ASSERT(outStr->lpStr != NULL);
    ret =
      MultiByteToWideChar(codepage, 0, inStr->lpStr, inStr->iLen,
        outStr->lpStr, outStr->iLen);
    OH_ASSERT(ret != 0);
  }

  return 0;
}

int OthBeginHookTextW(HDC hdc, OTHWSTR *inStr, OTHWSTR *outStr)
{
  OTHWWSTR wwstr;

  wwstr.inStr = inStr;
  wwstr.outStr = outStr;
  wwstr.bCharConv = 0;
  outStr->lpStr = NULL;
  outStr->iLen = 0;

  if (inStr->iLen <= 0)
    return 0;

  if (GetCurrentThreadId() != HookingThreadId)
  {
    OTHLOCK();

    EnumChildWindows(hwndClient, SendTextEnumProcW, (LPARAM)&wwstr);

    OTHUNLOCK();
  }

  if (outStr->lpStr == NULL)
    *outStr = *inStr;

  return 0;
}

int OthEndHookTextA(HDC hdc, OTHSTR *inStr, OTHWSTR *outStr)
{
  free(outStr->lpStr);

  return 0;
}

int OthEndHookTextW(HDC hdc, OTHWSTR *inStr, OTHWSTR *outStr)
{
  if (outStr->lpStr != inStr->lpStr)
    free(outStr->lpStr);

  return 0;
}

static HFONT LockedHookDC(HDC hdc)
{
  if (lpGlobalTxtWin && lpGlobalTxtWin->bUpdateOrg)
  {
    if (lpGlobalTxtWin->bChangeFontSize && lpGlobalTxtWin->hFont)
      SelectObject(hdc, lpGlobalTxtWin->hFont);
    else
    {
      LOGFONT lf1, lf2;
      HFONT hFont1, hFont2;

      hFont1 = GetCurrentObject(hdc, OBJ_FONT);
      OH_ASSERT(hFont1);
      GetObject(hFont1, sizeof(LOGFONT), &lf1);
      if (lpGlobalTxtWin->hFont)
      {
        GetObject(lpGlobalTxtWin->hFont, sizeof(LOGFONT), &lf2);
        lf1.lfCharSet = lf2.lfCharSet;
        lstrcpy(lf1.lfFaceName, lf2.lfFaceName);
      }
      else
      {
        lf1.lfCharSet = lpGlobalTxtWin->iCharSet;
      }
      hFont2 = CreateFontIndirect(&lf1);
      OH_ASSERT(hFont2);
      SelectObject(hdc, hFont2);

      return hFont2;
    }
  }
  return NULL;
}

HFONT OthHookDC(HDC hdc)
{
  HFONT hFont = NULL;

  if (GetCurrentThreadId() != HookingThreadId)
  {
    OTHLOCK();

    hFont = LockedHookDC(hdc);

    OTHUNLOCK();
  }
  return hFont;
}

int OthHookCodePage(UINT *CodePage)
{
  if (GetCurrentThreadId() != HookingThreadId)
  {
    OTHLOCK();

    if (lpGlobalTxtWin && lpGlobalTxtWin->bUpdateOrg)
    {
      CHARSETINFO csinfo;
      BOOL bret;

      bret =
        TranslateCharsetInfo((DWORD *)lpGlobalTxtWin->iCharSet, &csinfo,
          TCI_SRCCHARSET);
      if (bret)
        *CodePage = csinfo.ciACP;
      else
        *CodePage = CP_OEMCP;
    }

    OTHUNLOCK();
  }
  return 0;
}

extern int OthHookCharSet(UINT *CharSet)
{
  if (GetCurrentThreadId() != HookingThreadId)
  {
    OTHLOCK();

    if (lpGlobalTxtWin && lpGlobalTxtWin->bUpdateOrg)
      *CharSet = lpGlobalTxtWin->iCharSet;

    OTHUNLOCK();
  }
  return 0;
}

int OthHookWideCharToMultiByte(UINT CodePage, DWORD dwFlags,
  LPCWSTR lpWideCharStr, int cchWideChar, LPSTR lpMultiByteStr,
  int cbMultiByte, LPCSTR lpDefaultChar, LPBOOL lpUsedDefaultChar)
{
  int ret;
  BOOL bret;
  CHARSETINFO csinfo;

  if (GetCurrentThreadId() != HookingThreadId)
  {
    OTHLOCK();

    if (lpGlobalTxtWin && lpGlobalTxtWin->bUpdateOrg)
    {
      bret = TranslateCharsetInfo((DWORD *)lpGlobalTxtWin->iCharSet, &csinfo,
        TCI_SRCCHARSET);
      if (bret)
        CodePage = csinfo.ciACP;
      else
        CodePage = CP_OEMCP;
    }
    ret =
      WideCharToMultiByte(CodePage, dwFlags, lpWideCharStr, cchWideChar,
        lpMultiByteStr, cbMultiByte, lpDefaultChar, lpUsedDefaultChar);
    if (ret > 0 && cchWideChar > 0 && cbMultiByte > 0)
    {
      OTHWWSTR wwstr;
      OTHWSTR inStr;
      OTHWSTR outStr;

      inStr.lpStr = (LPWSTR)lpWideCharStr;
      inStr.iLen = cchWideChar;
      outStr.lpStr = NULL;
      outStr.iLen = 0;
      wwstr.inStr = &inStr;
      wwstr.outStr = &outStr;
      wwstr.bCharConv = 1;

      EnumChildWindows(hwndClient, SendTextEnumProcW, (LPARAM)&wwstr);

      if (inStr.lpStr != outStr.lpStr)
        free(outStr.lpStr);
    }

    OTHUNLOCK();
  }
  else
  {
    ret =
      WideCharToMultiByte(CodePage, dwFlags, lpWideCharStr, cchWideChar,
        lpMultiByteStr, cbMultiByte, lpDefaultChar, lpUsedDefaultChar);
  }

  return ret;
}

int OthHookMultiByteToWideChar(UINT CodePage, DWORD dwFlags,
  LPCSTR lpMultiByteStr, int cbMultiByte, LPWSTR lpWideCharStr,
  int cchWideChar)
{
  int ret;
  BOOL bret;
  CHARSETINFO csinfo;

  if (GetCurrentThreadId() != HookingThreadId)
  {
    OTHLOCK();

    if (lpGlobalTxtWin && lpGlobalTxtWin->bUpdateOrg)
    {
      bret = TranslateCharsetInfo((DWORD *)lpGlobalTxtWin->iCharSet, &csinfo,
        TCI_SRCCHARSET);
      if (bret)
        CodePage = csinfo.ciACP;
      else
        CodePage = CP_OEMCP;
    }
    ret =
      MultiByteToWideChar(CodePage, dwFlags, lpMultiByteStr, cbMultiByte,
        lpWideCharStr, cchWideChar);
    if (ret > 0 && cbMultiByte > 0 && cchWideChar > 0)
    {
      OTHAWSTR awstr;
      OTHSTR inStr;
      OTHWSTR outStr;
      int i;

      inStr.lpStr = (LPSTR)lpMultiByteStr;
      inStr.iLen = cbMultiByte;
      outStr.lpStr = NULL;
      outStr.iLen = 0;
      awstr.inStr = &inStr;
      awstr.outStr = &outStr;
      awstr.bCharConv = 1;

      EnumChildWindows(hwndClient, SendTextEnumProcA, (LPARAM)&awstr);

      if (outStr.lpStr)
      {
        if (lpGlobalTxtWin && lpGlobalTxtWin->bUpdateOrg)
        {
          for (i = 0; i < ret; i++)
          {
            lpWideCharStr[i] = outStr.lpStr[i];
          }
        }
        free(outStr.lpStr);
      }
    }

    OTHUNLOCK();
  }
  else
  {
    ret =
      MultiByteToWideChar(CodePage, dwFlags, lpMultiByteStr, cbMultiByte,
        lpWideCharStr, cchWideChar);
  }

  return ret;
}

static BOOL CALLBACK SendTextEnumProcA(HWND hwnd, LPARAM lParam)
{
  OTHTXTWIN *txtwin;
  OTHAWSTR *awstr;
  OTHWWSTR wwstr;
  OTHWSTR inStr, outStr;
  UINT codepage;
  int ret;
  BOOL bret;
  HWND hwndEdit;
  CHARSETINFO csinfo;

  hwndEdit = GetDlgItem(hwnd, IDM_OH_EDITBOX);
  if (hwndEdit == NULL) /* hack */
    return TRUE;

  txtwin = (OTHTXTWIN *)GetWindowLong(hwnd, 0);

  awstr = (OTHAWSTR *)lParam;

  if (awstr->bCharConv && !txtwin->bPrintInvisibleText)
    return TRUE;

  wwstr.inStr = &inStr;
  wwstr.outStr = &outStr;
  wwstr.bCharConv = 0;

  bret =
    TranslateCharsetInfo((DWORD *)txtwin->iCharSet, &csinfo, TCI_SRCCHARSET);
  if (bret)
    codepage = csinfo.ciACP;
  else
    codepage = CP_OEMCP;

  wwstr.inStr->iLen =
    MultiByteToWideChar(codepage, 0, awstr->inStr->lpStr, awstr->inStr->iLen,
      0, 0);

  OH_ASSERT(wwstr.inStr->iLen != 0);

  wwstr.inStr->lpStr = _alloca(wwstr.inStr->iLen << 1);

  OH_ASSERT(wwstr.inStr->lpStr != NULL);

  ret =
    MultiByteToWideChar(codepage, 0, awstr->inStr->lpStr, awstr->inStr->iLen,
      wwstr.inStr->lpStr, wwstr.inStr->iLen);

  OH_ASSERT(ret != 0);

  bret = SendTextEnumProcW(hwnd, (LPARAM)&wwstr);

  if (txtwin->bUpdateOrg)
    *awstr->outStr = *wwstr.outStr;

  return bret;
}

static BOOL CALLBACK SendTextEnumProcW(HWND hwnd, LPARAM lParam)
{
  OTHTXTWIN *txtwin;
  OTHWWSTR *wwstr;
  HWND hwndEdit;
  LPWSTR editStr;
  int editLen;
  OTHWSTR outStr;
  int dummyMsg = 0;

  hwndEdit = GetDlgItem(hwnd, IDM_OH_EDITBOX);
  if (hwndEdit == NULL) /* hack */
    return TRUE;

  txtwin = (OTHTXTWIN *)GetWindowLong(hwnd, 0);

  wwstr = (OTHWWSTR *)lParam;

  if (wwstr->bCharConv && !txtwin->bPrintInvisibleText)
    return TRUE;

  outStr = *wwstr->inStr;

  if (outStr.lpStr[0] == 0)
    dummyMsg = 1;

  if (!txtwin->bDontRemoveRepeat && dummyMsg == 0)
  {
    int len = outStr.iLen << 1;
    if (txtwin->LastStr.lpStr && txtwin->LastStr.iLen == outStr.iLen &&
      memcmp(txtwin->LastStr.lpStr, outStr.lpStr, len) == 0)
    {
      if (txtwin->bDontRemoveRepeatSmart)
      {
        return TRUE;
      }
      else
      {
        if (txtwin->iRepeatCount + 1 != OH_RT_SIZE)
        {
          txtwin->iRepeatCount++;
        }
        if (txtwin->iRepeatCount % (txtwin->iMaxRepeatIndex + 1) != 0)
        {
          return TRUE;
        }
      }
    }
    else
    {
      if (txtwin->iRepeatTable[txtwin->iRepeatCount] != 0x7fffffff &&
        txtwin->iRepeatCount != 0)
      {
        txtwin->iRepeatTable[txtwin->iRepeatCount]++;
      }
      if (txtwin->iRepeatTable[txtwin->iRepeatCount] >
        txtwin->iRepeatTable[txtwin->iMaxRepeatIndex])
      {
        txtwin->iMaxRepeatIndex = txtwin->iRepeatCount;
      }
      txtwin->iRepeatCount = 0;
      txtwin->LastStr.lpStr =
        realloc(txtwin->LastStr.lpStr, len);
      txtwin->LastStr.iLen = outStr.iLen;
      CopyMemory(txtwin->LastStr.lpStr, outStr.lpStr, len);
      if (txtwin->LastStr.lpStr)
      {
        txtwin->LastStr.lpStr =
          realloc(txtwin->LastStr.lpStr, len);
        txtwin->LastStr.iLen = outStr.iLen;
        CopyMemory(txtwin->LastStr.lpStr, outStr.lpStr, len);
      }
      else
      {
        txtwin->LastStr.lpStr = malloc(len);
        txtwin->LastStr.iLen = outStr.iLen;
        CopyMemory(txtwin->LastStr.lpStr, outStr.lpStr, len);
        OH_ASSERT(txtwin->LastStr.lpStr);
      }
    }
  }

  if (txtwin->bRemoveRepeatSentence && dummyMsg == 0)
  {
    LPWSTR lpStr;
    int iLen;

    iLen = (outStr.iLen + 1) << 1;
    lpStr = _alloca(iLen);
    CopyMemory(lpStr, outStr.lpStr, iLen - 2);
    lpStr[outStr.iLen] = 0;
    lpStr = OthGetPrintText(&txtwin->RepTextBuf, lpStr);
    if (lpStr == NULL)
    {
      outStr.lpStr = L"";
      outStr.iLen = 0;
    }
    else
    {
      outStr.lpStr = lpStr;
      outStr.iLen = lstrlenW(lpStr);
    }
  }

  if (txtwin->TransCtxt.Mod && !txtwin->bStop)
  {
    LPWSTR lpStr;
    int iLen;

    WPARAM startSelPos;
    LPARAM endSelPos;
    LPWSTR selStr = NULL;
    int totalLen;

    SendMessageW(hwndEdit, EM_GETSEL, (WPARAM)&startSelPos,
      (LPARAM)&endSelPos);
    totalLen = GetWindowTextLengthW(hwndEdit);
    if ((LPARAM)startSelPos != (LPARAM)endSelPos)
    {
      selStr = malloc((totalLen + 1) << 1);
      GetWindowTextW(hwndEdit, selStr, totalLen + 1);
      selStr[endSelPos] = 0;
      txtwin->TransCtxt.Setting.lpSelStr = selStr + startSelPos;
    }
    else
    {
      selStr = malloc(2);
      selStr[0] = 0;
      txtwin->TransCtxt.Setting.lpSelStr = selStr;
    }

    txtwin->TransCtxt.Translate(&txtwin->TransCtxt.Setting, outStr.lpStr,
      outStr.iLen);
    free(selStr);
    lpStr = txtwin->TransCtxt.Setting.lpStr;
    iLen = txtwin->TransCtxt.Setting.iLen;
    outStr.lpStr = _alloca(iLen << 1);
    OH_ASSERT(outStr.lpStr);
    outStr.iLen = iLen;
    CopyMemory(outStr.lpStr, lpStr, iLen << 1);
  }

  if (!txtwin->bAddNewLine || outStr.iLen == 0)
  {
    /* 1 char for 0x0000 ( null char ) */
    editStr = malloc((outStr.iLen + 1) << 1);
    OH_ASSERT(editStr != NULL);
    CopyMemory(editStr, outStr.lpStr, outStr.iLen << 1);
    editStr[outStr.iLen] = 0x0000;
  }
  else
  {
    /* 1 char for 0x000d ( carriage return ) */
    /* 1 char for 0x000a ( line feed ) */
    /* 1 char for 0x0000 ( null char ) */
    editStr = malloc((outStr.iLen + 3) << 1);
    OH_ASSERT(editStr != NULL);
    CopyMemory(editStr, outStr.lpStr, outStr.iLen << 1);
    editStr[outStr.iLen] = 0x000d;
    editStr[outStr.iLen + 1] = 0x000a;
    editStr[outStr.iLen + 2] = 0x0000;
  }

  if (!txtwin->bStop && lstrlenW(editStr) > 0)
  {
    editLen = GetWindowTextLengthW(hwndEdit);
    SendMessageW(hwndEdit, EM_SETSEL, editLen, editLen);
    SendMessageW(hwndEdit, EM_REPLACESEL, 0, (LPARAM)editStr);
  }

  if (txtwin->bUpdateOrg)
  {
    wwstr->outStr->lpStr = editStr;
    wwstr->outStr->iLen = outStr.iLen;
  }
  else
  {
    free(editStr);
    *wwstr->outStr = *wwstr->inStr;
  }

  return TRUE;
}
