#include <windows.h>
#include <commctrl.h>
#include <tlhelp32.h>
#include <tchar.h>
#include <stdio.h>

#include "oh_res.h"
#include "oh_port.h"
#include "oh_debug.h"
#include "oh_ver.h"
#include "oh_dlinj.h"
#include "oh_fs.h"

#define _OH_MAXCAPLEN   128
#define _OH_MAXPIDLEN   32

#define IDT_OH_TIMER 5000

extern oh_int oh_io_init(oh_void);

static oh_int _oh_port_init(oh_void)
{

#ifdef OH_DEBUG
  oh_io_init();
#endif /* OH_DEBUG */

  return 0;
}

static HINSTANCE hInst;

static TCHAR _oh_run_exe[OH_FS_MAXPATHLEN];
static TCHAR _oh_run_arg[OH_FS_MAXPATHLEN];
static TCHAR _oh_run_work[OH_FS_MAXPATHLEN];

static oh_int _oh_update_proc_list(HWND lview);
static BOOL CALLBACK _oh_main_dlg_proc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam);
static BOOL CALLBACK _oh_about_dlg_proc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam);
static BOOL CALLBACK _oh_run_dlg_proc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam);

static oh_int _oh_update_proc_list(HWND lview)
{
  HANDLE hProcessSnap = NULL;
  PROCESSENTRY32 pe32;
  oh_int ret = -1;
  oh_int sel;
  TCHAR pidstr[_OH_MAXPIDLEN];

  ZeroMemory(&pe32, sizeof(pe32));

  hProcessSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);

  if (hProcessSnap == INVALID_HANDLE_VALUE)
    return -1;

  pe32.dwSize = sizeof(PROCESSENTRY32);

  sel = ListView_GetNextItem(lview, -1, LVNI_FOCUSED | LVNI_SELECTED);
  pidstr[0] = 0;
  if (sel != -1)
    ListView_GetItemText(lview, sel, 0, pidstr, _OH_MAXPIDLEN);

  ListView_DeleteAllItems(lview);

  if (Process32First(hProcessSnap, &pe32))
  {
    LV_ITEM lvitem;
    ZeroMemory(&lvitem, sizeof(lvitem));
    do
    {
      TCHAR strbuf[_OH_MAXPIDLEN];

#ifdef UNICODE
      _snwprintf(strbuf, _OH_MAXPIDLEN, TEXT("%u"), pe32.th32ProcessID);
#else /* UNICODE */
      _snprintf(strbuf, _OH_MAXPIDLEN, TEXT("%u"), pe32.th32ProcessID);
#endif /* UNICODE */

      lvitem.mask = LVIF_TEXT;
      lvitem.iItem = 0;
      lvitem.iSubItem = 0;
      lvitem.pszText = strbuf;
      ListView_InsertItem(lview, &lvitem);
      lvitem.iSubItem = 1;
      lvitem.pszText = pe32.szExeFile;
      ListView_SetItem(lview, &lvitem);
      if (lstrcmp(strbuf, pidstr) == 0)
      {
        ListView_SetItemState(lview, lvitem.iItem,
          LVIS_FOCUSED | LVIS_SELECTED, LVIS_FOCUSED | LVIS_SELECTED);
      }
      lvitem.iItem++;
    }
    while (Process32Next(hProcessSnap, &pe32));
    ret = 0;
  }
  else
    ret = -1;

  CloseHandle(hProcessSnap);

  return ret;
}

static BOOL CALLBACK _oh_main_dlg_proc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam)
{
  oh_int ret;
  LV_COLUMN lvcol;
  oh_int sel;
  TCHAR pidstr[_OH_MAXPIDLEN];
  oh_int pid;
  static HWND lview;

  switch (message)
  {
    case WM_INITDIALOG:
      oh_db_log((OL_DBG, OD_EVENT, "MAINDLG WM_INITDIALOG"));
      ret = SetWindowTextA(hDlg, oh_vr_getInfo(OH_VR_TITLE));
      OH_ASSERT(ret != 0);
      lview = GetDlgItem(hDlg, IDC_OH_PROC_LIST);
      OH_ASSERT(lview != 0);
      ListView_SetExtendedListViewStyle(lview, LVS_EX_FULLROWSELECT);
      ZeroMemory(&lvcol, sizeof(lvcol));
      lvcol.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_SUBITEM;
      lvcol.cx = 80;
      lvcol.pszText = TEXT("Process ID");
      ListView_InsertColumn(lview, 0, &lvcol);
      lvcol.cx = 200;
      lvcol.pszText = TEXT("Process Name");
      ListView_InsertColumn(lview, 1, &lvcol);
      ret = _oh_update_proc_list(lview);
      OH_ASSERT(ret == 0);

      return TRUE;
    case WM_NOTIFY:
      switch (LOWORD(wParam))
      {
        case IDC_OH_PROC_LIST:
        if (((LPNMHDR)lParam)->code == (oh_uint)NM_DBLCLK)
        {
          sel = ListView_GetNextItem(lview, -1, LVNI_FOCUSED | LVNI_SELECTED);
          if (sel == -1)
            return FALSE;
          ListView_GetItemText(lview, sel, 0, pidstr, _OH_MAXPIDLEN);
          pid = _ttoi(pidstr);
          ret = EndDialog(hDlg, 0);
          OH_ASSERT(ret != 0);
          oh_di_injectDLLPID(pid, TEXT("ohthkdll.dll"));
          return TRUE;
        }
      }
      break;

    case WM_SIZE:
    case WM_SIZING:
      SetWindowPos(hDlg, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
      break;

    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDC_BUTTON_OH_ATTACH:
          sel = ListView_GetNextItem(lview, -1, LVNI_FOCUSED | LVNI_SELECTED);
          if (sel == -1)
            return FALSE;
          ListView_GetItemText(lview, sel, 0, pidstr, _OH_MAXPIDLEN);
          pid = _ttoi(pidstr);
          ret = EndDialog(hDlg, 0);
          OH_ASSERT(ret != 0);
          oh_di_injectDLLPID(pid, TEXT("ohthkdll.dll"));
          return TRUE;
        case IDC_BUTTON_OH_REFRESH:
          _oh_update_proc_list(lview);
          return TRUE;
        case IDC_BUTTON_OH_RUN:
          ret =
            DialogBox(hInst, MAKEINTRESOURCE(IDD_OH_RUN), hDlg,
              _oh_run_dlg_proc);

          if (ret == IDC_BUTTON_OH_RUN && _oh_run_exe[0] != 0)
          {
            ret = EndDialog(hDlg, 0);
            OH_ASSERT(ret != 0);
            oh_di_injectDLL(_oh_run_exe, _oh_run_arg, _oh_run_work,
              TEXT("ohthkdll.dll"));
            return TRUE;
          }
          return TRUE;
        case IDC_BUTTON_OH_EXIT:
        case IDCANCEL:
          ret = EndDialog(hDlg, 0);
          OH_ASSERT(ret != 0);
          return TRUE;
        case IDC_BUTTON_OH_ABOUT:
          ret =
            DialogBox(hInst, MAKEINTRESOURCE(IDD_OH_ABOUT), hDlg,
              _oh_about_dlg_proc);
          OH_ASSERT(ret != -1);
          return TRUE;
      }
      break;
  }
  return FALSE;
}

static BOOL CALLBACK _oh_about_dlg_proc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam)
{
  HWND hStatic;
  oh_int ret;
  oh_char capbuf[_OH_MAXCAPLEN];

  switch (message)
  {
    case WM_INITDIALOG:
      oh_db_log((OL_DBG, OD_EVENT, "ABOUTDLG WM_INITDIALOG"));

      SetWindowPos(hDlg, HWND_TOPMOST, 0,0,0,0, SWP_NOSIZE | SWP_NOMOVE);

      _snprintf(capbuf, _OH_MAXCAPLEN, "About %s",
        oh_vr_getInfo(OH_VR_TITLE));
      ret = SetWindowTextA(hDlg, capbuf);
      OH_ASSERT(ret != 0);

      hStatic = GetDlgItem(hDlg, IDC_OH_ABOUT_VERSION);
      OH_ASSERT(hStatic != NULL);
      ret = SetWindowTextA(hStatic, oh_vr_getInfo(OH_VR_VERSION));
      OH_ASSERT(ret != 0);

      hStatic = GetDlgItem(hDlg, IDC_OH_ABOUT_LICENSE);
      OH_ASSERT(hStatic != NULL);
      ret = SetWindowTextA(hStatic, oh_vr_getInfo(OH_VR_LICENSE));
      OH_ASSERT(ret != 0);

      hStatic = GetDlgItem(hDlg, IDC_OH_ABOUT_DATE);
      OH_ASSERT(hStatic != NULL);
      ret = SetWindowTextA(hStatic, oh_vr_getInfo(OH_VR_DATE));
      OH_ASSERT(ret != 0);

      hStatic = GetDlgItem(hDlg, IDC_OH_ABOUT_AUTHOR);
      OH_ASSERT(hStatic != NULL);
      ret = SetWindowTextA(hStatic, oh_vr_getInfo(OH_VR_AUTHOR));
      OH_ASSERT(ret != 0);

      hStatic = GetDlgItem(hDlg, IDC_OH_ABOUT_EMAIL);
      OH_ASSERT(hStatic != NULL);
      ret = SetWindowTextA(hStatic, oh_vr_getInfo(OH_VR_EMAIL));
      OH_ASSERT(ret != 0);

      hStatic = GetDlgItem(hDlg, IDC_OH_ABOUT_HOMEPAGE);
      OH_ASSERT(hStatic != NULL);
      ret = SetWindowTextA(hStatic, oh_vr_getInfo(OH_VR_HOMEPAGE));
      OH_ASSERT(ret != 0);

      return TRUE;
    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDOK:
        case IDCANCEL:
          oh_db_log((OL_DBG, OD_EVENT, "ABOUTDLG IDOK"));
          ret = EndDialog(hDlg, 0);
          OH_ASSERT(ret != 0);
          return TRUE;
      }
      break;
  }
  return FALSE;
}

static BOOL CALLBACK _oh_run_dlg_proc(HWND hDlg, UINT message, WPARAM wParam,
  LPARAM lParam)
{
  oh_int ret;

  switch (message)
  {
    case WM_INITDIALOG:
      return TRUE;
    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDC_BUTTON_OH_EXE_BROWSE:
          {
            OPENFILENAME ofn;
            static const LPCTSTR filter = TEXT(
              "Executable Files (*.exe)\0*.exe\0All Files (*.*)\0*.*\0\0");

            GetDlgItemText(hDlg, IDC_EDIT_OH_EXE, _oh_run_exe,
              OH_FS_MAXPATHLEN);
            ZeroMemory(&ofn, sizeof(OPENFILENAME));
            ofn.lStructSize = sizeof(OPENFILENAME);
            ofn.hwndOwner = hDlg;
            ofn.lpstrFilter = filter;
            ofn.lpstrFile = _oh_run_exe;
            ofn.nMaxFile = OH_FS_MAXPATHLEN;
            ofn.Flags = OFN_NOCHANGEDIR | OFN_HIDEREADONLY;
            ofn.lpstrDefExt = TEXT("exe");
            ret = GetOpenFileName(&ofn);
            if (ret)
            {
              TCHAR *fname;
              SetDlgItemText(hDlg, IDC_EDIT_OH_EXE, _oh_run_exe);
              if (_oh_run_exe[0] != 0)
              {
                GetFullPathName(_oh_run_exe, OH_FS_MAXPATHLEN, _oh_run_work,
                  &fname);
                fname[0] = 0;
                SetDlgItemText(hDlg, IDC_EDIT_OH_WORK, _oh_run_work);
              }
            }
          }
          return TRUE;
        case IDC_BUTTON_OH_WORK_BROWSE:
          return TRUE;
        case IDC_BUTTON_OH_RUN:
          GetDlgItemText(hDlg, IDC_EDIT_OH_EXE, _oh_run_exe, OH_FS_MAXPATHLEN);
          GetDlgItemText(hDlg, IDC_EDIT_OH_ARG, _oh_run_arg, OH_FS_MAXPATHLEN);
          GetDlgItemText(hDlg, IDC_EDIT_OH_WORK, _oh_run_work,
            OH_FS_MAXPATHLEN);
          ret = EndDialog(hDlg, IDC_BUTTON_OH_RUN);
          OH_ASSERT(ret != 0);
          return TRUE;
        case IDCANCEL:
          ret = EndDialog(hDlg, 0);
          OH_ASSERT(ret != 0);
          return TRUE;
      }
      break;
  }
  return FALSE;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
  PSTR szCmdLine, int iCmdShow)
{
  _oh_port_init();

  hInst = hInstance;
  DialogBox(hInstance, MAKEINTRESOURCE(IDD_OH_MAIN), NULL, _oh_main_dlg_proc);

  return 0;
}
