#include "oh_port.h"
#include "oh_tstcs.h"
#include "oh_args.h"
#include "oh_fs.h"
#include "oh_debug.h"

static oh_int _oh_tk_compareOpts(oh_as_opts_t *opts1, oh_as_opts_t *opts2);
static oh_int _oh_tk_testArgs00001(oh_tk_opt_t *opt);
static oh_int _oh_tk_testArgs00002(oh_tk_opt_t *opt);
static oh_int _oh_tk_testArgs00003(oh_tk_opt_t *opt);
static oh_int _oh_tk_testArgs00004(oh_tk_opt_t *opt);
static oh_int _oh_tk_testArgs00005(oh_tk_opt_t *opt);

static const oh_tk_tc_t _oh_tk_list[] =
  {
    {
      "Argument Parsing Test",
      "_oh_tk_testArgs00001",
      &_oh_tk_testArgs00001
    },
    {
      "Argument Merging Test",
      "_oh_tk_testArgs00002",
      &_oh_tk_testArgs00002
    },
    {
      "Argument Parsing Failure Test",
      "_oh_tk_testArgs00003",
      &_oh_tk_testArgs00003
    },
    {
      "CFGFile Save/Load Test",
      "_oh_tk_testArgs00004",
      &_oh_tk_testArgs00004
    },
    {
      "Argument Create/Add/Remove/Find Test",
      "_oh_tk_testArgs00005",
      &_oh_tk_testArgs00005
    }
  };
static const oh_int _oh_tk_numList = sizeof(_oh_tk_list) / sizeof(oh_tk_tc_t);

typedef struct oh_tk_args_t
{
  const oh_char *args;
  const oh_char *format;
} oh_tk_args_t;

static const oh_tk_args_t _oh_tk_args[] =
  {
    { /* 0 */
      "prg0 -a arga",
      "-a?:"
    },
    { /* 1 */
      "  prg1  -b -c -d",
      "-b.:"
    },
    { /* 2 */
      "\n\nprg2\t\t-c",
      "-b.:-c.:"
    },
    { /* 3 */
      "prg3\f\n --set cc,zz,qwe,ab  --set\n aa,bb,23,dd  \t",
      "--set*:--pot.:"
    },
    { /* 4 */
      " prg4\r\v -line 11,22,33,44 ",
      "-line*:"
    },
    { /* 5 */
      "prg5\v\v  -q -h  filename1 file2\n dir3",
      "-h?:-q.:"
    },
    { /* 6 */
      "\t \t\n  prg6  garbage arg -q qw,as,z,x -q qoa\t\n",
      "-q*:-g.:-a?:"
    },
    { /* 7 */
      "prg7 -g    \n -h hello --help",
      "-g?:-h*:"
    },
    { /* 8 */
      "prg8 dummy --u \"q,w -a abb cc,e,r,,zzz\"",
      "-a?:--u*:"
    },
    { /* 9 */
      "prg9 \"  space   arg \" --ppp",
      "--pp.:"
    },
    { /* 10 */
      "prg10\v\v  -q -h  \"filename1 file2  \" \n dir3",
      "-h?:-q.:"
    },
    { /* 11 */
      "prg10 -g  \"ccc \" \n -h hello --help",
      "-g?:-h*:"
    },
    { /* 12 */
      "prg12\v\v  -q -h  \"filename1 file2 \n dir3",
      "-h?:-q.:"
    },
    { /* 13 */
      "prg13 -g  \"ccc \" \n -h  ",
      "-g?:-h*:"
    }
  };

static oh_int _oh_tk_compareOpts(oh_as_opts_t *opts1, oh_as_opts_t *opts2)
{
  OH_ASSERT(opts1 != OH_NULL);
  OH_ASSERT(opts2 != OH_NULL);

  for (; opts1 && opts2; opts1 = opts1->next, opts2 = opts2->next)
  {
    oh_int i;
    oh_int nopts;

    nopts = opts1->nopts;

    if (nopts != opts2->nopts)
      return 1;

    for (i = nopts - 1; i >= 0; i--)
    {
      if (oh_c_strcmp(opts1->opts[i], opts2->opts[i]) != 0)
        return 2;
    }
  }

  if (opts1 || opts2)
    return 3;

  return 0;
}

static oh_int _oh_tk_testArgs00001(oh_tk_opt_t *opt)
{
  oh_int argc;
  oh_char **argv = OH_NULL;
  oh_as_opts_t *opts = OH_NULL;
  oh_int ret = 1;

  /* 0 */
  argv = oh_as_parseRawArgs(_oh_tk_args[0].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[0].format, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;
  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg0") == 0 &&
    opts->next->nopts == 2 && oh_c_strcmp(opts->next->opts[0], "-a") == 0 &&
    oh_c_strcmp(opts->next->opts[1], "arga") == 0 &&
    opts->next->next == OH_NULL && argc == 1 && argv[1] == OH_NULL))
    goto cleanupret;
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);
  oh_db_log((OL_DBG, OD_TESTKIT, "parse 0 is a success"));

  /* 1 */
  argv = oh_as_parseRawArgs(_oh_tk_args[1].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[1].format, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;
  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg1") == 0 &&
    opts->next->nopts == 1 && oh_c_strcmp(opts->next->opts[0], "-b") == 0 &&
    opts->next->next == OH_NULL && argc == 3 && argv[3] == OH_NULL &&
    oh_c_strcmp(argv[0], "prg1") == 0 && oh_c_strcmp(argv[1], "-c") == 0 &&
    oh_c_strcmp(argv[2], "-d") == 0))
    goto cleanupret;
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);
  oh_db_log((OL_DBG, OD_TESTKIT, "parse 1 is a success"));

  /* 2 */
  argv = oh_as_parseRawArgs(_oh_tk_args[2].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[2].format, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;
  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg2") == 0 &&
    opts->next->nopts == 1 && oh_c_strcmp(opts->next->opts[0], "-c") == 0 &&
    opts->next->next == OH_NULL && argc == 1 && argv[1] == OH_NULL &&
    oh_c_strcmp(argv[0], "prg2") == 0))
    goto cleanupret;
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);
  oh_db_log((OL_DBG, OD_TESTKIT, "parse 2 is a success"));

  /* 3 */
  argv = oh_as_parseRawArgs(_oh_tk_args[3].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[3].format, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;
  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg3") == 0 &&
    opts->next->nopts == 5 && oh_c_strcmp(opts->next->opts[0], "--set") == 0 &&
    oh_c_strcmp(opts->next->opts[1], "aa") == 0 &&
    oh_c_strcmp(opts->next->opts[2], "bb") == 0 &&
    oh_c_strcmp(opts->next->opts[3], "23") == 0 &&
    oh_c_strcmp(opts->next->opts[4], "dd") == 0 &&
    opts->next->next == OH_NULL && argc == 1 && argv[1] == OH_NULL &&
    oh_c_strcmp(argv[0], "prg3") == 0))
    goto cleanupret;
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);
  oh_db_log((OL_DBG, OD_TESTKIT, "parse 3 is a success"));

  /* 4 */
  argv = oh_as_parseRawArgs(_oh_tk_args[4].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[4].format, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;
  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg4") == 0 &&
    opts->next->nopts == 5 && oh_c_strcmp(opts->next->opts[0], "-line") == 0 &&
    oh_c_strcmp(opts->next->opts[1], "11") == 0 &&
    oh_c_strcmp(opts->next->opts[2], "22") == 0 &&
    oh_c_strcmp(opts->next->opts[3], "33") == 0 &&
    oh_c_strcmp(opts->next->opts[4], "44") == 0 &&
    opts->next->next == OH_NULL && argc == 1 && argv[1] == OH_NULL &&
    oh_c_strcmp(argv[0], "prg4") == 0))
    goto cleanupret;
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);
  oh_db_log((OL_DBG, OD_TESTKIT, "parse 4 is a success"));

  /* 5 */
  argv = oh_as_parseRawArgs(_oh_tk_args[5].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[5].format, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;
  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg5") == 0 &&
    opts->next->nopts == 2 && oh_c_strcmp(opts->next->opts[0], "-h") == 0 &&
    oh_c_strcmp(opts->next->opts[1], "filename1") == 0 &&
    opts->next->next->nopts == 1 &&
    oh_c_strcmp(opts->next->next->opts[0], "-q") == 0 &&
    opts->next->next->next == OH_NULL && argc == 3 && argv[3] == OH_NULL &&
    oh_c_strcmp(argv[0], "prg5") == 0 && oh_c_strcmp(argv[1], "file2") == 0 &&
    oh_c_strcmp(argv[2], "dir3") == 0))
    goto cleanupret;
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);
  oh_db_log((OL_DBG, OD_TESTKIT, "parse 5 is a success"));

  /* 6 */
  argv = oh_as_parseRawArgs(_oh_tk_args[6].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[6].format, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;
  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg6") == 0 &&
    opts->next->nopts == 2 && oh_c_strcmp(opts->next->opts[0], "-q") == 0 &&
    oh_c_strcmp(opts->next->opts[1], "qoa") == 0 &&
    opts->next->next == OH_NULL && argc == 3 && argv[3] == OH_NULL &&
    oh_c_strcmp(argv[0], "prg6") == 0 &&
    oh_c_strcmp(argv[1], "garbage") == 0 && oh_c_strcmp(argv[2], "arg") == 0))
    goto cleanupret;
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);
  oh_db_log((OL_DBG, OD_TESTKIT, "parse 6 is a success"));

  /* 7 */
  argv = oh_as_parseRawArgs(_oh_tk_args[7].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[7].format, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;
  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg7") == 0 &&
    opts->next->nopts == 2 && oh_c_strcmp(opts->next->opts[0], "-g") == 0 &&
    oh_c_strcmp(opts->next->opts[1], "-h") == 0 &&
    opts->next->next == OH_NULL && argc == 3 && argv[3] == OH_NULL &&
    oh_c_strcmp(argv[0], "prg7") == 0 &&
    oh_c_strcmp(argv[1], "hello") == 0 && oh_c_strcmp(argv[2], "--help") == 0))
    goto cleanupret;
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);
  oh_db_log((OL_DBG, OD_TESTKIT, "parse 7 is a success"));

  /* 8 */
  argv = oh_as_parseRawArgs(_oh_tk_args[8].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[8].format, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;
  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg8") == 0 &&
    opts->next->nopts == 7 && oh_c_strcmp(opts->next->opts[0], "--u") == 0 &&
    oh_c_strcmp(opts->next->opts[1], "q") == 0 &&
    oh_c_strcmp(opts->next->opts[2], "w -a abb cc") == 0 &&
    oh_c_strcmp(opts->next->opts[3], "e") == 0 &&
    oh_c_strcmp(opts->next->opts[4], "r") == 0 &&
    oh_c_strcmp(opts->next->opts[5], "") == 0 &&
    oh_c_strcmp(opts->next->opts[6], "zzz") == 0 &&
    opts->next->next == OH_NULL && argc == 2 && argv[2] == OH_NULL &&
    oh_c_strcmp(argv[0], "prg8") == 0 &&
    oh_c_strcmp(argv[1], "dummy") == 0))
    goto cleanupret;
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);
  oh_db_log((OL_DBG, OD_TESTKIT, "parse 8 is a success"));

  /* 9 */
  argv = oh_as_parseRawArgs(_oh_tk_args[9].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[9].format, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;
  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg9") == 0 &&
    opts->next == OH_NULL && argc == 3 && argv[3] == OH_NULL &&
    oh_c_strcmp(argv[0], "prg9") == 0 &&
    oh_c_strcmp(argv[1], "  space   arg ") == 0 &&
    oh_c_strcmp(argv[2], "--ppp") == 0))
    goto cleanupret;
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);
  oh_db_log((OL_DBG, OD_TESTKIT, "parse 9 is a success"));

  return 0;

cleanupret:
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);

  return ret;
}

static oh_int _oh_tk_testArgs00002(oh_tk_opt_t *opt)
{
  oh_int argc;
  oh_char **argv = OH_NULL;
  oh_as_opts_t *opts1 = OH_NULL;
  oh_as_opts_t *opts2 = OH_NULL;
  oh_as_opts_t *opts = OH_NULL;
  oh_int ret = 1;

  argv = oh_as_parseRawArgs(_oh_tk_args[10].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts1 = oh_as_parseArgs(_oh_tk_args[10].format, argv, &argc);
  if (opts1 == OH_NULL)
    goto cleanupret;
  oh_as_freeArgs(argv);
  argv = oh_as_parseRawArgs(_oh_tk_args[11].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts2 = oh_as_parseArgs(_oh_tk_args[11].format, argv, &argc);
  if (opts2 == OH_NULL)
    goto cleanupret;
  oh_as_freeArgs(argv);

  opts = oh_as_mergeOpts(opts1, opts2);

  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg10") == 0 &&
    opts->next->nopts == 2 && oh_c_strcmp(opts->next->opts[0], "-g") == 0 &&
    oh_c_strcmp(opts->next->opts[1], "ccc ") == 0 &&
    opts->next->next->nopts == 2 &&
    oh_c_strcmp(opts->next->next->opts[0], "-h") == 0 &&
    oh_c_strcmp(opts->next->next->opts[1], "hello") == 0 &&
    opts->next->next->next->nopts == 1 &&
    oh_c_strcmp(opts->next->next->next->opts[0], "-q") == 0 &&
    opts->next->next->next->next == OH_NULL))
    goto cleanupret;

  oh_as_freeOpts(opts);

  return 0;

cleanupret:
  oh_as_freeOpts(opts1);
  oh_as_freeOpts(opts2);
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);

  return ret;
}

static oh_int _oh_tk_testArgs00003(oh_tk_opt_t *opt)
{
  oh_int argc;
  oh_char **argv = OH_NULL;
  oh_as_opts_t *opts = OH_NULL;
  oh_int ret = 1;

  argv = oh_as_parseRawArgs(_oh_tk_args[12].args, &argc);
  if (argv != OH_NULL)
    goto cleanupret;

  argv = oh_as_parseRawArgs(_oh_tk_args[13].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[13].format, argv, &argc);
  if (opts != OH_NULL)
    goto cleanupret;
  oh_as_freeArgs(argv);

  return 0;

cleanupret:
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);

  return ret;

}

static oh_int _oh_tk_testArgs00004(oh_tk_opt_t *opt)
{
  oh_int argc;
  oh_int i;
  oh_char **argv = OH_NULL;
  oh_as_opts_t *opts = OH_NULL, *opts2 = OH_NULL;
  oh_int ret = 1;

  for (i = 0; i < 10; i++)
  {
    argv = oh_as_parseRawArgs(_oh_tk_args[i].args, &argc);
    if (argv == OH_NULL)
      goto cleanupret;
    opts = oh_as_parseArgs(_oh_tk_args[i].format, argv, &argc);
    if (opts == OH_NULL)
      goto cleanupret;
    oh_fs_unlink("testcfg.cfg");
    if (oh_as_saveToCFG(opts, "testcfg.cfg") != 0)
      goto cleanupret;
    opts2 = oh_as_parseCFG(_oh_tk_args[i].format, "testcfg.cfg");
    if (_oh_tk_compareOpts(opts, opts2) != 0)
      goto cleanupret;
    oh_as_freeOpts(opts);
    oh_as_freeOpts(opts2);
    oh_as_freeArgs(argv);
    if (oh_fs_unlink("testcfg.cfg") != 0)
      goto cleanupret;
    oh_db_log((OL_DBG, OD_TESTKIT, "save/load CFG %d is a success", i));
  }

  return 0;

cleanupret:
  oh_as_freeOpts(opts);
  oh_as_freeOpts(opts2);
  oh_as_freeArgs(argv);

  return ret;
}

static oh_int _oh_tk_testArgs00005(oh_tk_opt_t *opt)
{
  oh_int argc;
  oh_char **argv = OH_NULL;
  oh_as_opts_t *opts = OH_NULL;
  oh_as_opts_t *newopts;
  oh_int ret = 1;

  argv = oh_as_parseRawArgs(_oh_tk_args[3].args, &argc);
  if (argv == OH_NULL)
    goto cleanupret;
  opts = oh_as_parseArgs(_oh_tk_args[3].format, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;
  if (!(opts->nopts == 1 && oh_c_strcmp(opts->opts[0], "prg3") == 0 &&
    opts->next->nopts == 5 && oh_c_strcmp(opts->next->opts[0], "--set") == 0 &&
    oh_c_strcmp(opts->next->opts[1], "aa") == 0 &&
    oh_c_strcmp(opts->next->opts[2], "bb") == 0 &&
    oh_c_strcmp(opts->next->opts[3], "23") == 0 &&
    oh_c_strcmp(opts->next->opts[4], "dd") == 0 &&
    opts->next->next == OH_NULL && argc == 1 && argv[1] == OH_NULL &&
    oh_c_strcmp(argv[0], "prg3") == 0))
    goto cleanupret;
  oh_db_log((OL_DBG, OD_TESTKIT, "argument parse is a success"));

  newopts = oh_as_newOpt(5, "-t", "args", "io", "compiler", "clib");
  if (newopts == OH_NULL)
    goto cleanupret;
  if (oh_as_addOpt(opts, newopts) != 0)
    goto cleanupret;
  if (!(opts->next->next->nopts == 5 &&
    oh_c_strcmp(opts->next->next->opts[0], "-t") == 0 &&
    oh_c_strcmp(opts->next->next->opts[1], "args") == 0 &&
    oh_c_strcmp(opts->next->next->opts[2], "io") == 0 &&
    oh_c_strcmp(opts->next->next->opts[3], "compiler") == 0 &&
    oh_c_strcmp(opts->next->next->opts[4], "clib") == 0 &&
    opts->next->next->next == OH_NULL))
    goto cleanupret;
  oh_db_log((OL_DBG, OD_TESTKIT, "argument add is a success"));

  if (!(opts->next == oh_as_findOpt(opts, "--set") &&
    opts->next->next == oh_as_findOpt(opts, "-t")))
    goto cleanupret;
  oh_db_log((OL_DBG, OD_TESTKIT, "argument find is a success"));

  newopts = oh_as_newOpt(3, "--set", "15", "93");
  if (newopts == OH_NULL)
    goto cleanupret;
  if (oh_as_addOpt(opts, newopts) != 0)
    goto cleanupret;
  if (!(opts->next == oh_as_findOpt(opts, "--set") &&
    opts->next->next == oh_as_findOpt(opts, "-t")))
    goto cleanupret;
  if (!(opts->next->nopts == 3 &&
    oh_c_strcmp(opts->next->opts[0], "--set") == 0 &&
    oh_c_strcmp(opts->next->opts[1], "15") == 0 &&
    oh_c_strcmp(opts->next->opts[2], "93") == 0))
    goto cleanupret;
  oh_db_log((OL_DBG, OD_TESTKIT, "argument replace is a success"));

  if (oh_as_removeOpt(opts, "--set") != 0)
    goto cleanupret;
  if (!(oh_c_strcmp(opts->next->opts[0], "-t") == 0))
    goto cleanupret;
  oh_db_log((OL_DBG, OD_TESTKIT, "argument remove is a success"));

  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);

  return 0;

cleanupret:
  oh_as_freeOpts(opts);
  oh_as_freeArgs(argv);

  return ret;
}

oh_int oh_tk_testArgs(oh_tk_opt_t *opt)
{
  oh_int ret = 0;

  oh_int i;

  for (i = 0; i < _oh_tk_numList; i++)
  {
    oh_int tcret;

    oh_db_log((OL_MSG, OD_TESTKIT, "%s Test ( %s )", _oh_tk_list[i].tcfuncname,
      _oh_tk_list[i].explain));
    tcret = (*_oh_tk_list[i].tcfunc)(opt);
    if (tcret == 0)
      oh_db_log((OL_MSG, OD_TESTKIT, "%s Success", _oh_tk_list[i].tcfuncname));
    else
      oh_db_log((OL_MSG, OD_TESTKIT, "%s Fail", _oh_tk_list[i].tcfuncname,
        tcret));
    ret += tcret;

    OH_ASSERT(ret >= 0); /* overflow check */
  }

  return ret;
}
