#include "oh_port.h"
#include "oh_tstcs.h"
#include "oh_debug.h"

static oh_int _oh_tk_testCompiler00001(oh_tk_opt_t *opt);
static oh_int _oh_tk_testCompiler00002(oh_tk_opt_t *opt);
static oh_int _oh_tk_testCompiler00003(oh_tk_opt_t *opt);
static oh_int _oh_tk_testCompiler00004(oh_tk_opt_t *opt);

static oh_tk_tc_t _oh_tk_list[] =
  {
    {
      "Primitive Data Type Size Test",
      "_oh_tk_testCompiler00001",
      &_oh_tk_testCompiler00001
    },
    {
      "Character Type Signed Test",
      "_oh_tk_testCompiler00002",
      &_oh_tk_testCompiler00002
    },
    {
      "Integer Type Arithmetic Right Shift Test",
      "_oh_tk_testCompiler00003",
      &_oh_tk_testCompiler00003
    },
    {
      "Integer Type Endian Test",
      "_oh_tk_testCompiler00004",
      &_oh_tk_testCompiler00004
    }
  };
static const oh_int _oh_tk_numList = sizeof(_oh_tk_list) / sizeof(oh_tk_tc_t);

static oh_int _oh_tk_testCompiler00001(oh_tk_opt_t *opt)
{
  oh_int ret = 0;

  oh_int oh_int_size;
  oh_int oh_uint_size;
  oh_int oh_long_size;
  oh_int oh_ulong_size;
  oh_int oh_char_size;
  oh_int oh_float_size;
  oh_int oh_double_size;
  oh_int oh_int32_size;
  oh_int oh_uint32_size;
  oh_int oh_int16_size;
  oh_int oh_uint16_size;
  oh_int oh_int8_size;
  oh_int oh_uint8_size;
  oh_int oh_float32_size;
  oh_int oh_float64_size;
  oh_int oh_pointer_size;

  oh_int_size = sizeof(oh_int);
  oh_uint_size = sizeof(oh_uint);
  oh_long_size = sizeof(oh_long);
  oh_ulong_size = sizeof(oh_ulong);
  oh_char_size = sizeof(oh_char);
  oh_float_size = sizeof(oh_float);
  oh_double_size = sizeof(oh_double);
  oh_int32_size = sizeof(oh_int32);
  oh_uint32_size = sizeof(oh_uint32);
  oh_int16_size = sizeof(oh_int16);
  oh_uint16_size = sizeof(oh_uint16);
  oh_int8_size = sizeof(oh_int8);
  oh_uint8_size = sizeof(oh_uint8);
  oh_float32_size = sizeof(oh_float32);
  oh_float64_size = sizeof(oh_float64);
  oh_pointer_size = sizeof(oh_void *);

  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_int) = %d", oh_int_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_uint) = %d", oh_uint_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_long) = %d", oh_long_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_ulong) = %d", oh_ulong_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_char) = %d", oh_char_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_float) = %d", oh_float_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_double) = %d", oh_double_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_int32) = %d", oh_int32_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_uint32) = %d", oh_uint32_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_int16) = %d", oh_int16_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_uint16) = %d", oh_uint16_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_int8) = %d", oh_int8_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_uint8) = %d", oh_uint8_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_float32) = %d", oh_float32_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_float64) = %d", oh_float64_size));
  oh_db_log((OL_DBG, OD_TESTKIT, "sizeof(oh_void *) = %d", oh_pointer_size));

  if (oh_int_size < 2)
  {
    oh_db_log((OL_MSG, OD_TESTKIT, "oh_int must be bigger than 2 bytes."));
    ret = 1;
  }

  if (oh_int_size != oh_uint_size)
  {
    oh_db_log((OL_MSG, OD_TESTKIT,
      "oh_int and oh_uint must be the same data size."));
    ret = 1;
  }

  if (oh_long_size < 4)
  {
    oh_db_log((OL_MSG, OD_TESTKIT, "oh_long must be bigger than 4 bytes."));
    ret = 1;
  }

  if (oh_long_size != oh_ulong_size)
  {
    oh_db_log((OL_MSG, OD_TESTKIT,
      "oh_long and oh_ulong must be the same data size."));
    ret = 1;
  }

  if (oh_char_size != 1)
  {
    oh_db_log((OL_MSG, OD_TESTKIT, "oh_char must be 1 byte."));
    ret = 1;
  }

  if (oh_int32_size != 4)
  {
    oh_db_log((OL_MSG, OD_TESTKIT, "oh_int32 must be 4 bytes."));
    ret = 1;
  }

  if (oh_int32_size != oh_uint32_size)
  {
    oh_db_log((OL_MSG, OD_TESTKIT,
      "oh_int32 and oh_uint32 must be the same data size."));
    ret = 1;
  }

  if (oh_int16_size != 2)
  {
    oh_db_log((OL_MSG, OD_TESTKIT, "oh_int16 must be 2 bytes."));
    ret = 1;
  }

  if (oh_int16_size != oh_uint16_size)
  {
    oh_db_log((OL_MSG, OD_TESTKIT,
      "oh_int16 and oh_uint16 must be the same data size."));
    ret = 1;
  }

  if (oh_int8_size != 1)
  {
    oh_db_log((OL_MSG, OD_TESTKIT, "oh_int8 must be 1 byte."));
    ret = 1;
  }

  if (oh_int8_size != oh_uint8_size)
  {
    oh_db_log((OL_MSG, OD_TESTKIT,
      "oh_int8 and oh_uint8 must be the same data size."));
    ret = 1;
  }

  if (oh_float_size < 4)
  {
    oh_db_log((OL_MSG, OD_TESTKIT, "oh_float must be bigger than 4 bytes."));
    ret = 1;
  }

  if (oh_double_size < 8)
  {
    oh_db_log((OL_MSG, OD_TESTKIT, "oh_double must be bigger than 8 bytes."));
    ret = 1;
  }

  if (oh_float32_size != 4)
  {
    oh_db_log((OL_MSG, OD_TESTKIT, "oh_float32 must be 4 bytes."));
    ret = 1;
  }

  if (oh_float64_size != 8)
  {
    oh_db_log((OL_MSG, OD_TESTKIT, "oh_float64 must be 8 bytes."));
    ret = 1;
  }

  if (oh_pointer_size < 4)
  {
    oh_db_log((OL_MSG, OD_TESTKIT,
      "oh_void pointer must be bigger than 4 bytes."));
    ret = 1;
  }

  return ret;
}

static oh_int _oh_tk_testCompiler00002(oh_tk_opt_t *opt)
{
  oh_int ret = 0;

  oh_char ch;
  oh_int ich;

  ch = '\xff';
  ich = (oh_int)ch;

  if (ich == -1)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "oh_char is signed."));
  }
  else if (ich == 255)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "oh_char is unsigned."));
  }
  else
  {
    oh_db_log((OL_MSG, OD_TESTKIT,
      "oh_char signedness has an unknown problem."));
    ret = 1;
  }

  return ret;
}

static oh_int _oh_tk_testCompiler00003(oh_tk_opt_t *opt)
{
  oh_int ret = 0;

  oh_int32 i;

  i = -1;
  i = i >> 1;

  if (i == -1)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "oh_int32 right shift is arithmetic."));
  }
  else if (i == 0x7fffffffL)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "oh_int32 right shift is logical."));
  }
  else
  {
    oh_db_log((OL_MSG, OD_TESTKIT,
      "oh_int32 shift operation has an unknown problem."));
    ret = 1;
  }

  return ret;
}

static oh_int _oh_tk_testCompiler00004(oh_tk_opt_t *opt)
{
  oh_int ret = 0;

  oh_int32 i;
  oh_uint8 *p;

  i = 0xabcdef12L;
  p = (oh_uint8 *)&i;

  if (p[0] == (oh_uint8)'\x12' && p[1] == (oh_uint8)'\xef' &&
    p[2] == (oh_uint8)'\xcd' && p[3] == (oh_uint8)'\xab')
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "oh_int32 is little endian."));
  }
  else if (p[0] == (oh_uint8)'\xab' && p[1] == (oh_uint8)'\xcd' &&
    p[2] == (oh_uint8)'\xef' && p[3] == (oh_uint8)'\x12')
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "oh_int32 is big endian."));
  }
  else
  {
    oh_db_log((OL_DBG, OD_TESTKIT,
      "oh_int32 endianess has an unknown problem."));
    ret = 1;
  }

  return ret;
}

oh_int oh_tk_testCompiler(oh_tk_opt_t *opt)
{
  oh_int ret = 0;

  oh_int i;

  for (i = 0; i < _oh_tk_numList; i++)
  {
    oh_int tcret;

    oh_db_log((OL_MSG, OD_TESTKIT, "%s Test ( %s )", _oh_tk_list[i].tcfuncname,
      _oh_tk_list[i].explain));
    tcret = (*_oh_tk_list[i].tcfunc)(opt);
    if (tcret == 0)
      oh_db_log((OL_MSG, OD_TESTKIT, "%s Success", _oh_tk_list[i].tcfuncname));
    else
      oh_db_log((OL_MSG, OD_TESTKIT, "%s Fail", _oh_tk_list[i].tcfuncname,
        tcret));
    ret += tcret;

    OH_ASSERT(ret >= 0); /* overflow check */
  }

  return ret;
}
