#include "oh_port.h"
#include "oh_tstcs.h"
#include "oh_fs.h"
#include "oh_debug.h"

static oh_int _oh_tk_createTestFile(const oh_char *path, oh_int idx,
  oh_ulong size);
static oh_ulong _oh_tk_compareTestFile(const oh_uint8 *data, oh_int idx,
  oh_ulong size);

static oh_int _oh_tk_testFS00001(oh_tk_opt_t *opt);
static oh_int _oh_tk_testFS00002(oh_tk_opt_t *opt);
static oh_int _oh_tk_testFS00003(oh_tk_opt_t *opt);
static oh_int _oh_tk_testFS00004(oh_tk_opt_t *opt);
static oh_int _oh_tk_testFS00005(oh_tk_opt_t *opt);
static oh_int _oh_tk_testFS00006(oh_tk_opt_t *opt);
static oh_int _oh_tk_testFS00007(oh_tk_opt_t *opt);
static oh_int _oh_tk_testFS00008(oh_tk_opt_t *opt);
static oh_int _oh_tk_testFS00009(oh_tk_opt_t *opt);
static oh_int _oh_tk_testFS00010(oh_tk_opt_t *opt);
static oh_int _oh_tk_testFS00011(oh_tk_opt_t *opt);

static oh_tk_tc_t _oh_tk_list[] =
  {
    {
      "File Open/Close Test",
      "_oh_tk_testFS00001",
      &_oh_tk_testFS00001
    },
    {
      "File Read/Write Test",
      "_oh_tk_testFS00002",
      &_oh_tk_testFS00002
    },
    {
      "File Lseek Test",
      "_oh_tk_testFS00003",
      &_oh_tk_testFS00003
    },
    {
      "File Ftruncate Test",
      "_oh_tk_testFS00004",
      &_oh_tk_testFS00004
    },
    {
      "File Fstat Test",
      "_oh_tk_testFS00005",
      &_oh_tk_testFS00005
    },
    {
      "File Rename Test",
      "_oh_tk_testFS00006",
      &_oh_tk_testFS00006
    },
    {
      "File Unlink Test",
      "_oh_tk_testFS00007",
      &_oh_tk_testFS00007
    },
    {
      "File Realpath Test",
      "_oh_tk_testFS00008",
      &_oh_tk_testFS00008
    },
    {
      "File Getcwd/Chdir Test",
      "_oh_tk_testFS00009",
      &_oh_tk_testFS00009
    },
    {
      "File Mkdir/Rmdir Test",
      "_oh_tk_testFS00010",
      &_oh_tk_testFS00010
    },
    {
      "File Opendir/Readdir/Closedir Test",
      "_oh_tk_testFS00011",
      &_oh_tk_testFS00011
    }
  };
static const oh_int _oh_tk_numList = sizeof(_oh_tk_list) / sizeof(oh_tk_tc_t);

static const oh_uint8 _oh_tk_fileData[] =
  {
    '\x7e', '\x53', '\xc9', '\x10', '\xd2', '\xcc', '\xee', '\xee',
    '\x1e', '\x01', '\x7c', '\x38', '\x8f', '\x8d', '\x0d', '\x0a',
    '\x5a', '\xd5', '\x0e', '\x4e', '\xd8', '\xc6', '\x47', '\x98',
    '\x41', '\x8f', '\x4d', '\x84', '\xcb', '\x2b', '\x08', '\x84',
    '\x2d', '\xa9', '\xe3', '\x0c', '\x11', '\x13', '\x2f', '\xd3',
    '\xcc', '\xbe', '\xeb', '\xd5', '\x3c', '\xe4', '\x0a', '\xab',
    '\x38', '\x28', '\x43', '\x08', '\x82', '\xd0', '\x3c', '\x59',
    '\x5b', '\x12', '\x2c', '\x37', '\x72', '\x29', '\xa3', '\x7c',
    '\xae', '\x27', '\xd1', '\x06', '\x09', '\x30', '\x1d', '\x2f',
    '\x0f', '\x8b', '\xca', '\xce', '\x54', '\x0b', '\xb9', '\x89',
    '\x43', '\xc8', '\xb0', '\x65', '\x75', '\x55', '\xac', '\x99',
    '\x53', '\xef', '\x60', '\x23', '\xc9', '\x73', '\x24', '\xbe',
    '\xe8', '\xfe', '\x6c', '\xcf', '\x7e', '\x9f', '\xce', '\xe9',
    '\x4c', '\x56', '\xf8', '\xfa', '\xcb', '\x50', '\x98', '\x3f',
    '\xb1', '\xd9', '\xe8', '\x04', '\xb2', '\x7a', '\x22', '\x26',
    '\xf3', '\xed', '\x23', '\xcd', '\xe5', '\xc2', '\xd2', '\xa1',
    '\xe3', '\x86', '\x5d', '\xc4', '\x48', '\xa7', '\x7a', '\x9a',
    '\xdf', '\xf4', '\x9b', '\xe4', '\x1e', '\x40', '\x1a', '\x7b',
    '\x39', '\x0d', '\x66', '\xd0', '\xe2', '\x02', '\x1b', '\x2e',
    '\xbb', '\xfe', '\x71', '\x09', '\x52', '\xbf', '\x0e', '\x5f',
    '\x48', '\xcf', '\x11', '\x0e', '\x39', '\xc5', '\xd3', '\xae',
    '\x62', '\x69', '\xcf', '\xcf', '\xf3', '\xd0', '\xb6', '\x1d',
    '\x18', '\xb6', '\x06', '\xc0', '\xb9', '\x41', '\xd4', '\xe1',
    '\xad', '\x11', '\x3a', '\x82', '\x3a', '\xb9', '\xeb', '\x74',
    '\xe8', '\x2f', '\xaf', '\xe1', '\x02', '\x1c', '\x5b', '\x81',
    '\xee', '\x3f', '\x65', '\xab', '\xc1', '\x90', '\x01', '\x12',
    '\x17', '\xd2', '\x78', '\x94', '\x5f', '\x04', '\x28', '\x0e',
    '\x0f', '\xda', '\x98', '\x31', '\x8b', '\x4d', '\xc5', '\xf7',
    '\x43', '\xd0', '\x10', '\xb9', '\x1c', '\xe2', '\xb9', '\x7b',
    '\x67', '\xcb', '\x92', '\x19', '\x76', '\xd7', '\xc2', '\x34',
    '\x8f', '\x19', '\xbb', '\x93', '\xd5', '\x85', '\x5f', '\xbf',
    '\x33', '\xa5', '\x08', '\xde', '\x1d', '\x14', '\xd5', '\xff',
    '\x10', '\x3d', '\xb2', '\x96', '\xa8', '\xce', '\x10', '\x32',
    '\xbf', '\x81', '\xa6', '\x68', '\x1f', '\xee', '\x0b', '\x2b',
    '\x73', '\x0c', '\x8c', '\x46', '\x59', '\x55', '\x0c', '\xd9',
    '\x3c', '\x20', '\x9e', '\x9d', '\xda', '\x56', '\xca', '\xf9',
    '\xc0', '\xf0', '\xb3', '\x3d', '\x5c', '\x88', '\x53', '\x82',
    '\x3d', '\x3f', '\xc5', '\x90', '\x8a', '\x44', '\x4c', '\x44',
    '\x54', '\x02', '\xe4', '\x3e', '\xcf', '\x4b', '\xdc', '\xbc',
    '\xe5', '\x2e', '\x3b', '\x4c', '\xdf', '\xb3', '\x5c', '\x0c',
    '\x02', '\xe0', '\xc0', '\x76', '\x63', '\x12', '\x9f', '\xb2',
    '\xc0', '\x9d', '\x9c', '\x36', '\xf0', '\x97', '\x69', '\x57',
    '\x65', '\x37', '\x65', '\xc8', '\x27', '\x73', '\x6d', '\xdc',
    '\x46', '\xae', '\xc4', '\x13', '\xa8', '\xd2', '\xe1', '\x76',
    '\x51', '\x12', '\x12', '\x46', '\x48', '\x3c', '\x8c', '\x8d',
    '\x0b', '\x39', '\x15', '\xa4', '\x98', '\x23', '\xd4', '\x9f',
    '\x85', '\xda', '\xd9', '\xb4', '\xbe', '\xfb', '\x29', '\x62',
    '\x9b', '\x51', '\x55', '\xdf', '\x33', '\x11', '\xe9', '\xe8',
    '\x72', '\x21', '\x5e', '\x33', '\xd9', '\x0d', '\x0a', '\x82',
    '\x04', '\x06', '\x0e', '\xc8', '\x7e', '\xd3', '\x9c', '\x9b',
    '\x41', '\x19', '\xa1', '\x0a', '\xca', '\x21', '\x15', '\xc9',
    '\x08', '\x61', '\x7d', '\xe1', '\x24', '\x32', '\xcc', '\xe6',
    '\xfb', '\xd3', '\xd1', '\x7e', '\x0c', '\x37', '\x2a', '\xc2',
    '\xff', '\x9b', '\x10', '\x38', '\xfa', '\x6a', '\x8b', '\xe9',
    '\xde', '\x32', '\x30', '\xd2', '\xbe', '\x21', '\xaf', '\x80',
    '\x62', '\x93', '\x6d', '\x0e', '\xef', '\x07', '\x47', '\x21',
    '\xe0', '\xa0', '\x00', '\x5f', '\xe8', '\x7c', '\x86', '\x5c',
    '\x04', '\x7e', '\x04', '\x26', '\x65', '\xb7', '\x36', '\x29',
    '\x5c', '\x7d', '\x7d', '\xc9', '\xaf', '\x23', '\xbb', '\x81',
    '\xf3', '\xd5', '\x32', '\x9a', '\xfc', '\x23', '\x0f', '\xdd',
    '\xf0', '\x47', '\xe1', '\x4a', '\x6b', '\x25', '\x88', '\x5e',
    '\x10', '\x78', '\xea', '\x66', '\xbf', '\xbc', '\x14', '\xd7',
    '\x7d', '\x81', '\x7b', '\x25', '\xba', '\x2c', '\x20', '\xf3',
    '\x41', '\x19', '\xed', '\x78', '\xcb', '\x87', '\x64', '\x47',
    '\x5f', '\x56', '\xc4', '\x27', '\x6d', '\x6b', '\x48', '\xf2',
    '\x61', '\xd9', '\x8e', '\x6b', '\x55', '\xe4', '\x97', '\x29',
    '\xca', '\xf8', '\x90', '\x50', '\x6c', '\x04', '\xbc', '\xc9',
    '\xd0', '\x26', '\x02', '\xd6', '\x21', '\x50', '\xad', '\xd6',
    '\x4f', '\xbb', '\x41', '\x91', '\x87', '\xe7', '\x5e', '\x7b',
    '\xb8', '\xd2', '\x3d', '\x3f', '\x67', '\x28', '\x41', '\xf0',
    '\x9a', '\xe7', '\x09', '\xa7', '\x27', '\x2f', '\x37', '\x5f',
    '\xf1', '\x67', '\x56', '\x9f', '\x24', '\x62', '\x0b', '\xab',
    '\x4f', '\x51', '\x37', '\x16', '\xfb', '\xfe', '\x3a', '\xb3',
    '\xa2', '\xb9', '\x7e', '\x8a', '\xd0', '\x45', '\xb5', '\x72',
    '\x1a', '\xa5', '\x91', '\x38', '\x85', '\xd1', '\xde', '\x15',
    '\x7e', '\xb2', '\x7f', '\xf4', '\x89', '\x1c', '\xed', '\xeb',
    '\xe8', '\x71', '\xbb', '\x73', '\x99', '\x51', '\x7a', '\xcb',
    '\xba', '\x5c', '\xc6', '\x89', '\xa6', '\xe7', '\xd4', '\x4d',
    '\x4f', '\xed', '\xa7', '\x80', '\xc3', '\xa1', '\x76', '\xf2',
    '\xa6', '\x21', '\xfd', '\xad', '\xb6', '\x02', '\xaa', '\x20',
    '\x1e', '\x7b', '\x12', '\xd5', '\xa3', '\x2a', '\x3f', '\x86',
    '\xf5', '\x4f', '\x2d', '\x0d', '\xea', '\xd4', '\xd1', '\x4f',
    '\x1e', '\xe8', '\x12', '\x29', '\x25', '\xd9', '\x11', '\x40',
    '\xb2', '\xd5', '\x70', '\xd6', '\xeb', '\x6a', '\x15', '\x97',
    '\xf9', '\x66', '\xa6', '\x13', '\xb6', '\xea', '\x84', '\x59',
    '\xe1', '\x38', '\x28', '\x7e', '\x22', '\x1f', '\x43', '\x59',
    '\x53', '\x41', '\x5c', '\xc6', '\x5a', '\x1e', '\xe8', '\x26',
    '\xc9', '\xcb', '\xcc', '\x30', '\x64', '\x2e', '\x10', '\xbc',
    '\x0a', '\x51', '\xfb', '\xed', '\xaf', '\x79', '\x73', '\x8b',
    '\xef', '\x1e', '\x38', '\xd2', '\x0b', '\x53', '\x41', '\x3a',
    '\xa5', '\x2d', '\x42', '\x96', '\xe4', '\x73', '\x2d', '\x4e',
    '\xb8', '\xa9', '\x91', '\xa8', '\x82', '\x56', '\x2a', '\x7a',
    '\xf0', '\x0d', '\xa7', '\x4c', '\x90', '\xaf', '\xbb', '\x4c',
    '\xbb', '\xb7', '\xa9', '\x90', '\x34', '\xa1', '\x73', '\x90',
    '\xbd', '\x7b', '\x2e', '\x3d', '\x7f', '\x24', '\xe3', '\x7c',
    '\xb9', '\x89', '\x00', '\xdd', '\xf0', '\xc8', '\x2c', '\x96',
    '\xde', '\xac', '\x45', '\x81', '\x66', '\xc8', '\xe9', '\xdc',
    '\x4c', '\xb7', '\x40', '\xcc', '\xad', '\x11', '\x2a', '\xa1',
    '\x3d', '\xa5', '\xac', '\x82', '\x84', '\xb6', '\xb3', '\x30',
    '\x35', '\xcb', '\x52', '\x98', '\xc9', '\xf8', '\xb4', '\x12',
    '\x26', '\x11', '\x6e', '\x7c', '\x31', '\xcf', '\xba', '\xa3',
    '\x5f', '\x1b', '\xf6', '\x25', '\xa8', '\xac', '\x78', '\x40',
    '\xc3', '\xda', '\xdf', '\x14', '\x52', '\xd5', '\xc4', '\x41',
    '\x8d', '\xde', '\xed', '\x4e', '\xc1', '\x90', '\xcd', '\x86',
    '\xb7', '\x7a', '\xad', '\x14', '\xc7', '\x0a', '\x71', '\x4f',
    '\xad', '\x7b', '\xac', '\xc5', '\x1e', '\xab', '\x42', '\xa4',
    '\xf2', '\x0d', '\xf8', '\x3e', '\xb2', '\x3f', '\x8d', '\x7d',
    '\xe1', '\x15', '\x9a', '\x9c', '\x43', '\x24', '\x7d', '\x81',
    '\xa8', '\x14', '\x78', '\x3c', '\xbd', '\x6c', '\x31', '\xf6',
    '\x2d', '\x55', '\xe0', '\x5c', '\x7b', '\xa8', '\x59', '\x4a',
    '\x77', '\x04', '\xad', '\xab', '\x3e', '\xb7', '\xb0', '\x49',
    '\xd3', '\x6e', '\xbc', '\xc4', '\x9f', '\xdd', '\x63', '\xe6',
    '\xbd', '\x82', '\x21', '\x46', '\x4d', '\x02', '\x4f', '\x31',
    '\x5c', '\x5c', '\x66', '\x1e', '\x3d', '\xc7', '\x90', '\xd2',
    '\x0f', '\x66', '\xad', '\x26', '\xc6', '\xee', '\xcc', '\x39',
    '\x4e', '\x63', '\x83', '\x2e', '\x2f', '\x2a', '\x4b', '\x4b',
    '\xed', '\x71', '\xd1', '\x22', '\x37', '\x5e', '\xab', '\x39',
    '\x87', '\xc6', '\x1c', '\xc5', '\xa0', '\xea', '\xce', '\xde',
    '\x8e', '\xc8', '\x15', '\x50', '\xb7', '\x74', '\x56', '\xb4',
    '\x4e', '\xbb', '\x27', '\xe4', '\x78', '\x68', '\xc1', '\x53',
    '\xe8', '\x22', '\x79', '\x97', '\xae', '\x09', '\x03', '\x0a',
    '\x09', '\x9b', '\x9f', '\x8a', '\x3d', '\xd8', '\x2b', '\xee',
    '\xe0', '\xbb', '\xeb', '\x66', '\x82', '\xa6', '\x6c', '\x8f',
    '\xa3', '\x50', '\x1b', '\x10', '\x55', '\x96', '\xaa', '\x13',
    '\x9e', '\xfa', '\xcf', '\x84', '\x38', '\xef', '\x4c', '\x79',
    '\x99', '\xff', '\x03', '\x29', '\xb9', '\x72', '\xfe', '\x41',
    '\x17', '\xdc', '\x8f', '\xfb', '\xfe', '\x99', '\xae', '\xab',
    '\xb9', '\xe7', '\x42', '\x7b', '\x1a', '\x00', '\xd8', '\x6a',
    '\xc1', '\x10', '\x3b', '\x20', '\x9a', '\xcd', '\x05', '\x5b',
    '\x8a', '\x87', '\x8a', '\xd6', '\x64', '\xdc', '\xfb', '\xad',
    '\x6c', '\xde', '\x32', '\xb7', '\xd3', '\x0e', '\x11', '\x95',
    '\x5e', '\xf6', '\x27', '\x08', '\xbc', '\xef', '\xa3', '\x62',
    '\x51', '\xba', '\xd6', '\x34', '\xb7', '\x9e', '\x8f', '\x9c',
    '\x15', '\x80', '\x56', '\x46', '\xdf', '\xad', '\x42', '\x87',
    '\x45', '\x9e', '\x4f', '\x16', '\xdc', '\x58', '\xb8', '\x22',
    '\x86', '\x76', '\x32', '\x38', '\xf5', '\x53', '\x7c', '\x8c',
    '\x1d', '\x1b', '\x58', '\x5d', '\x7d', '\x0c', '\x84', '\x60',
    '\x9c', '\x42', '\x22', '\xa2', '\xc8', '\x1d', '\x7a', '\x75',
    '\x3b', '\x29', '\x1c', '\x22', '\x91', '\x52', '\xc7', '\x0c',
    '\x14', '\xb0', '\xdf', '\xa2', '\x6e', '\x78', '\x82', '\x5f',
    '\x4b', '\xb2', '\x16', '\x44', '\xc5', '\xdc', '\x0a', '\x22',
    '\xf3', '\x7a', '\xf8', '\x8f', '\x5a', '\x33', '\xe3', '\x57',
    '\x25', '\xd3', '\x07', '\x36', '\x67', '\x46', '\x3c', '\x94',
    '\x9d', '\x0a', '\xf2', '\x74', '\xdf', '\xab', '\x1f', '\x98',
    '\xe0', '\xe7', '\xf1', '\xe3', '\x3c', '\x5c', '\x2a', '\xcf',
    '\xa3', '\x7b', '\xe5', '\x25', '\x16', '\xf2', '\x70', '\x1d',
    '\x14', '\x3e', '\x23', '\xb9', '\x3e', '\xe1', '\xc3', '\x1c',
    '\x31', '\xd7', '\xaf', '\xf1', '\x39', '\xf0', '\x93', '\x95',
    '\x41', '\x9b', '\x4e', '\xc0', '\x4c', '\xd0', '\x29', '\xe5',
    '\x33', '\x8c', '\xb1', '\xef', '\x66', '\x88', '\xdb', '\xab',
    '\x65', '\x6a', '\xa6', '\xde', '\xc0', '\x1c', '\x8d', '\xc8',
    '\x1e', '\x23', '\x11', '\xec', '\xcd', '\xac', '\x96', '\xb3',
    '\xbe', '\xb2', '\x10', '\x2a', '\xf9', '\xe4', '\xd9', '\x95',
    '\x6b', '\x32', '\x94', '\xeb', '\x4a', '\x7b', '\xba', '\xb0',
    '\xbc', '\xc5', '\x5b', '\x60', '\xc9', '\x2d', '\x25', '\x1a',
    '\xb8', '\x85', '\xfb', '\x4b', '\x6e', '\x52', '\xe3', '\xaf',
    '\x16', '\xa6', '\x87', '\x76', '\xd2', '\x13', '\xf4', '\xd9',
    '\x9b', '\x85', '\xee', '\x84', '\xf3', '\xdb', '\x96', '\x8b',
    '\x12', '\x36', '\x18', '\x41', '\xd0', '\x75', '\x45', '\x8f',
    '\x1e', '\xe5', '\x74', '\x81', '\x9a', '\x16', '\xd6', '\x01',
    '\xfa', '\x11', '\x6e', '\x42', '\xcf', '\x1f', '\x6c', '\x9b',
    '\xe4', '\x78', '\xf9', '\x95', '\x75', '\x67', '\x01', '\x27',
    '\xb6', '\x2f', '\x35', '\x88', '\x4a', '\x6d', '\xb2', '\x41',
    '\x01', '\x53', '\xd9', '\x0e', '\x9f', '\x95', '\xfe', '\x3c',
    '\x9a', '\x3e', '\xd8', '\xae', '\x39', '\x73', '\xc8', '\xcb',
    '\x78', '\x27', '\x81', '\x63', '\x62', '\xc7', '\xae', '\xd6',
    '\x57', '\xb6', '\x03', '\xfd', '\x0e', '\x93', '\x08', '\x82',
    '\x69', '\xd0', '\x14', '\xfd', '\xc2', '\x86', '\xb4', '\x69',
    '\x24', '\xb2', '\x30', '\x9d', '\x94', '\x9c', '\x76', '\x84',
    '\xe7', '\x2a', '\x9e', '\x9c', '\x7a', '\xaf', '\x82', '\x31',
    '\x06', '\x76', '\x40', '\xed', '\xc2', '\x5b', '\x9a', '\x79',
    '\x7c', '\x1c', '\xd4', '\x5e', '\x63', '\x69', '\xb3', '\x6b',
    '\x4e', '\xc1', '\x25', '\xf1', '\x7d', '\xb0', '\x1f', '\x23',
    '\x5e', '\xdf', '\x62', '\x57', '\x48', '\x14', '\xb0', '\xf7',
    '\x3e', '\xce', '\x8e', '\xe0', '\x3a', '\x1a', '\x41', '\xc5',
    '\x44', '\x85', '\xc0', '\xcd', '\x24', '\x34', '\xa9', '\x54',
    '\xdb', '\x18', '\xa8', '\xca', '\x98', '\xbf', '\x02', '\x6c',
    '\xf3', '\xb9', '\x96', '\x0e', '\xce', '\x58', '\xca', '\xec',
    '\xec', '\xdb', '\xe8', '\x5c', '\xe1', '\xf5', '\x35', '\x13',
    '\x71', '\xb7', '\xad', '\xe4', '\x11', '\xfe', '\xd1', '\xcc',
    '\x22', '\x4a', '\xde', '\xc3', '\x73', '\x40', '\x48', '\xa1',
    '\xe6', '\x8d', '\x6b', '\xa0', '\x2b', '\x75', '\xda', '\xb0',
    '\x70', '\x8a', '\x19', '\xa3', '\x1e', '\xd0', '\xe4', '\xc4',
    '\x27', '\x86', '\xd9', '\xc5', '\xc2', '\xb3', '\x89', '\x6a',
    '\x8f', '\x60', '\x16', '\x33', '\x69', '\x7e', '\x57', '\xa8',
    '\xed', '\xe3', '\x35', '\x4d', '\x3b', '\x31', '\x76', '\xa8',
    '\xa9', '\xa4', '\x34', '\x6f', '\xbd', '\x3c', '\xc5', '\x88',
    '\xbd', '\xbf', '\x19', '\x8e', '\x85', '\xbd', '\xe8', '\x14',
    '\x32', '\x68', '\xc0', '\x67', '\x8c', '\x0a', '\x2d', '\x20',
    '\x73', '\x3f', '\x1d', '\xb5', '\x46', '\x48', '\xd9', '\xd2',
    '\xf5', '\xde', '\x04', '\xb0', '\x51', '\x63', '\x30', '\xf8',
    '\x85', '\xf3', '\x16', '\xdf', '\x80', '\xb3', '\x4f', '\x59',
    '\x3d', '\xff', '\x63', '\xe4', '\x8b', '\x24', '\xb1', '\x88',
    '\xe3', '\x76', '\xd8', '\xd3', '\x9c', '\x99', '\xf5', '\xa8',
    '\x3b', '\xe3', '\x88', '\x4c', '\x9c', '\xf7', '\x34', '\x3c',
    '\x93', '\x4b', '\x81', '\x51', '\xdd', '\x0b', '\x07', '\xd6',
    '\x8c', '\xe3', '\x95', '\x9f', '\x95', '\x2c', '\x0f', '\x55',
    '\xec', '\xf5', '\x5d', '\x00', '\x45', '\x57', '\x8c', '\xff',
    '\xf1', '\x67', '\x68', '\xdc', '\xf3', '\x20', '\x6b', '\xa1',
    '\x83', '\x4d', '\x45', '\xfa', '\xe7', '\xc0', '\xd2', '\x8e',
    '\x30', '\x7f', '\xea', '\x43', '\x41', '\x0f', '\x07', '\x15',
    '\xc7', '\x12', '\x9c', '\xee', '\x9f', '\x36', '\x45', '\xe0',
    '\x20', '\x30', '\x67', '\x69', '\xa9', '\x6d', '\xd3', '\x3e',
    '\x3f', '\xb3', '\xc4', '\xe7', '\x39', '\x09', '\x81', '\x4a',
    '\xeb', '\x7b', '\xf3', '\x68', '\x74', '\xb6', '\x4a', '\x8d',
    '\x7f', '\x5b', '\x33', '\xa5', '\x08', '\x98', '\xc1', '\x6d',
    '\x6c', '\x28', '\xdd', '\x2b', '\x70', '\xbc', '\xa3', '\x8a',
    '\xc2', '\x33', '\x07', '\x9c', '\xdf', '\x00', '\x8c', '\xf7',
    '\xc2', '\x45', '\x2e', '\xd6', '\x46', '\x71', '\xc9', '\xd3',
    '\x34', '\xe3', '\x26', '\x83', '\x84', '\xbd', '\xb5', '\xb6',
    '\x15', '\x69', '\x2e', '\x48', '\xbf', '\x2c', '\x1f', '\x04',
    '\xe3', '\x50', '\xf5', '\x90', '\x7e', '\x54', '\xa7', '\x07',
    '\x79', '\xa7', '\xfe', '\xbf', '\xea', '\x6b', '\x11', '\x71',
    '\x62', '\x90', '\x90', '\x3a', '\x61', '\xee', '\xfb', '\xa2',
    '\x07', '\x4c', '\x3a', '\x8f', '\x3c', '\x0c', '\x70', '\xc7',
    '\x16', '\x25', '\x8f', '\xae', '\x77', '\xf6', '\x58', '\xc7',
    '\x1f', '\x4d', '\xa1', '\x00', '\x92', '\x10', '\x9d', '\x7f',
    '\x2a', '\x69', '\x5f', '\xcd', '\xdf', '\xae', '\xa8', '\xc1',
    '\xd5', '\x76', '\xd8', '\x3a', '\x20', '\xc6', '\x7b', '\x30',
    '\x36', '\x39', '\x1d', '\xd1', '\x11', '\xc5', '\x8c', '\xc7',
    '\x8f', '\x57', '\x27', '\x5d', '\x54', '\x82', '\x30', '\xc2',
    '\x92', '\xe2', '\x06', '\x8d', '\xd5', '\xe6', '\x28', '\x36',
    '\xc6', '\xe7', '\x4d', '\xa8', '\x9a', '\xcc', '\xb3', '\x76',
    '\x4a', '\x44', '\x6a', '\x43', '\x9d', '\x40', '\x30', '\x30',
    '\xfc', '\xe8', '\x63', '\xc2', '\x14', '\x05', '\x26', '\xce',
    '\xd0', '\x30', '\x34', '\x1b', '\x57', '\x14', '\x4d', '\x8b',
    '\xd9', '\x06', '\xbe', '\x1d', '\x3b', '\x8d', '\xf4', '\x4d',
    '\x49', '\x06', '\xf8', '\x37', '\x9c', '\x31', '\xc4', '\x26',
    '\x74', '\xb1', '\xe9', '\x7b', '\x68', '\x68', '\xcc', '\x1c',
    '\x90', '\x8f', '\x8c', '\x61', '\x5b', '\x79', '\x6b', '\x95',
    '\xca', '\xb3', '\xa6', '\x82', '\x55', '\x5a', '\x55', '\x80',
    '\xe2', '\x0e', '\x36', '\x4f', '\xe4', '\x5a', '\xf1', '\x26',
    '\x63', '\x82', '\xfd', '\x7a', '\x7b', '\x7d', '\xbb', '\x31',
    '\x36', '\x96', '\x58', '\x93', '\x64', '\x46', '\x68', '\x4e',
    '\x1a', '\x50', '\x52', '\x1e', '\x68', '\x5a', '\x01', '\x90',
    '\x41', '\x8e', '\xb7', '\x13', '\xca', '\x27', '\x21', '\x62',
    '\x1e', '\xd5', '\x8e', '\x8e', '\xf8', '\x7c', '\x24', '\xb4',
    '\x28', '\x82', '\x41', '\x2d', '\x1f', '\xcc', '\xd6', '\xb9',
    '\x13', '\xd4', '\x50', '\x58', '\x83', '\x77', '\x09', '\x55',
    '\xd7', '\x23', '\x67', '\x70', '\x42', '\x58', '\x16', '\x21',
    '\x68', '\x5e', '\x29', '\xa5', '\xed', '\x79', '\x2f', '\x92',
    '\x10', '\x87', '\x02', '\xf0', '\x12', '\xa7', '\x09', '\xb8',
    '\xd0', '\xd3', '\xe8', '\x75', '\xab', '\x4a', '\x3e', '\x97',
    '\x1b', '\xa8', '\xc1', '\x3b', '\x09', '\xb3', '\x7b', '\x1a',
    '\xf3', '\x99', '\xee', '\x08', '\xb0', '\xcf', '\x4d', '\x19',
    '\x28', '\x1a', '\x24', '\xe1', '\x3d', '\xf0', '\x2c', '\xf0',
    '\x40', '\x93', '\x92', '\x54', '\x44', '\x0a', '\x07', '\xb1',
    '\x58', '\x02', '\x05', '\xa7', '\xcd', '\xad', '\x7d', '\xd7',
    '\xef', '\x56', '\x8a', '\x9c', '\xca', '\x8f', '\x8e', '\x16',
    '\x7a', '\x42', '\xad', '\x45', '\xc0', '\x66', '\x56', '\xa6',
    '\x30', '\x15', '\x6b', '\x2b', '\x73', '\x7b', '\x34', '\x2b',
    '\x6b', '\xec', '\x71', '\xcc', '\x4c', '\x1c', '\x65', '\x0e',
    '\x87', '\x4a', '\x2a', '\x1c', '\xd8', '\xea', '\xf8', '\xf3',
    '\x28', '\xe4', '\xdc', '\xa1', '\x85', '\xa2', '\xa2', '\x9a',
    '\x4e', '\x23', '\xaf', '\x4a', '\x86', '\xdc', '\xd5', '\x5a',
    '\x8d', '\xc8', '\x6f', '\x1c', '\x81', '\xe6', '\x25', '\x03',
    '\x6c', '\x9c', '\x73', '\x59', '\x71', '\xa6', '\xdb', '\xd8',
    '\xb6', '\x12', '\xdb', '\xb8', '\xe0', '\x3e', '\x43', '\xe9',
    '\x30', '\x63', '\x28', '\xd5', '\x59', '\xd2', '\x12', '\xfd',
    '\x19', '\x6e', '\xd6', '\xe8', '\xc5', '\xa8', '\xf7', '\xe2',
    '\x5b', '\x70', '\x74', '\x78', '\x11', '\x85', '\x2b', '\xc6',
    '\x47', '\x5a', '\x75', '\x87', '\x3a', '\x09', '\x9b', '\xe8',
    '\x5a', '\x5e', '\xaa', '\x81', '\xbe', '\x5a', '\xfe', '\xdd',
    '\x53', '\xf6', '\x16', '\xdc', '\xfb', '\x6c', '\xec', '\x21',
    '\xa1', '\x7a', '\xa1', '\x2f', '\xf8', '\xba', '\xd2', '\xb2',
    '\xed', '\x14', '\xd0', '\x45', '\xb5', '\x33', '\x53', '\x04',
    '\x43', '\x94', '\x86', '\x55', '\xe0', '\xb5', '\x70', '\x69',
    '\x29', '\x87', '\x75', '\x79', '\xad', '\x53', '\x87', '\xda',
    '\xa2', '\x88', '\xc6', '\x07', '\x17', '\x50', '\x08', '\xac',
    '\xe8', '\xab', '\x1e', '\x62', '\xd1', '\x7b', '\x6e', '\x9d',
    '\x5d', '\x86', '\x00', '\x68', '\xad', '\x56', '\xd7', '\x55',
    '\xfe', '\x28', '\x41', '\x99', '\x3b', '\x44', '\x51', '\x2e',
    '\x5f', '\x12', '\xff', '\x94', '\xf1', '\x9d', '\xa8', '\xf4',
    '\xf0', '\xf3', '\x57', '\x80', '\x16', '\x5a', '\x5e', '\xe9',
    '\x1b', '\xc5', '\xc8', '\x9d', '\x38', '\xcf', '\x11', '\x35',
    '\x70', '\x8b', '\x08', '\xeb', '\xfb', '\x9a', '\x6e', '\x9c',
    '\xfb', '\xc7', '\xac', '\xb5', '\x83', '\xac', '\x88', '\x1d',
    '\x78', '\x71', '\xeb', '\x77', '\xb3', '\x36', '\x1f', '\xce',
    '\xf3', '\x03', '\x6c', '\x61', '\x20', '\xc4', '\x3f', '\x22',
    '\x1a', '\xdd', '\xcc', '\x6e', '\x64', '\xd0', '\x35', '\x68',
    '\x44', '\x1c', '\x66', '\xd6', '\x2a', '\xb1', '\xcc', '\x28',
    '\xf6', '\xac', '\x7e', '\x50', '\x2c', '\x8e', '\x49', '\xbf',
    '\x6a', '\x1c', '\xd0', '\x6b', '\xf9', '\xd0', '\x9e', '\x50',
    '\x5e', '\x91', '\x3c', '\xf3', '\x2d', '\x32', '\xc5', '\xf0',
    '\x35', '\xda', '\xfa', '\x29', '\x86', '\x6f', '\x3d', '\xa7',
    '\x28', '\x79', '\x96', '\x43', '\xfa', '\x27', '\x27', '\xa0',
    '\x14', '\x27', '\xa4', '\x78', '\xae', '\x7c', '\x5b', '\xb2',
    '\x29', '\x32', '\xe7', '\x99', '\x7b', '\x8b', '\x99', '\x11',
    '\x7f', '\xab', '\x5a', '\xe6', '\x68', '\xae', '\xa3', '\xd4',
    '\x5b', '\x4a', '\x59', '\xb4', '\x35', '\x3a', '\xcc', '\x9d',
    '\x9f', '\x85', '\xdb', '\x0b', '\xe8', '\x29', '\x69', '\x93',
    '\xb8', '\x2e', '\x74', '\x4b', '\x01', '\xcf', '\x24', '\x80',
    '\x0e', '\xa7', '\x94', '\x8a', '\xae', '\xa2', '\x14', '\xb5',
    '\xb1', '\x7e', '\x3c', '\x35', '\x35', '\xa4', '\x2c', '\x0c',
    '\xd6', '\xec', '\x1e', '\x29', '\x78', '\xfc', '\x5a', '\x39',
    '\x62', '\xa5', '\xd7', '\x3e', '\x2a', '\xce', '\x5f', '\x2c',
    '\x8e', '\xe7', '\xcb', '\x11', '\x2f', '\x5f', '\x4e', '\x35',
    '\x65', '\xa9', '\xc0', '\x80', '\x3e', '\x13', '\x2f', '\x4b',
    '\xa9', '\x76', '\x44', '\x18', '\xb5', '\xd0', '\x30', '\x81',
    '\x5a', '\x37', '\x88', '\x70', '\x3a', '\xdf', '\x66', '\x98',
    '\xf2', '\x0b', '\x29', '\x39', '\xa0', '\x42', '\x0e', '\x49',
    '\x13', '\xea', '\x21', '\x76', '\x23', '\x32', '\xc8', '\x9c',
    '\x30', '\xb1', '\xc9', '\x1c', '\xe9', '\x22', '\x35', '\x79',
    '\x76', '\xd2', '\xb7', '\x26', '\x7a', '\x8d', '\x09', '\x3c',
    '\xfb', '\xd0', '\xd8', '\xbf', '\x7b', '\x63', '\x64', '\x04',
    '\xfe', '\x33', '\x11', '\x1d', '\xea', '\xd9', '\x0b', '\x05',
    '\xb5', '\xaa', '\x42', '\x39', '\xb7', '\x00', '\xd5', '\x06',
    '\xf6', '\x81', '\x83', '\x69', '\x5b', '\x4a', '\x66', '\xe3',
    '\xbc', '\x99', '\xf6', '\x9a', '\xe1', '\x00', '\x0a', '\xc0',
    '\x54', '\x9a', '\x6b', '\xa0', '\x78', '\x3e', '\x43', '\x2c',
    '\xb7', '\xff', '\xd3', '\xe9', '\x7b', '\xf4', '\x6d', '\x7a',
    '\x4a', '\x38', '\x34', '\x83', '\x9b', '\x18', '\x7b', '\x08',
    '\x21', '\xff', '\x8c', '\x3a', '\x85', '\xed', '\xb7', '\x33',
    '\x78', '\xa4', '\xed', '\x4a', '\x2e', '\x1a', '\x08', '\x37',
    '\xf0', '\xe7', '\xa8', '\xe3', '\xb4', '\xf6', '\x26', '\x38',
    '\xd6', '\xa2', '\x68', '\x80', '\x6d', '\x38', '\xbf', '\x33',
    '\x75', '\x5e', '\x84', '\xca', '\x8d', '\x01', '\x7b', '\x91',
    '\x42', '\x98', '\xe7', '\x86', '\x99', '\xe3', '\x57', '\xa9',
    '\x5e', '\x46', '\x67', '\xd3', '\x6d', '\x58', '\xca', '\x4f',
    '\xbb', '\xf2', '\x52', '\xa9', '\x9c', '\xda', '\xb8', '\x58',
    '\xe9', '\x64', '\x9f', '\x67', '\x7e', '\x84', '\x10', '\xac',
    '\x4b', '\xc6', '\xfa', '\xdc', '\x11', '\xee', '\xa2', '\x40',
    '\x44', '\xb8', '\xa8', '\x1a', '\x9c', '\xb3', '\x99', '\x18',
    '\x9a', '\xad', '\xeb', '\x05', '\xb8', '\xc9', '\x8f', '\x36',
    '\x17', '\x9e', '\x69', '\x66', '\x1c', '\x9d', '\x31', '\x0d',
    '\x30', '\xdd', '\xc6', '\xf0', '\x60', '\xa7', '\xee', '\xd2',
    '\x37', '\x9f', '\x6c', '\x8c', '\x8d', '\xd9', '\x1f', '\xd5',
    '\x5d', '\x7d', '\x2a', '\xd4', '\x32', '\x1a', '\xa6', '\xb5',
    '\x8b', '\x07', '\x33', '\x8a', '\x55', '\xc1', '\xf1', '\x16',
    '\xd7', '\x33', '\xa5', '\x7f', '\x6c', '\xb9', '\xe3', '\x2f',
    '\x1c', '\x31', '\x8f', '\x2f', '\x47', '\xbe', '\x0c', '\x4e',
    '\xfa', '\xf8', '\x37', '\x10', '\x8a', '\xda', '\x32', '\x40',
    '\x49', '\x95', '\x06', '\x49', '\x21', '\x1f', '\x11', '\x1e',
    '\xb9', '\x16', '\x5e', '\x5c', '\xd8', '\x3d', '\xe5', '\xf8',
    '\x33', '\x89', '\x45', '\xf5', '\xf2', '\x67', '\x07', '\x72',
    '\x3a', '\x74', '\xa7', '\xe6', '\x71', '\xbb', '\xca', '\x2a',
    '\x4f', '\xc7', '\x9b', '\x04', '\x70', '\x12', '\x56', '\x8f',
    '\x22', '\x16', '\xdd', '\x72', '\xad', '\xeb', '\x27', '\x86',
    '\x1e', '\xae', '\x87', '\x80', '\x3b', '\xdd', '\x87', '\xf0',
    '\x82', '\xd5', '\xac', '\x3b', '\xf2', '\xc4', '\x05', '\x05',
    '\x20', '\x36', '\x4d', '\x4b', '\x09', '\x89', '\xdc', '\x07',
    '\x7e', '\x5a', '\xe8', '\xbd', '\xff', '\x43', '\xc7', '\xbe',
    '\xe3', '\xab', '\x92', '\xdb', '\xbe', '\x15', '\xa6', '\xd8',
    '\x94', '\x55', '\x4d', '\x24', '\x94', '\xfd', '\x0b', '\x05',
    '\x48', '\x1d', '\x13', '\x15', '\x73', '\x7a', '\x72', '\x71',
    '\x98', '\xe8', '\xd6', '\x4c', '\x93', '\xbf', '\xdb', '\xf8',
    '\xf3', '\x92', '\x57', '\x3b', '\x67', '\xe0', '\xfa', '\x3a',
    '\x58', '\x59', '\x9d', '\x68', '\x83', '\x1d', '\x29', '\x5b',
    '\xe2', '\xed', '\x78', '\x02', '\xdb', '\x4d', '\xea', '\x27',
    '\xe7', '\xf5', '\x45', '\x3d', '\x85', '\x01', '\x91', '\xd8',
    '\x34', '\x94', '\xf2', '\xba', '\xdd', '\xd9', '\x69', '\xc0',
    '\xa1', '\x3a', '\xf1', '\xfc', '\xb5', '\x3a', '\x6e', '\x95',
    '\x1c', '\xd6', '\x88', '\x98', '\xfb', '\x4e', '\x72', '\x0d',
    '\xce', '\x33', '\xcc', '\xb9', '\xf0', '\xfe', '\x3c', '\x2c',
    '\x00', '\x08', '\x23', '\x19', '\xe5', '\x54', '\xfe', '\x52',
    '\xf6', '\x17', '\x14', '\x7f', '\x11', '\x74', '\x3c', '\xfa',
    '\xc2', '\x5c', '\xd0', '\x81', '\xfc', '\x14', '\xeb', '\xb4',
    '\xe4', '\x1f', '\xb7', '\xf9', '\x90', '\x2e', '\x70', '\xcc',
    '\x33', '\x7c', '\xbf', '\x7b', '\xc8', '\x4d', '\xc3', '\xaf',
    '\x52', '\x9f', '\x7e', '\xbf', '\x93', '\xa7', '\xdd', '\xea',
    '\xbe', '\xdb', '\x4f', '\xc9', '\x60', '\xf6', '\x3c', '\x71',
    '\x3d', '\x50', '\xbb', '\x3c', '\x6a', '\xba', '\x17', '\x78',
    '\x2b', '\xc2', '\x2b', '\x23', '\xba', '\x50', '\x94', '\x0b',
    '\xef', '\xe2', '\x8d', '\x2e', '\x75', '\x15', '\x1e', '\x3a',
    '\xa2', '\x1e', '\x6c', '\x1b', '\xe8', '\x77', '\xaa', '\x80',
    '\xaa', '\xc2', '\xa9', '\xc8', '\x75', '\xaa', '\x84', '\xb3',
    '\xce', '\x01', '\xce', '\x34', '\x40', '\x6f', '\xf4', '\xb1',
    '\x0a', '\x1f', '\xb4', '\x67', '\x50', '\x17', '\xe0', '\x96',
    '\x2e', '\xe9', '\xed', '\x00', '\x76', '\xb1', '\x2e', '\x21',
    '\x06', '\x28', '\x27', '\xea', '\x39', '\x12', '\x8c', '\xa4',
    '\x95', '\xbc', '\x8b', '\x6d', '\x96', '\x2b', '\xe6', '\x9d',
    '\xaa', '\x8e', '\xba', '\xa7', '\x48', '\xc9', '\xb0', '\xd8',
    '\xd4', '\x86', '\xed', '\x1e', '\xf7', '\xc6', '\xc9', '\xb6',
    '\x76', '\x37', '\x67', '\xf3', '\x7c', '\x42', '\x9b', '\xf5',
    '\x8e', '\xe9', '\x34', '\xea', '\x13', '\x62', '\xc2', '\x1e',
    '\x66', '\x35', '\xe9', '\x50', '\x2e', '\xb8', '\x5f', '\x6d',
    '\x3e', '\x56', '\xd2', '\x73', '\x49', '\x4e', '\xde', '\xdf',
    '\xb1', '\xf5', '\xd8', '\x33', '\xe6', '\xf1', '\xca', '\xac',
    '\x62', '\xf8', '\xff', '\xdd', '\x93', '\x40', '\x08', '\x6e',
    '\x29', '\xae', '\x44', '\x63', '\xa8', '\x92', '\x95', '\xb0',
    '\xd6', '\x5e', '\x43', '\x97', '\x1e', '\xc3', '\x84', '\x9a',
    '\x49', '\x07', '\xe3', '\xfa', '\xa4', '\x6f', '\xf5', '\x0e',
    '\x5d', '\xe3', '\x01', '\x50', '\xd8', '\x2a', '\x3c', '\x57',
    '\xf9', '\xfb', '\xc4', '\xff', '\x55', '\xd3', '\x66', '\x47',
    '\x4c', '\xd2', '\x1c', '\xf6', '\xe3', '\xf2', '\xdd', '\x63',
    '\xed', '\xfe', '\x9a', '\x98', '\x10', '\xce', '\xd1', '\x7b',
    '\x72', '\x3f', '\x9d', '\xe2', '\xf6', '\xae', '\xb6', '\xcb',
    '\xc0', '\x53', '\x86', '\xe0', '\xf5', '\x71', '\xee', '\x81',
    '\x19', '\xa7', '\x69', '\x11', '\xa9', '\x66', '\x6e', '\x4c',
    '\xb0', '\xa9', '\x74', '\x54', '\x59', '\x15', '\x02', '\x4c',
    '\x43', '\x45', '\x02', '\x85', '\xb3', '\x71', '\x7b', '\x87',
    '\x0d', '\xed', '\x05', '\xdc', '\x8e', '\x8e', '\xdf', '\xb9',
    '\x07', '\x21', '\x56', '\xbb', '\x07', '\xd6', '\x72', '\x21',
    '\x4f', '\x5c', '\xff', '\x72', '\x26', '\x6a', '\x32', '\xb0',
    '\x25', '\xe4', '\x98', '\x3c', '\xf0', '\x1a', '\x14', '\xad',
    '\xdd', '\xd0', '\x50', '\x63', '\x92', '\x21', '\x1e', '\xac',
    '\xb9', '\x5d', '\x28', '\x55', '\x02', '\x97', '\x34', '\x82',
    '\x79', '\x46', '\x97', '\x2c', '\x3d', '\x44', '\x25', '\x8d',
    '\x27', '\xcc', '\x80', '\xd9', '\x0f', '\x44', '\x60', '\x63',
    '\x61', '\xa4', '\x3e', '\xf8', '\x05', '\xa9', '\x55', '\xdd',
    '\x2c', '\xe2', '\x35', '\x10', '\xf3', '\x05', '\x5f', '\x05',
    '\x13', '\xb8', '\x1d', '\xb1', '\x2b', '\x94', '\xcf', '\x5b',
    '\x19', '\x80', '\x01', '\xb9', '\x5a', '\x6a', '\x6f', '\x74',
    '\xc3', '\x78', '\xa7', '\xb5', '\xa4', '\xd8', '\x8a', '\xf2',
    '\x42', '\x30', '\xd6', '\x28', '\x6c', '\xf5', '\x5b', '\x50',
    '\x7d', '\x77', '\xab', '\x23', '\xe5', '\xf8', '\x6e', '\x02',
    '\x8f', '\x5e', '\x03', '\x85', '\x66', '\xd4', '\x55', '\xea',
    '\xed', '\x99', '\x97', '\xc6', '\x0a', '\x41', '\xb6', '\x0f',
    '\x45', '\x4a', '\x5f', '\x23', '\x1f', '\x1f', '\x9e', '\x38',
    '\xdb', '\x08', '\x4e', '\x98', '\x74', '\xe5', '\x9b', '\xbc',
    '\xeb', '\xb4', '\x84', '\xf1', '\x7a', '\x88', '\xfe', '\xac',
    '\x4d', '\x5b', '\x86', '\xe7', '\xd4', '\x09', '\x57', '\x37',
    '\x6e', '\x49', '\x14', '\x0e', '\xc4', '\x91', '\x03', '\xe1',
    '\x59', '\x02', '\xb7', '\x12', '\x98', '\xd8', '\x62', '\xee',
    '\x60', '\xc8', '\x0d', '\x74', '\xf9', '\x35', '\x0b', '\x22',
    '\x9f', '\xe8', '\x8d', '\xe2', '\xcd', '\x8f', '\x0c', '\xab',
    '\x6c', '\xe6', '\x23', '\xbe', '\xff', '\x18', '\x21', '\xd9',
    '\x72', '\x55', '\xf3', '\x81', '\x6b', '\x83', '\x59', '\x08',
    '\xf8', '\xea', '\x2b', '\x0e', '\xc1', '\x23', '\x9f', '\xcb',
    '\xad', '\x0c', '\x9b', '\x0d', '\x0c', '\x22', '\x29', '\x42',
    '\xe8', '\x09', '\x92', '\x0a', '\x44', '\xb1', '\xa1', '\x39',
    '\x33', '\x9f', '\x22', '\xc3', '\x0e', '\xf8', '\xb4', '\x64',
    '\x9c', '\x7f', '\xdf', '\xfe', '\x97', '\x20', '\x3e', '\xdf',
    '\x1b', '\x08', '\xb0', '\xcb', '\x23', '\xa7', '\x48', '\xcf',
    '\x0c', '\x5a', '\x3e', '\x0b', '\xc1', '\xf6', '\x94', '\xc2',
    '\x87', '\x76', '\x2e', '\x9d', '\x4a', '\xde', '\x5c', '\x23',
    '\x19', '\x11', '\x1c', '\x85', '\x8a', '\x6e', '\x9b', '\xed',
    '\x2a', '\x5c', '\x0a', '\x04', '\x3b', '\x4e', '\xe7', '\x7a',
    '\x08', '\xca', '\xcb', '\x5b', '\x3f', '\x8d', '\xc6', '\xf9',
    '\x76', '\xb6', '\x7d', '\xbb', '\x36', '\x97', '\x07', '\x0a',
    '\x26', '\x5f', '\x35', '\xa5', '\x58', '\xb1', '\x72', '\x7b',
    '\x7d', '\x94', '\x66', '\xa8', '\x43', '\x39', '\xf7', '\x1d',
    '\x9b', '\x24', '\x9b', '\x50', '\x0e', '\x95', '\x1a', '\x48',
    '\x72', '\xd2', '\xab', '\xb1', '\xdf', '\x74', '\x4a', '\x64',
    '\x69', '\x64', '\x5a', '\xdb', '\xd9', '\xde', '\x60', '\x9d',
    '\xe2', '\x1b', '\x2b', '\x2c', '\x10', '\x32', '\x77', '\xa8',
    '\x95', '\x9b', '\xc7', '\x3d', '\xe4', '\x14', '\xdb', '\x1a',
    '\x95', '\x1e', '\x3c', '\xf2', '\xed', '\xe0', '\xb2', '\xd9',
    '\x7e', '\x6d', '\x09', '\xde', '\x5f', '\x27', '\xb4', '\x9c',
    '\x16', '\x04', '\xaa', '\x05', '\x87', '\x51', '\x08', '\x76',
    '\x60', '\x58', '\x10', '\xb0', '\xc9', '\x64', '\x12', '\xf9',
    '\xef', '\x1f', '\x55', '\xd7', '\x4a', '\x8e', '\xd8', '\x53',
    '\xff', '\x22', '\x87', '\x61', '\x20', '\xec', '\x49', '\x8d',
    '\x8c', '\xda', '\x4c', '\x2b', '\xc0', '\xb0', '\x82', '\xb6',
    '\x84', '\x05', '\xdb', '\xad', '\xfb', '\xa4', '\xe0', '\xad',
    '\xc8', '\xdc', '\x75', '\x96', '\xc6', '\xa9', '\x79', '\xd9',
    '\x84', '\x95', '\x52', '\xc9', '\x9a', '\xa6', '\x5a', '\xf2',
    '\x30', '\x70', '\xcb', '\x94', '\x2c', '\x16', '\x8c', '\xbf',
    '\x30', '\x61', '\x21', '\xee', '\xd2', '\x25', '\xd0', '\x5b',
    '\xe5', '\x26', '\x21', '\x42', '\xbf', '\x00', '\x8c', '\x6b',
    '\xb2', '\x53', '\x9d', '\xf5', '\xf9', '\xd6', '\x57', '\x58',
    '\x33', '\xab', '\x6c', '\xaf', '\xbf', '\xa6', '\x31', '\x73',
    '\xbb', '\xc8', '\x6a', '\x21', '\xa9', '\xd4', '\x2f', '\xa0',
    '\xd0', '\xe8', '\xa6', '\xca', '\xc5', '\x35', '\x54', '\xe5',
    '\x37', '\x6c', '\xc2', '\xf4', '\x80', '\xf5', '\xc0', '\xf5',
    '\xca', '\x52', '\x2e', '\xeb', '\x11', '\x8b', '\x74', '\x9c',
    '\x2f', '\xc5', '\xbf', '\x28', '\xc2', '\xad', '\x63', '\xb1',
    '\x1e', '\x7a', '\xc5', '\xf7', '\x43', '\xea', '\x7d', '\xd9',
    '\xd8', '\x86', '\xb7', '\xc6', '\xeb', '\x5e', '\xf5', '\x44',
    '\xfa', '\xdd', '\x05', '\x2a', '\x87', '\xac', '\xeb', '\x4b',
    '\xcc', '\x9c', '\xa2', '\x52', '\x2c', '\x29', '\x3c', '\x79',
    '\xb3', '\xe8', '\x70', '\x69', '\x22', '\xf1', '\x2b', '\x6c',
    '\x72', '\xec', '\x88', '\x1a', '\xd6', '\x40', '\x10', '\xa7',
    '\x5b', '\x41', '\x0c', '\x47', '\x78', '\x50', '\x50', '\x35',
    '\x8f', '\xdd', '\x05', '\x9d', '\xac', '\xa8', '\x52', '\xb6',
    '\x0c', '\x44', '\x7d', '\x94', '\x71', '\x78', '\x13', '\x39',
    '\x04', '\x95', '\xcb', '\x0e', '\x26', '\x88', '\xb7', '\x01',
    '\xdc', '\xc0', '\xbf', '\x92', '\x53', '\xd7', '\x1b', '\x10',
    '\xb9', '\xef', '\x3e', '\xf3', '\x9e', '\xe2', '\x52', '\x17',
    '\x7f', '\xf2', '\x5b', '\xc2', '\x08', '\x38', '\x94', '\x2a',
    '\xb2', '\x39', '\x13', '\xe6', '\x6a', '\xd4', '\xf7', '\x55',
    '\x8f', '\xb5', '\x31', '\x4d', '\xc6', '\x60', '\x14', '\xf3',
    '\x53', '\x9f', '\xfd', '\x68', '\xdf', '\x53', '\x5d', '\x5d',
    '\x87', '\xf8', '\xcb', '\x0b', '\x32', '\xa2', '\xc7', '\x5b',
    '\xcb', '\x5d', '\x6b', '\xd5', '\x45', '\x5d', '\x17', '\x68',
    '\x68', '\x64', '\x3d', '\x27', '\xe2', '\x79', '\xec', '\xb3',
    '\xb4', '\xab', '\x46', '\x76', '\xb2', '\xad', '\x55', '\x75',
    '\x07', '\x4b', '\x92', '\x5a', '\x52', '\x0f', '\x99', '\xf1',
    '\xd2', '\x5f', '\xca', '\xaf', '\xd4', '\xe6', '\x6f', '\x56',
    '\x0a', '\xc7', '\xc1', '\xc3', '\x43', '\xd0', '\xd2', '\x41',
    '\x0a', '\x54', '\x60', '\x41', '\xa5', '\x3f', '\x3c', '\x96',
    '\x90', '\x1d', '\x33', '\x64', '\x87', '\xeb', '\xe1', '\x00',
    '\x7f', '\x86', '\x88', '\xb1', '\x0f', '\xa9', '\x3c', '\x3b',
    '\x87', '\x59', '\xd2', '\x2c', '\x35', '\xdd', '\x09', '\x09',
    '\xdc', '\xec', '\xcf', '\xcb', '\x78', '\x6d', '\x82', '\x65',
    '\xa7', '\x29', '\xa3', '\xab', '\x51', '\xe7', '\x74', '\x63',
    '\xca', '\x10', '\xd4', '\x3a', '\x37', '\x4a', '\x78', '\xc9',
    '\x27', '\xec', '\xf2', '\x59', '\xd9', '\x93', '\x7e', '\x4c',
    '\x93', '\x60', '\x4e', '\x3e', '\x02', '\x12', '\x61', '\x0a',
    '\x0d', '\x07', '\x00', '\x7c', '\x3e', '\x23', '\x3c', '\x9c',
    '\xee', '\x3f', '\x2d', '\x88', '\x35', '\xbd', '\xb2', '\xe9',
    '\x34', '\x6e', '\x47', '\xaa', '\x4f', '\x0c', '\x5e', '\x85',
    '\xfe', '\xca', '\xbc', '\x20', '\x18', '\xf9', '\x30', '\x55',
    '\x01', '\x71', '\x45', '\xe8', '\x88', '\x5e', '\x4f', '\xb3',
    '\x72', '\x67', '\xd4', '\x79', '\x03', '\x3b', '\xde', '\x58',
    '\xb8', '\xbd', '\xd0', '\x63', '\xd5', '\x37', '\xac', '\xc2',
    '\xdb', '\xfd', '\x19', '\x95', '\x74', '\x45', '\xb7', '\xd9',
    '\x8a', '\x9b', '\x01', '\xc7', '\xbd', '\x24', '\x03', '\x13',
    '\x2a', '\xfd', '\x44', '\x49', '\x12', '\x2c', '\x30', '\x46',
    '\x47', '\x72', '\x9c', '\x2f', '\xf6', '\x89', '\xd8', '\x01',
    '\x59', '\x14', '\x79', '\x9f', '\x98', '\xdd', '\x8b', '\x0c',
    '\xbc', '\x79', '\x10', '\xb5', '\x6a', '\xfc', '\x05', '\x68',
    '\x42', '\xa6', '\xa9', '\x4e', '\xb0', '\x17', '\xf2', '\xf5',
    '\xc4', '\xb0', '\xfd', '\x9b', '\xa8', '\xa6', '\x53', '\x93',
    '\xa1', '\xff', '\xfd', '\x57', '\xb6', '\xe3', '\x5f', '\x61',
    '\xfc', '\x0f', '\x4a', '\x0f', '\xbb', '\x95', '\x70', '\x7c',
    '\x41', '\x41', '\x53', '\xb6', '\x6d', '\x8e', '\x56', '\x54',
    '\x29', '\xf3', '\xc3', '\x92', '\x75', '\x14', '\x26', '\x74',
    '\x58', '\x10', '\xc3', '\x31', '\x97', '\x18', '\x5c', '\x6a',
    '\x48', '\xc4', '\x42', '\xf0', '\x21', '\xf6', '\xe2', '\x19',
    '\x0f', '\xf8', '\x30', '\x47', '\x82', '\x1f', '\x50', '\x9f',
    '\x4a', '\xdc', '\x6d', '\xdf', '\xa3', '\xe5', '\x88', '\xb0'
  };
static const oh_int _oh_tk_fileDataSize = sizeof(_oh_tk_fileData);

static oh_int _oh_tk_createTestFile(const oh_char *path, oh_int idx,
  oh_ulong size)
{
  oh_int fd;

  oh_fs_unlink(path);
  fd = oh_fs_open(path, OH_FS_O_CREAT | OH_FS_O_WRONLY);
  if (fd < 0)
    goto cleanupret;

  idx = idx % _oh_tk_fileDataSize;

  for (; size > 0;)
  {
    oh_long fsret;
    oh_ulong wrsize;

    wrsize = _oh_tk_fileDataSize - idx;
    if (wrsize > size)
    {
      wrsize = size;
    }

    fsret = oh_fs_write(fd, _oh_tk_fileData + idx, wrsize);

    if (fsret >= 0)
    {
      idx = (idx + fsret) % _oh_tk_fileDataSize;
      size -= fsret;
    }
    else
      goto cleanupret;
  }

  oh_fs_close(fd);

  return 0;

cleanupret:
  oh_fs_close(fd);
  oh_fs_unlink(path);
  return -1;
}

static oh_ulong _oh_tk_compareTestFile(const oh_uint8 *data, oh_int idx,
  oh_ulong size)
{
  oh_ulong i;
  oh_ulong ret;

  for (i = 0, ret = 0; i < size; i++)
  {
    if (data[i] != _oh_tk_fileData[(i + idx) % _oh_tk_fileDataSize])
      ret++;
  }

  return ret;
}

static oh_int _oh_tk_testFS00001(oh_tk_opt_t *opt)
{
  oh_char path[OH_FS_MAXPATHLEN];
  oh_int fd;
  oh_fs_stat_t statbuf;
  oh_int ret = 0;
  oh_char ch = 'a';

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00001-01.dat", opt->homedir,
    opt->testdir);

  oh_fs_unlink(path);

  fd = oh_fs_open(path, OH_FS_O_RDONLY);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 1 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 1 is a success"));

  fd = oh_fs_open(path, OH_FS_O_WRONLY);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 2 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 2 is a success"));

  fd = oh_fs_open(path, OH_FS_O_RDWR);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 3 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 3 is a success"));

  fd = oh_fs_open(path, OH_FS_O_RDONLY | OH_FS_O_TRUNC);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 4 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 4 is a success"));

  fd = oh_fs_open(path, OH_FS_O_WRONLY | OH_FS_O_TRUNC);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 5 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 5 is a success"));

  fd = oh_fs_open(path, OH_FS_O_RDWR | OH_FS_O_TRUNC);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 6 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 6 is a success"));

  fd = oh_fs_open(path, OH_FS_O_RDONLY | OH_FS_O_EXCL);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 7 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 7 is a success"));

  fd = oh_fs_open(path, OH_FS_O_WRONLY | OH_FS_O_EXCL);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 8 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 8 is a success"));

  fd = oh_fs_open(path, OH_FS_O_RDWR | OH_FS_O_EXCL);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 9 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 9 is a success"));

  fd = oh_fs_open(path, OH_FS_O_RDONLY | OH_FS_O_EXCL | OH_FS_O_TRUNC);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 10 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 10 is a success"));

  fd = oh_fs_open(path, OH_FS_O_WRONLY | OH_FS_O_EXCL | OH_FS_O_TRUNC);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 11 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 11 is a success"));

  fd = oh_fs_open(path, OH_FS_O_RDWR | OH_FS_O_EXCL | OH_FS_O_TRUNC);
  if (fd >= 0)
  {
    oh_fs_close(fd);
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 12 is a fail"));
    ret++;
  }
  else
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 12 is a success"));

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00001-02.dat", opt->homedir,
    opt->testdir);

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 0, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 13 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_RDONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 13 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_fs_write(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 13 is a fail 2"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 13 is a success"));
      }
      oh_fs_close(fd);
    }

    fd = oh_fs_open(path, OH_FS_O_WRONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 14 is a fail 0"));
      ret++;
    }
    else
    {
      if (oh_fs_read(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 14 is a fail 1"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 14 is a success"));
      }
      oh_fs_close(fd);
    }

    fd = oh_fs_open(path, OH_FS_O_RDWR);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 15 is a fail"));
      ret++;
    }
    else
    {
      oh_fs_close(fd);
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 15 is a success"));
    }

    fd = oh_fs_open(path, OH_FS_O_RDONLY | OH_FS_O_TRUNC);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 17 is a fail 0"));
      ret++;
    }
    else
    {
      if (oh_fs_write(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 17 is a fail 1"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 17 is a success"));
      }
      oh_fs_close(fd);
    }

    fd = oh_fs_open(path, OH_FS_O_RDONLY | OH_FS_O_EXCL);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 18 is a fail 0"));
      ret++;
    }
    else
    {
      if (oh_fs_write(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 18 is a fail 1"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 18 is a success"));
      }
      oh_fs_close(fd);
    }

    fd = oh_fs_open(path, OH_FS_O_WRONLY | OH_FS_O_EXCL);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 19 is a fail 0"));
      ret++;
    }
    else
    {
      if (oh_fs_read(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 19 is a fail 1"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 19 is a success"));
      }
      oh_fs_close(fd);
    }

    fd = oh_fs_open(path, OH_FS_O_RDWR | OH_FS_O_EXCL);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 20 is a fail"));
      ret++;
    }
    else
    {
      oh_fs_close(fd);
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 20 is a success"));
    }

    fd = oh_fs_open(path, OH_FS_O_RDONLY | OH_FS_O_EXCL | OH_FS_O_TRUNC);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 21 is a fail"));
      ret++;
    }
    else
    {
      if (oh_fs_write(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 21 is a fail 1"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 21 is a success"));
      }
      oh_fs_close(fd);
    }

    fd = oh_fs_open(path, OH_FS_O_RDONLY | OH_FS_O_CREAT);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 22 is a fail 0"));
      ret++;
    }
    else
    {
      if (oh_fs_write(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 22 is a fail 1"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 22 is a success"));
      }
      oh_fs_close(fd);
    }

    fd = oh_fs_open(path, OH_FS_O_WRONLY | OH_FS_O_CREAT);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 23 is a fail 0"));
      ret++;
    }
    else
    {
      if (oh_fs_read(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 23 is a fail 1"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 23 is a success"));
      }
      oh_fs_close(fd);
    }

    fd = oh_fs_open(path, OH_FS_O_RDWR | OH_FS_O_CREAT);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 24 is a fail"));
      ret++;
    }
    else
    {
      oh_fs_close(fd);
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 24 is a success"));
    }

    fd = oh_fs_open(path, OH_FS_O_RDONLY | OH_FS_O_CREAT | OH_FS_O_TRUNC);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 25 is a fail 0"));
      ret++;
    }
    else
    {
      if (oh_fs_write(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 25 is a fail 1"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 25 is a success"));
      }
      oh_fs_close(fd);
    }

    fd = oh_fs_open(path, OH_FS_O_EXCL | OH_FS_O_RDONLY | OH_FS_O_CREAT);
    if (fd >= 0)
    {
      oh_fs_close(fd);
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 26 is a fail"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 26 is a success"));
    }

    fd = oh_fs_open(path, OH_FS_O_EXCL | OH_FS_O_WRONLY | OH_FS_O_CREAT);
    if (fd >= 0)
    {
      oh_fs_close(fd);
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 27 is a fail"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 27 is a success"));
    }

    fd = oh_fs_open(path, OH_FS_O_EXCL | OH_FS_O_RDWR | OH_FS_O_CREAT);
    if (fd >= 0)
    {
      oh_fs_close(fd);
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 28 is a fail"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 28 is a success"));
    }

    fd =
      oh_fs_open(path, OH_FS_O_TRUNC | OH_FS_O_EXCL | OH_FS_O_RDONLY |
        OH_FS_O_CREAT);
    if (fd >= 0)
    {
      oh_fs_close(fd);
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 29 is a fail"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 29 is a success"));
    }

    fd =
      oh_fs_open(path, OH_FS_O_TRUNC | OH_FS_O_EXCL | OH_FS_O_WRONLY |
        OH_FS_O_CREAT);
    if (fd >= 0)
    {
      oh_fs_close(fd);
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 30 is a fail"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 30 is a success"));
    }

    fd =
      oh_fs_open(path, OH_FS_O_TRUNC | OH_FS_O_EXCL | OH_FS_O_RDWR |
        OH_FS_O_CREAT);
    if (fd >= 0)
    {
      oh_fs_close(fd);
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 31 is a fail"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 31 is a success"));
    }

    fd = oh_fs_open(path, OH_FS_O_RDONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 16 is a fail 0"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 16 is a fail 1"));
        ret++;
      }
      else if (statbuf.st_size != opt->filelen)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 16 is a fail 2"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 16 is a success"));
      }
      oh_fs_close(fd);
    }
  }

  oh_fs_unlink(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00001-03.dat", opt->homedir,
    opt->testdir);

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 0, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 32 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_TRUNC | OH_FS_O_WRONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 32 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_fs_read(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 32 is a fail 2"));
        ret++;
      }
      else
      {
        if (oh_fs_fstat(fd, &statbuf) != 0)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 32 is a fail 3"));
          ret++;
        }
        else if (statbuf.st_size != 0)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 32 is a fail 4"));
          ret++;
        }
        else
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 32 is a success"));
        }
      }
      oh_fs_close(fd);
    }
  }

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 0, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 33 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_TRUNC | OH_FS_O_RDWR);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 33 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 33 is a fail 2"));
        ret++;
      }
      else if (statbuf.st_size != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 33 is a fail 3"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 33 is a success"));
      }
      oh_fs_close(fd);
    }
  }

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 0, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 34 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_EXCL | OH_FS_O_TRUNC | OH_FS_O_WRONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 34 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_fs_read(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 34 is a fail 2"));
        ret++;
      }
      else
      {
        if (oh_fs_fstat(fd, &statbuf) != 0)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 34 is a fail 3"));
          ret++;
        }
        else if (statbuf.st_size != 0)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 34 is a fail 4"));
          ret++;
        }
        else
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 34 is a success"));
        }
      }
      oh_fs_close(fd);
    }
  }

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 0, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 35 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_EXCL | OH_FS_O_TRUNC | OH_FS_O_RDWR);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 35 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 35 is a fail 2"));
        ret++;
      }
      else if (statbuf.st_size != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 35 is a fail 3"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 35 is a success"));
      }
      oh_fs_close(fd);
    }
  }

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 0, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 36 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_CREAT | OH_FS_O_TRUNC | OH_FS_O_WRONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 36 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_fs_read(fd, &ch, 1) >= 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 36 is a fail 2"));
        ret++;
      }
      else
      {
        if (oh_fs_fstat(fd, &statbuf) != 0)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 36 is a fail 3"));
          ret++;
        }
        else if (statbuf.st_size != 0)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 36 is a fail 4"));
          ret++;
        }
        else
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 36 is a success"));
        }
      }
      oh_fs_close(fd);
    }
  }

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 0, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 37 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_CREAT | OH_FS_O_TRUNC | OH_FS_O_RDWR);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 37 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 37 is a fail 2"));
        ret++;
      }
      else if (statbuf.st_size != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 37 is a fail 3"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 37 is a success"));
      }
      oh_fs_close(fd);
    }
  }

  oh_fs_unlink(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00001-04.dat", opt->homedir,
    opt->testdir);

  oh_fs_unlink(path);

  fd = oh_fs_open(path, OH_FS_O_CREAT | OH_FS_O_RDONLY);
  if (fd < 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 38 is a fail 1"));
    ret++;
  }
  else
  {
    if (oh_fs_write(fd, &ch, 1) >= 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 38 is a fail 2"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 38 is a fail 3"));
        ret++;
      }
      else if (statbuf.st_size != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 38 is a fail 4"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 38 is a success"));
      }
    }
    oh_fs_close(fd);
  }

  oh_fs_unlink(path);

  fd = oh_fs_open(path, OH_FS_O_CREAT | OH_FS_O_RDONLY | OH_FS_O_TRUNC);
  if (fd < 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 39 is a fail 1"));
    ret++;
  }
  else
  {
    if (oh_fs_write(fd, &ch, 1) >= 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 39 is a fail 2"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 39 is a fail 3"));
        ret++;
      }
      else if (statbuf.st_size != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 39 is a fail 4"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 39 is a success"));
      }
    }
    oh_fs_close(fd);
  }

  oh_fs_unlink(path);

  fd = oh_fs_open(path, OH_FS_O_CREAT | OH_FS_O_RDONLY | OH_FS_O_EXCL);
  if (fd < 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 40 is a fail 1"));
    ret++;
  }
  else
  {
    if (oh_fs_write(fd, &ch, 1) >= 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 40 is a fail 2"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 40 is a fail 3"));
        ret++;
      }
      else if (statbuf.st_size != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 40 is a fail 4"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 40 is a success"));
      }
    }
    oh_fs_close(fd);
  }

  oh_fs_unlink(path);

  fd =
    oh_fs_open(path, OH_FS_O_CREAT | OH_FS_O_RDONLY | OH_FS_O_EXCL |
      OH_FS_O_TRUNC);
  if (fd < 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 41 is a fail 1"));
    ret++;
  }
  else
  {
    if (oh_fs_write(fd, &ch, 1) >= 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 41 is a fail 2"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 41 is a fail 3"));
        ret++;
      }
      else if (statbuf.st_size != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 41 is a fail 4"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "open/close test 41 is a success"));
      }
    }
    oh_fs_close(fd);
  }

  oh_fs_unlink(path);

  return ret;
}

static oh_int _oh_tk_testFS00002(oh_tk_opt_t *opt)
{
  oh_char path[OH_FS_MAXPATHLEN];
  oh_int fd;
  oh_int ret = 0;
  oh_uint8 *buff;
  oh_uint8 sbuff[8];

  buff = (oh_uint8 *)oh_c_malloc(opt->bufflen);
  if (buff == OH_NULL)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "read/write test oh_c_malloc fail"));
    return 1;
  }

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00002.dat", opt->homedir,
    opt->testdir);

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 500, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "read/write test 1 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_RDONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "read/write test 1 is a fail 1"));
      ret++;
    }
    else
    {
      oh_ulong pos;
      oh_int fail = 0;

      for (pos = 0;;)
      {
        oh_long fret;
        fret = oh_fs_read(fd, buff, opt->bufflen);
        if (fret == 0)
        {
          break;
        }
        if (fret < 0)
        {
          fail = 1;
          break;
        }
        if (_oh_tk_compareTestFile(buff, 500 + pos, fret) != 0)
        {
          fail = 1;
          break;
        }
        pos += fret;
      }
      oh_fs_close(fd);
      if (fail)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "read/write test 1 is fail 2"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "read/write test 1 is a success"));
      }
    }
  }

  fd = oh_fs_open(path, OH_FS_O_RDWR);
  if (fd < 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "read/write test 2 is a fail 0"));
    ret++;
  }
  else
  {
    oh_ulong pos;
    oh_long fret;
    oh_int fail = 0;

    for (pos = 0;;)
    {
      fret = oh_fs_read(fd, sbuff + pos, 3 - pos);
      if (fret == 0)
      {
        break;
      }
      if (fret < 0)
      {
        fail = 1;
        break;
      }
      pos += fret;
    }

    if (fail)
    {
      oh_fs_close(fd);
      oh_db_log((OL_DBG, OD_TESTKIT, "read/write test 2 is a fail 1"));
      ret++;
    }
    else
    {
      sbuff[0]++;
      sbuff[1]++;
      sbuff[2]++;

      for (pos = 0;;)
      {
        fret = oh_fs_write(fd, sbuff + pos, 3 - pos);
        if (fret == 0)
        {
          break;
        }
        if (fret < 0)
        {
          fail = 1;
          break;
        }
        pos += fret;
      }

      oh_fs_close(fd);

      if (fail)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "read/write test 2 is a fail 2"));
        ret++;
      }
      else
      {
        fd = oh_fs_open(path, OH_FS_O_RDONLY);
        if (fd < 0)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "read/write test 2 is a fail 3"));
          ret++;
        }
        else
        {
          for (pos = 0;;)
          {
            fret = oh_fs_read(fd, sbuff + pos, 6 - pos);
            if (fret == 0)
            {
              break;
            }
            if (fret < 0)
            {
              fail = 1;
              break;
            }
            pos += fret;
          }
          oh_fs_close(fd);
          if (fail)
          {
            oh_db_log((OL_DBG, OD_TESTKIT, "read/write test 2 is a fail 4"));
            ret++;
          }
          else
          {
            if (sbuff[0] + 1 == sbuff[3] && sbuff[1] + 1 == sbuff[4] &&
              sbuff[2] + 1 == sbuff[5])
            {
              oh_db_log((OL_DBG, OD_TESTKIT,
                "read/write test 2 is a success"));
            }
            else
            {
              oh_db_log((OL_DBG, OD_TESTKIT, "read/write test 2 is a fail 5"));
              ret++;
            }
          }
        }
      }
    }
  }

  oh_fs_unlink(path);
  oh_c_free(buff);

  return ret;
}

static oh_int _oh_tk_testFS00003(oh_tk_opt_t *opt)
{
  oh_char path[OH_FS_MAXPATHLEN];
  oh_int fd;
  oh_int ret = 0;
  oh_long sret;
  oh_fs_stat_t statbuf;

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00003.dat", opt->homedir,
    opt->testdir);

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 300, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 1 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_RDWR);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 1 is a fail 1"));
      ret++;
    }
    else
    {

      sret = oh_fs_lseek(fd, -2L, OH_FS_SEEK_SET);
      if (sret != -1)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 1 is a fail 2"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 1 is a success"));
      }

      sret = oh_fs_lseek(fd, 80000L, OH_FS_SEEK_SET);
      if (sret != 80000L)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 2 is a fail"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 2 is a success"));
      }

      sret = oh_fs_lseek(fd, -70000L, OH_FS_SEEK_CUR);
      if (sret != 10000L)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 3 is a fail"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 3 is a success"));
      }

      sret = oh_fs_lseek(fd, 75000L, OH_FS_SEEK_END);
      if (sret != (oh_long)(75000L + opt->filelen))
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 4 is a fail"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 4 is a success"));
      }

      sret = oh_fs_lseek(fd, -77000L - opt->filelen, OH_FS_SEEK_CUR);
      if (sret != -1)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 5 is a fail"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 5 is a success"));
      }

      sret = oh_fs_lseek(fd, 0, OH_FS_SEEK_CUR);
      if (sret != (oh_long)(75000L + opt->filelen))
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 6 is a fail"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 6 is a success"));
      }

      sret = oh_fs_write(fd, "a", 1);
      if (sret != 1)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 7 is a fail"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 7 is a success"));
      }

      if (oh_fs_fstat(fd, &statbuf) < 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 8 is a fail 0"));
        ret++;
      }
      else
      {
        if (statbuf.st_size != 1075001L)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 8 is a fail 1"));
          ret++;
        }
        else
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "lseek test 8 is a success"));
        }
      }
      oh_fs_close(fd);
    }
  }

  oh_fs_unlink(path);

  return ret;
}

static oh_int _oh_tk_testFS00004(oh_tk_opt_t *opt)
{
  oh_char path[OH_FS_MAXPATHLEN];
  oh_int fd;
  oh_int ret = 0;
  oh_fs_stat_t statbuf;

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00004.dat", opt->homedir,
    opt->testdir);

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 250, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 1 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_RDONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 1 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_fs_ftruncate(fd, 100) == 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 1 is a fail 2"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 1 is a success"));
      }
    }
    oh_fs_close(fd);

    fd = oh_fs_open(path, OH_FS_O_WRONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 2 is a fail 0"));
      ret++;
    }
    else
    {
      if (oh_fs_ftruncate(fd, (oh_long)(100 + opt->filelen)) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 2 is a fail 1"));
        ret++;
      }
      else
      {
        if (oh_fs_fstat(fd, &statbuf) != 0)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 2 is a fail 2"));
          ret++;
        }
        else
        {
          if (statbuf.st_size != 100 + opt->filelen)
          {
            oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 2 is a fail 3"));
            ret++;
          }
          else
          {
            oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 2 is a success"));
          }
        }
      }

      if (oh_fs_ftruncate(fd, 100) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 2 is a fail 1"));
        ret++;
      }
      else
      {
        if (oh_fs_fstat(fd, &statbuf) != 0)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 2 is a fail 2"));
          ret++;
        }
        else
        {
          if (statbuf.st_size != 100)
          {
            oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 2 is a fail 3"));
            ret++;
          }
          else
          {
            oh_db_log((OL_DBG, OD_TESTKIT, "ftruncate test 2 is a success"));
          }
        }
      }

    }
    oh_fs_close(fd);
  }

  oh_fs_unlink(path);

  return ret;
}

static oh_int _oh_tk_testFS00005(oh_tk_opt_t *opt)
{
  oh_char path[OH_FS_MAXPATHLEN];
  oh_int fd;
  oh_int ret = 0;
  oh_fs_stat_t statbuf;

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00005.dat", opt->homedir,
    opt->testdir);

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 250, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 1 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_RDONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 1 is a fail 0"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 1 is a fail 1"));
        ret++;
      }
      else
      {
        if (statbuf.st_size == opt->filelen &&
          OH_FS_S_ISREG(statbuf.st_mode) && !OH_FS_S_ISDIR(statbuf.st_mode))
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 1 is a success"));
        }
        else
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 1 is a fail 2"));
          ret++;
        }
      }
    }
    oh_fs_close(fd);
  }

  oh_fs_unlink(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00005.dir", opt->homedir,
    opt->testdir);

  oh_fs_unlink(path);

  if (oh_fs_mkdir(path) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 2 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_RDONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 2 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 2 is a fail 2"));
        ret++;
      }
      else
      {
        if (!OH_FS_S_ISREG(statbuf.st_mode) && OH_FS_S_ISDIR(statbuf.st_mode))
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 2 is a success"));
        }
        else
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 2 is a fail 3"));
          ret++;
        }
      }
    }
    oh_fs_close(fd);
  }
  oh_fs_rmdir(path);

  fd = oh_fs_open("/", OH_FS_O_RDONLY);
  if (fd < 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 3 is a fail 0"));
    ret++;
  }
  else
  {
    if (oh_fs_fstat(fd, &statbuf) != 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 3 is a fail 1"));
      ret++;
    }
    else
    {
      if (!OH_FS_S_ISREG(statbuf.st_mode) && OH_FS_S_ISDIR(statbuf.st_mode))
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 3 is a success"));
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "fstat test 3 is a fail 2"));
        ret++;
      }
    }
    oh_fs_close(fd);
  }

  return ret;
}

static oh_int _oh_tk_testFS00006(oh_tk_opt_t *opt)
{
  oh_char path[OH_FS_MAXPATHLEN];
  oh_char newpath[OH_FS_MAXPATHLEN];
  oh_int fd;
  oh_int ret = 0;

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00006.dat", opt->homedir,
    opt->testdir);
  oh_c_snprintf(newpath, OH_FS_MAXPATHLEN, "%s/%s/00006new.dat", opt->homedir,
    opt->testdir);

  oh_fs_unlink(path);
  oh_fs_unlink(newpath);

  if (_oh_tk_createTestFile(path, 150, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "rename test 1 is a fail 0"));
    ret++;
  }
  else
  {
    if (oh_fs_rename(path, newpath) != 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "rename test 1 is a fail 1"));
      ret++;
    }
    else
    {
      fd = oh_fs_open(path, OH_FS_O_RDONLY);
      if (fd >= 0)
      {
        oh_fs_close(fd);
        oh_db_log((OL_DBG, OD_TESTKIT, "rename test 1 is a fail 2"));
        ret++;
      }
      else
      {
        oh_fs_close(fd);
        fd = oh_fs_open(newpath, OH_FS_O_RDONLY);
        if (fd < 0)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "rename test 1 is a fail 3"));
          ret++;
        }
        else
        {
          oh_uint8 *buff;
          oh_ulong pos;
          oh_int fail = 0;

          buff = (oh_uint8 *)oh_c_malloc(opt->bufflen);
          if (buff == OH_NULL)
          {
            fail = 1;
          }
          else
          {
            for (pos = 0;;)
            {
              oh_long fret;
              fret = oh_fs_read(fd, buff, opt->bufflen);
              if (fret == 0)
              {
                break;
              }
              if (fret < 0)
              {
                fail = 1;
                break;
              }
              if (_oh_tk_compareTestFile(buff, 150 + pos, fret) != 0)
              {
                fail = 1;
                break;
              }
              pos += fret;
            }
            if (fail)
            {
              oh_db_log((OL_DBG, OD_TESTKIT, "rename test 1 is a fail 4"));
              ret++;
            }
            else
            {
              oh_db_log((OL_DBG, OD_TESTKIT, "rename test 1 is a success"));
            }
            oh_c_free(buff);
          }
          oh_fs_close(fd);
        }
      }
    }
  }

  oh_fs_unlink(newpath);

  return ret;
}

static oh_int _oh_tk_testFS00007(oh_tk_opt_t *opt)
{
  oh_char path[OH_FS_MAXPATHLEN];
  oh_int fd;
  oh_int ret = 0;

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00007.dat", opt->homedir,
    opt->testdir);

  oh_fs_unlink(path);

  if (_oh_tk_createTestFile(path, 0, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "unlink test 1 is a fail 0"));
    ret++;
  }
  else
  {
    if (oh_fs_unlink(path) != 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "unlink test 1 is a fail 1"));
      ret++;
    }
    else
    {
      fd = oh_fs_open(path, OH_FS_O_RDONLY);
      if (fd >= 0)
      {
        oh_fs_close(fd);
        oh_db_log((OL_DBG, OD_TESTKIT, "unlink test 1 is a fail 2"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "unlink test 1 is a success"));
      }
    }
  }

  if (oh_fs_unlink(path) == 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "unlink test 2 is a fail 0"));
    ret++;
  }
  else
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "unlink test 2 is a success"));
  }

  if (oh_fs_mkdir(path) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "unlink test 3 is a fail 0"));
    ret++;
  }
  else
  {
    if (oh_fs_unlink(path) == 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "unlink test 3 is a fail 1"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "unlink test 3 is a success"));
    }

    oh_fs_rmdir(path);
  }

  return ret;
}

static oh_int _oh_tk_testFS00008(oh_tk_opt_t *opt)
{
  oh_char path[OH_FS_MAXPATHLEN];
  oh_char curpath[OH_FS_MAXPATHLEN];
  oh_int ret = 0;
  oh_ulong i;
  oh_int fail = 0;

  for (i = 0; i < OH_FS_MAXPATHLEN; i++)
    path[i] = 'a';

  if (oh_fs_realpath("/c/tmp//./realpath////test/dir/../././.././dotdot", path,
    OH_FS_MAXPATHLEN) == OH_NULL)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 1 is a fail 0"));
    ret++;
  }
  else
  {
    fail = 0;
    if (oh_c_strcmp(path, "/c/tmp/realpath/dotdot") != 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 1 is a fail 1"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 1 is a success"));
    }

    for (i = oh_c_strlen(path) + 1; i < OH_FS_MAXPATHLEN; i++)
    {
      if (path[i] != 'a')
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 2 is a fail"));
        fail = 1;
        ret++;
        break;
      }
    }

    if (!fail)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 2 is a success"));
    }
  }

  for (i = 0; i < OH_FS_MAXPATHLEN; i++)
    path[i] = 'a';

  if (oh_fs_realpath("/c/tmp//./realpath////test/dir/../././.././dotdot", path,
    22) != OH_NULL)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 3 is a fail"));
    ret++;
  }
  else
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 3 is a success"));
  }

  if (oh_fs_realpath("/c/tmp//./realpath////test/dir/../././.././dotdot", path,
    23) == OH_NULL)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 4 is a fail 0"));
    ret++;
  }
  else
  {
    if (oh_c_strcmp(path, "/c/tmp/realpath/dotdot") != 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 4 is a fail 1"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 4 is a success"));
    }
  }

  if (oh_fs_getcwd(curpath, OH_FS_MAXPATHLEN) == OH_NULL)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 5 is a fail 0"));
    ret++;
  }
  else
  {
    if (oh_fs_realpath(".", path, OH_FS_MAXPATHLEN) == OH_NULL)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 5 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_c_strcmp(curpath, path) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 5 is a fail 2"));
        ret++;
      }
      else
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "realpath test 5 is a success"));
      }
    }
  }

  return ret;
}

static oh_int _oh_tk_testFS00009(oh_tk_opt_t *opt)
{
  oh_char path[OH_FS_MAXPATHLEN];
  oh_char path2[OH_FS_MAXPATHLEN];
  oh_int ret = 0;

  if (oh_fs_getcwd(path, OH_FS_MAXPATHLEN) == OH_NULL)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "getcwd/chdir test 1 is a fail 0"));
    ret++;
  }
  else
  {
    if (oh_fs_chdir("/") != 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "getcwd/chdir test 1 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_fs_getcwd(path2, OH_FS_MAXPATHLEN) == OH_NULL)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "getcwd/chdir test 1 is a fail 2"));
        ret++;
      }
      else
      {
        if (oh_c_strcmp(path2, path) == 0)
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "getcwd/chdir test 1 is a fail 3"));
          ret++;
        }
        else
        {
          if (oh_fs_chdir(path) != 0)
          {
            oh_db_log((OL_DBG, OD_TESTKIT, "getcwd/chdir test 1 is a fail 4"));
            ret++;
          }
          else
          {
            if (oh_fs_getcwd(path2, OH_FS_MAXPATHLEN) == OH_NULL)
            {
              oh_db_log((OL_DBG, OD_TESTKIT,
                "getcwd/chdir test 1 is a fail 5"));
              ret++;
            }
            else
            {
              if (oh_c_strcmp(path, path2) != 0)
              {
                oh_db_log((OL_DBG, OD_TESTKIT,
                  "getcwd/chdir test 1 is a fail 6"));
                ret++;
              }
              else
              {
                oh_db_log((OL_DBG, OD_TESTKIT,
                  "getcwd/chdir test 1 is a success"));
              }
            }
          }
        }
      }
    }
  }

  if (oh_fs_chdir("/.noexdir") == 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "getcwd/chdir test 2 is a fail 0"));
    ret++;
  }
  else
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "getcwd/chdir test 2 is a success"));
  }

  if (oh_fs_getcwd(path, 1) != OH_NULL)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "getcwd/chdir test 3 is a fail"));
    ret++;
  }
  else
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "getcwd/chdir test 3 is a success"));
  }

  return ret;
}

static oh_int _oh_tk_testFS00010(oh_tk_opt_t *opt)
{
  oh_char path[OH_FS_MAXPATHLEN];
  oh_int ret = 0;
  oh_int fd;
  oh_fs_stat_t statbuf;

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00010.dir", opt->homedir,
    opt->testdir);

  oh_fs_rmdir(path);

  if (oh_fs_mkdir(path) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 1 is a fail 0"));
    ret++;
  }
  else
  {
    fd = oh_fs_open(path, OH_FS_O_RDONLY); 
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 1 is a fail 1"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 1 is a fail 2"));
        ret++;
      }
      else
      {
        if (!OH_FS_S_ISREG(statbuf.st_mode) && OH_FS_S_ISDIR(statbuf.st_mode))
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 1 is a success"));
        }
        else
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 1 is a fail 3"));
          ret++;
        }
      }
    }
    oh_fs_close(fd);
    if (oh_fs_mkdir(path) == 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 2 is a fail"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 2 is a success"));
    }
  }

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00010.dir/testfile.dat",
    opt->homedir, opt->testdir);

  if (_oh_tk_createTestFile(path, 0, opt->filelen) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 3 is a fail 0"));
    ret++;
  }
  else
  {
    if (oh_fs_rmdir(path) == 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 3 is a fail 1"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 3 is a success"));
    }

    fd = oh_fs_open(path, OH_FS_O_RDONLY);
    if (fd < 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 4 is a fail 0"));
      ret++;
    }
    else
    {
      if (oh_fs_fstat(fd, &statbuf) != 0)
      {
        oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 4 is a fail 1"));
        ret++;
      }
      else
      {
        if (OH_FS_S_ISREG(statbuf.st_mode) && !OH_FS_S_ISDIR(statbuf.st_mode))
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 4 is a success"));
        }
        else
        {
          oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 4 is a fail 2"));
          ret++;
        }
      }
    }
    oh_fs_close(fd);
    if (oh_fs_mkdir(path) == 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 5 is a fail"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 5 is a success"));
    }
  }

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00010.dir", opt->homedir,
    opt->testdir);

  if (oh_fs_rmdir(path) == 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 6 is a fail"));
    ret++;
  }
  else
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 6 is a success"));
  }

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00010.dir/testfile.dat",
    opt->homedir, opt->testdir);

  oh_fs_unlink(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00010.dir", opt->homedir,
    opt->testdir);

  if (oh_fs_rmdir(path) != 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 7 is a fail"));
    ret++;
  }
  else
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 7 is a success"));
  }

  if (oh_fs_rmdir(path) == 0)
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 8 is a fail"));
    ret++;
  }
  else
  {
    oh_db_log((OL_DBG, OD_TESTKIT, "mkdir/rmdir test 8 is a success"));
  }

  return ret;
}

static oh_int _oh_tk_testFS00011(oh_tk_opt_t *opt)
{
  oh_char path[OH_FS_MAXPATHLEN];
  oh_int ret = 0;
  oh_fs_dir_t *dir;
  oh_fs_dirent_t *dirent;
  struct _oh_tk_readdir_t
  {
    const oh_char *name;
    oh_int check;
  } dirfiles[] =
    {
      {
        ".",
        0
      },
      {
        "..",
        0
      },
      {
        "dir1",
        0
      },
      {
        "dir2",
        0
      },
      {
        "dir3",
        0
      },
      {
        "file1",
        0
      },
      {
        "file2",
        0
      },
      {
        "file3",
        0
      }
    };
  const oh_int ndirfiles = sizeof(dirfiles) / sizeof(struct _oh_tk_readdir_t);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir", opt->homedir,
    opt->testdir);
  oh_fs_mkdir(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/dir1", opt->homedir,
    opt->testdir);
  oh_fs_mkdir(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/file1", opt->homedir,
    opt->testdir);
  _oh_tk_createTestFile(path, 0, opt->filelen);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/dir2", opt->homedir,
    opt->testdir);
  oh_fs_mkdir(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/file2", opt->homedir,
    opt->testdir);
  _oh_tk_createTestFile(path, 130, opt->filelen);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/dir3", opt->homedir,
    opt->testdir);
  oh_fs_mkdir(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/file3", opt->homedir,
    opt->testdir);
  _oh_tk_createTestFile(path, 210, opt->filelen);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir", opt->homedir,
    opt->testdir);

  dir = oh_fs_opendir(path);
  if (dir == OH_NULL)
  {
    oh_db_log((OL_DBG, OD_TESTKIT,
      "opendir/readdir/closedir test 1 is a fail 0"));
    ret++;
  }
  else
  {
    oh_int i;

    for (;;)
    {
      dirent = oh_fs_readdir(dir);
      if (dirent == OH_NULL)
        break;
      for (i = ndirfiles - 1; i >= 0; i--)
      {
        if (oh_c_strcmp(dirent->d_name, dirfiles[i].name) == 0)
        {
          dirfiles[i].check++;
          break;
        }
      }
    }

    for (i = ndirfiles - 1; i >= 0; i--)
    {
      if (dirfiles[i].check != 1)
      {
        oh_db_log((OL_DBG, OD_TESTKIT,
          "opendir/readdir/closedir test 1 is a fail 1"));
        ret++;
        break;
      }
    }

    if (oh_fs_closedir(dir) != 0)
    {
      oh_db_log((OL_DBG, OD_TESTKIT,
        "opendir/readdir/closedir test 2 is a fail"));
      ret++;
    }
    else
    {
      oh_db_log((OL_DBG, OD_TESTKIT,
        "opendir/readdir/closedir test 2 is a success"));
    }
  }

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/file1", opt->homedir,
    opt->testdir);
  _oh_tk_createTestFile(path, 0, opt->filelen);

  dir = oh_fs_opendir(path);
  if (dir != OH_NULL)
  {
    oh_fs_closedir(dir);
    oh_db_log((OL_DBG, OD_TESTKIT,
      "opendir/readdir/closedir test 3 is a fail"));
    ret++;
  }
  else
  {
    oh_db_log((OL_DBG, OD_TESTKIT,
      "opendir/readdir/closedir test 3 is a success"));
  }

  dir = oh_fs_opendir("/.noexdir");
  if (dir != OH_NULL)
  {
    oh_fs_closedir(dir);
    oh_db_log((OL_DBG, OD_TESTKIT,
      "opendir/readdir/closedir test 4 is a fail"));
    ret++;
  }
  else
  {
    oh_db_log((OL_DBG, OD_TESTKIT,
      "opendir/readdir/closedir test 4 is a success"));
  }

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/dir1", opt->homedir,
    opt->testdir);
  oh_fs_rmdir(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/file1", opt->homedir,
    opt->testdir);
  oh_fs_unlink(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/dir2", opt->homedir,
    opt->testdir);
  oh_fs_rmdir(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/file2", opt->homedir,
    opt->testdir);
  oh_fs_unlink(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/dir3", opt->homedir,
    opt->testdir);
  oh_fs_rmdir(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir/file3", opt->homedir,
    opt->testdir);
  oh_fs_unlink(path);

  oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s/00011.dir", opt->homedir,
    opt->testdir);
  oh_fs_rmdir(path);

  return ret;
}

oh_int oh_tk_testFS(oh_tk_opt_t *opt)
{
  oh_int ret = 0;

  oh_int i;

  for (i = 0; i < _oh_tk_numList; i++)
  {
    oh_int tcret;

    oh_db_log((OL_MSG, OD_TESTKIT, "%s Test ( %s )", _oh_tk_list[i].tcfuncname,
      _oh_tk_list[i].explain));
    tcret = (*_oh_tk_list[i].tcfunc)(opt);
    if (tcret == 0)
      oh_db_log((OL_MSG, OD_TESTKIT, "%s Success", _oh_tk_list[i].tcfuncname));
    else
      oh_db_log((OL_MSG, OD_TESTKIT, "%s Fail", _oh_tk_list[i].tcfuncname,
        tcret));
    ret += tcret;

    OH_ASSERT(ret >= 0); /* overflow check */
  }

  return ret;
}
